//
// Copyright 1997-2008 Sun Microsystems, Inc.  All Rights Reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
// CA 95054 USA or visit www.sun.com if you need additional information or
// have any questions.
//
//

// Machine Generated File.  Do Not Edit!


#include "incls/_precompiled.incl"
#include "incls/_ad_x86_32.cpp.incl"


//SourceForm

#define   RELOC_IMM32    Assembler::imm_operand
#define   RELOC_DISP32   Assembler::disp32_operand

#define __ _masm.

// How to find the high register of a Long pair, given the low register
#define   HIGH_FROM_LOW(x) ((x)+2)

// These masks are used to provide 128-bit aligned bitmasks to the XMM
// instructions, to allow sign-masking or sign-bit flipping.  They allow
// fast versions of NegF/NegD and AbsF/AbsD.

// Note: 'double' and 'long long' have 32-bits alignment on x86.
static jlong* double_quadword(jlong *adr, jlong lo, jlong hi) {
  // Use the expression (adr)&(~0xF) to provide 128-bits aligned address
  // of 128-bits operands for SSE instructions.
  jlong *operand = (jlong*)(((uintptr_t)adr)&((uintptr_t)(~0xF)));
  // Store the value to a 128-bits operand.
  operand[0] = lo;
  operand[1] = hi;
  return operand;
}

// Buffer for 128-bits masks used by SSE instructions.
static jlong fp_signmask_pool[(4+1)*2]; // 4*128bits(data) + 128bits(alignment)

// Static initialization during VM startup.
static jlong *float_signmask_pool  = double_quadword(&fp_signmask_pool[1*2], CONST64(0x7FFFFFFF7FFFFFFF), CONST64(0x7FFFFFFF7FFFFFFF));
static jlong *double_signmask_pool = double_quadword(&fp_signmask_pool[2*2], CONST64(0x7FFFFFFFFFFFFFFF), CONST64(0x7FFFFFFFFFFFFFFF));
static jlong *float_signflip_pool  = double_quadword(&fp_signmask_pool[3*2], CONST64(0x8000000080000000), CONST64(0x8000000080000000));
static jlong *double_signflip_pool = double_quadword(&fp_signmask_pool[4*2], CONST64(0x8000000000000000), CONST64(0x8000000000000000));

// !!!!! Special hack to get all type of calls to specify the byte offset
//       from the start of the call to the point where the return address
//       will point.
int MachCallStaticJavaNode::ret_addr_offset() {
  return 5 + (Compile::current()->in_24_bit_fp_mode() ? 6 : 0);  // 5 bytes from start of call to where return address points
}

int MachCallDynamicJavaNode::ret_addr_offset() {
  return 10 + (Compile::current()->in_24_bit_fp_mode() ? 6 : 0);  // 10 bytes from start of call to where return address points
}

static int sizeof_FFree_Float_Stack_All = -1;

int MachCallRuntimeNode::ret_addr_offset() {
  assert(sizeof_FFree_Float_Stack_All != -1, "must have been emitted already");
  return sizeof_FFree_Float_Stack_All + 5 + (Compile::current()->in_24_bit_fp_mode() ? 6 : 0);
}

// Indicate if the safepoint node needs the polling page as an input.
// Since x86 does have absolute addressing, it doesn't.
bool SafePointNode::needs_polling_address_input() {
  return false;
}

//
// Compute padding required for nodes which need alignment
//

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallStaticJavaDirectNode::compute_padding(int current_offset) const {
  if (Compile::current()->in_24_bit_fp_mode())
    current_offset += 6;    // skip fldcw in pre_call_FPU, if any
  current_offset += 1;      // skip call opcode byte
  return round_to(current_offset, alignment_required()) - current_offset;
}

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallDynamicJavaDirectNode::compute_padding(int current_offset) const {
  if (Compile::current()->in_24_bit_fp_mode())
    current_offset += 6;    // skip fldcw in pre_call_FPU, if any
  current_offset += 5;      // skip MOV instruction
  current_offset += 1;      // skip call opcode byte
  return round_to(current_offset, alignment_required()) - current_offset;
}

#ifndef PRODUCT
void MachBreakpointNode::format( PhaseRegAlloc *, outputStream* st ) const {
  st->print("INT3");
}
#endif

// EMIT_RM()
void emit_rm(CodeBuffer &cbuf, int f1, int f2, int f3) {
  unsigned char c = (unsigned char)((f1 << 6) | (f2 << 3) | f3);
  *(cbuf.code_end()) = c;
  cbuf.set_code_end(cbuf.code_end() + 1);
}

// EMIT_CC()
void emit_cc(CodeBuffer &cbuf, int f1, int f2) {
  unsigned char c = (unsigned char)( f1 | f2 );
  *(cbuf.code_end()) = c;
  cbuf.set_code_end(cbuf.code_end() + 1);
}

// EMIT_OPCODE()
void emit_opcode(CodeBuffer &cbuf, int code) {
  *(cbuf.code_end()) = (unsigned char)code;
  cbuf.set_code_end(cbuf.code_end() + 1);
}

// EMIT_OPCODE() w/ relocation information
void emit_opcode(CodeBuffer &cbuf, int code, relocInfo::relocType reloc, int offset = 0) {
  cbuf.relocate(cbuf.inst_mark() + offset, reloc);
  emit_opcode(cbuf, code);
}

// EMIT_D8()
void emit_d8(CodeBuffer &cbuf, int d8) {
  *(cbuf.code_end()) = (unsigned char)d8;
  cbuf.set_code_end(cbuf.code_end() + 1);
}

// EMIT_D16()
void emit_d16(CodeBuffer &cbuf, int d16) {
  *((short *)(cbuf.code_end())) = d16;
  cbuf.set_code_end(cbuf.code_end() + 2);
}

// EMIT_D32()
void emit_d32(CodeBuffer &cbuf, int d32) {
  *((int *)(cbuf.code_end())) = d32;
  cbuf.set_code_end(cbuf.code_end() + 4);
}

// emit 32 bit value and construct relocation entry from relocInfo::relocType
void emit_d32_reloc(CodeBuffer &cbuf, int d32, relocInfo::relocType reloc,
        int format) {
  cbuf.relocate(cbuf.inst_mark(), reloc, format);

  *((int *)(cbuf.code_end())) = d32;
  cbuf.set_code_end(cbuf.code_end() + 4);
}

// emit 32 bit value and construct relocation entry from RelocationHolder
void emit_d32_reloc(CodeBuffer &cbuf, int d32, RelocationHolder const& rspec,
        int format) {
#ifdef ASSERT
  if (rspec.reloc()->type() == relocInfo::oop_type && d32 != 0 && d32 != (int)Universe::non_oop_word()) {
    assert(oop(d32)->is_oop() && oop(d32)->is_perm(), "cannot embed non-perm oops in code");
  }
#endif
  cbuf.relocate(cbuf.inst_mark(), rspec, format);

  *((int *)(cbuf.code_end())) = d32;
  cbuf.set_code_end(cbuf.code_end() + 4);
}

// Access stack slot for load or store
void store_to_stackslot(CodeBuffer &cbuf, int opcode, int rm_field, int disp) {
  emit_opcode( cbuf, opcode );               // (e.g., FILD   [ESP+src])
  if( -128 <= disp && disp <= 127 ) {
    emit_rm( cbuf, 0x01, rm_field, ESP_enc );  // R/M byte
    emit_rm( cbuf, 0x00, ESP_enc, ESP_enc);    // SIB byte
    emit_d8 (cbuf, disp);     // Displacement  // R/M byte
  } else {
    emit_rm( cbuf, 0x02, rm_field, ESP_enc );  // R/M byte
    emit_rm( cbuf, 0x00, ESP_enc, ESP_enc);    // SIB byte
    emit_d32(cbuf, disp);     // Displacement  // R/M byte
  }
}

   // eRegI ereg, memory mem) %{    // emit_reg_mem
void encode_RegMem( CodeBuffer &cbuf, int reg_encoding, int base, int index, int scale, int displace, bool displace_is_oop ) {
  // There is no index & no scale, use form without SIB byte
  if ((index == 0x4) &&
      (scale == 0) && (base != ESP_enc)) {
    // If no displacement, mode is 0x0; unless base is [EBP]
    if ( (displace == 0) && (base != EBP_enc) ) {
      emit_rm(cbuf, 0x0, reg_encoding, base);
    }
    else {                    // If 8-bit displacement, mode 0x1
      if ((displace >= -128) && (displace <= 127)
          && !(displace_is_oop) ) {
        emit_rm(cbuf, 0x1, reg_encoding, base);
        emit_d8(cbuf, displace);
      }
      else {                  // If 32-bit displacement
        if (base == -1) { // Special flag for absolute address
          emit_rm(cbuf, 0x0, reg_encoding, 0x5);
          // (manual lies; no SIB needed here)
          if ( displace_is_oop ) {
            emit_d32_reloc(cbuf, displace, relocInfo::oop_type, 1);
          } else {
            emit_d32      (cbuf, displace);
          }
        }
        else {                // Normal base + offset
          emit_rm(cbuf, 0x2, reg_encoding, base);
          if ( displace_is_oop ) {
            emit_d32_reloc(cbuf, displace, relocInfo::oop_type, 1);
          } else {
            emit_d32      (cbuf, displace);
          }
        }
      }
    }
  }
  else {                      // Else, encode with the SIB byte
    // If no displacement, mode is 0x0; unless base is [EBP]
    if (displace == 0 && (base != EBP_enc)) {  // If no displacement
      emit_rm(cbuf, 0x0, reg_encoding, 0x4);
      emit_rm(cbuf, scale, index, base);
    }
    else {                    // If 8-bit displacement, mode 0x1
      if ((displace >= -128) && (displace <= 127)
          && !(displace_is_oop) ) {
        emit_rm(cbuf, 0x1, reg_encoding, 0x4);
        emit_rm(cbuf, scale, index, base);
        emit_d8(cbuf, displace);
      }
      else {                  // If 32-bit displacement
        if (base == 0x04 ) {
          emit_rm(cbuf, 0x2, reg_encoding, 0x4);
          emit_rm(cbuf, scale, index, 0x04);
        } else {
          emit_rm(cbuf, 0x2, reg_encoding, 0x4);
          emit_rm(cbuf, scale, index, base);
        }
        if ( displace_is_oop ) {
          emit_d32_reloc(cbuf, displace, relocInfo::oop_type, 1);
        } else {
          emit_d32      (cbuf, displace);
        }
      }
    }
  }
}


void encode_Copy( CodeBuffer &cbuf, int dst_encoding, int src_encoding ) {
  if( dst_encoding == src_encoding ) {
    // reg-reg copy, use an empty encoding
  } else {
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, dst_encoding, src_encoding );
  }
}

void encode_CopyXD( CodeBuffer &cbuf, int dst_encoding, int src_encoding ) {
  if( dst_encoding == src_encoding ) {
    // reg-reg copy, use an empty encoding
  } else {
    MacroAssembler _masm(&cbuf);

    __ movdqa(as_XMMRegister(dst_encoding), as_XMMRegister(src_encoding));
  }
}


//=============================================================================
#ifndef PRODUCT
void MachPrologNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  Compile* C = ra_->C;
  if( C->in_24_bit_fp_mode() ) {
    st->print("FLDCW  24 bit fpu control word");
    st->print_cr(""); st->print("\t");
  }

  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove two words for return addr and rbp,
  framesize -= 2*wordSize;

  // Calls to C2R adapters often do not accept exceptional returns.
  // We require that their callers must bang for them.  But be careful, because
  // some VM calls (such as call site linkage) can use several kilobytes of
  // stack.  But the stack safety zone should account for that.
  // See bugs 4446381, 4468289, 4497237.
  if (C->need_stack_bang(framesize)) {
    st->print_cr("# stack bang"); st->print("\t");
  }
  st->print_cr("PUSHL  EBP"); st->print("\t");

  if( VerifyStackAtCalls ) { // Majik cookie to verify stack depth
    st->print("PUSH   0xBADB100D\t# Majik cookie for stack depth check");
    st->print_cr(""); st->print("\t");
    framesize -= wordSize;
  }

  if ((C->in_24_bit_fp_mode() || VerifyStackAtCalls ) && framesize < 128 ) {
    if (framesize) {
      st->print("SUB    ESP,%d\t# Create frame",framesize);
    }
  } else {
    st->print("SUB    ESP,%d\t# Create frame",framesize);
  }
}
#endif


void MachPrologNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile* C = ra_->C;

  if (UseSSE >= 2 && VerifyFPU) {
    MacroAssembler masm(&cbuf);
    masm.verify_FPU(0, "FPU stack must be clean on entry");
  }

  // WARNING: Initial instruction MUST be 5 bytes or longer so that
  // NativeJump::patch_verified_entry will be able to patch out the entry
  // code safely. The fldcw is ok at 6 bytes, the push to verify stack
  // depth is ok at 5 bytes, the frame allocation can be either 3 or
  // 6 bytes. So if we don't do the fldcw or the push then we must
  // use the 6 byte frame allocation even if we have no frame. :-(
  // If method sets FPU control word do it now
  if( C->in_24_bit_fp_mode() ) {
    MacroAssembler masm(&cbuf);
    masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
  }

  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove two words for return addr and rbp,
  framesize -= 2*wordSize;

  // Calls to C2R adapters often do not accept exceptional returns.
  // We require that their callers must bang for them.  But be careful, because
  // some VM calls (such as call site linkage) can use several kilobytes of
  // stack.  But the stack safety zone should account for that.
  // See bugs 4446381, 4468289, 4497237.
  if (C->need_stack_bang(framesize)) {
    MacroAssembler masm(&cbuf);
    masm.generate_stack_overflow_check(framesize);
  }

  // We always push rbp, so that on return to interpreter rbp, will be
  // restored correctly and we can correct the stack.
  emit_opcode(cbuf, 0x50 | EBP_enc);

  if( VerifyStackAtCalls ) { // Majik cookie to verify stack depth
    emit_opcode(cbuf, 0x68); // push 0xbadb100d
    emit_d32(cbuf, 0xbadb100d);
    framesize -= wordSize;
  }

  if ((C->in_24_bit_fp_mode() || VerifyStackAtCalls ) && framesize < 128 ) {
    if (framesize) {
      emit_opcode(cbuf, 0x83);   // sub  SP,#framesize
      emit_rm(cbuf, 0x3, 0x05, ESP_enc);
      emit_d8(cbuf, framesize);
    }
  } else {
    emit_opcode(cbuf, 0x81);   // sub  SP,#framesize
    emit_rm(cbuf, 0x3, 0x05, ESP_enc);
    emit_d32(cbuf, framesize);
  }
  C->set_frame_complete(cbuf.code_end() - cbuf.code_begin());

#ifdef ASSERT
  if (VerifyStackAtCalls) {
    Label L;
    MacroAssembler masm(&cbuf);
    masm.push(rax);
    masm.mov(rax, rsp);
    masm.andptr(rax, StackAlignmentInBytes-1);
    masm.cmpptr(rax, StackAlignmentInBytes-wordSize);
    masm.pop(rax);
    masm.jcc(Assembler::equal, L);
    masm.stop("Stack is not properly aligned!");
    masm.bind(L);
  }
#endif

}

uint MachPrologNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_); // too many variables; just compute it the hard way
}

int MachPrologNode::reloc() const {
  return 0; // a large enough number
}

//=============================================================================
#ifndef PRODUCT
void MachEpilogNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  Compile *C = ra_->C;
  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove two words for return addr and rbp,
  framesize -= 2*wordSize;

  if( C->in_24_bit_fp_mode() ) {
    st->print("FLDCW  standard control word");
    st->cr(); st->print("\t");
  }
  if( framesize ) {
    st->print("ADD    ESP,%d\t# Destroy frame",framesize);
    st->cr(); st->print("\t");
  }
  st->print_cr("POPL   EBP"); st->print("\t");
  if( do_polling() && C->is_method_compilation() ) {
    st->print("TEST   PollPage,EAX\t! Poll Safepoint");
    st->cr(); st->print("\t");
  }
}
#endif

void MachEpilogNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile *C = ra_->C;

  // If method set FPU control word, restore to standard control word
  if( C->in_24_bit_fp_mode() ) {
    MacroAssembler masm(&cbuf);
    masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
  }

  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove two words for return addr and rbp,
  framesize -= 2*wordSize;

  // Note that VerifyStackAtCalls' Majik cookie does not change the frame size popped here

  if( framesize >= 128 ) {
    emit_opcode(cbuf, 0x81); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d32(cbuf, framesize);
  }
  else if( framesize ) {
    emit_opcode(cbuf, 0x83); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, framesize);
  }

  emit_opcode(cbuf, 0x58 | EBP_enc);

  if( do_polling() && C->is_method_compilation() ) {
    cbuf.relocate(cbuf.code_end(), relocInfo::poll_return_type, 0);
    emit_opcode(cbuf,0x85);
    emit_rm(cbuf, 0x0, EAX_enc, 0x5); // EAX
    emit_d32(cbuf, (intptr_t)os::get_polling_page());
  }
}

uint MachEpilogNode::size(PhaseRegAlloc *ra_) const {
  Compile *C = ra_->C;
  // If method set FPU control word, restore to standard control word
  int size = C->in_24_bit_fp_mode() ? 6 : 0;
  if( do_polling() && C->is_method_compilation() ) size += 6;

  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove two words for return addr and rbp,
  framesize -= 2*wordSize;

  size++; // popl rbp,

  if( framesize >= 128 ) {
    size += 6;
  } else {
    size += framesize ? 3 : 0;
  }
  return size;
}

int MachEpilogNode::reloc() const {
  return 0; // a large enough number
}

const Pipeline * MachEpilogNode::pipeline() const {
  return MachNode::pipeline_class();
}

int MachEpilogNode::safepoint_offset() const { return 0; }

//=============================================================================

enum RC { rc_bad, rc_int, rc_float, rc_xmm, rc_stack };
static enum RC rc_class( OptoReg::Name reg ) {

  if( !OptoReg::is_valid(reg)  ) return rc_bad;
  if (OptoReg::is_stack(reg)) return rc_stack;

  VMReg r = OptoReg::as_VMReg(reg);
  if (r->is_Register()) return rc_int;
  if (r->is_FloatRegister()) {
    assert(UseSSE < 2, "shouldn't be used in SSE2+ mode");
    return rc_float;
  }
  assert(r->is_XMMRegister(), "must be");
  return rc_xmm;
}

static int impl_helper( CodeBuffer *cbuf, bool do_size, bool is_load, int offset, int reg,
                        int opcode, const char *op_str, int size, outputStream* st ) {
  if( cbuf ) {
    emit_opcode  (*cbuf, opcode );
    encode_RegMem(*cbuf, Matcher::_regEncode[reg], ESP_enc, 0x4, 0, offset, false);
#ifndef PRODUCT
  } else if( !do_size ) {
    if( size != 0 ) st->print("\n\t");
    if( opcode == 0x8B || opcode == 0x89 ) { // MOV
      if( is_load ) st->print("%s   %s,[ESP + #%d]",op_str,Matcher::regName[reg],offset);
      else          st->print("%s   [ESP + #%d],%s",op_str,offset,Matcher::regName[reg]);
    } else { // FLD, FST, PUSH, POP
      st->print("%s [ESP + #%d]",op_str,offset);
    }
#endif
  }
  int offset_size = (offset == 0) ? 0 : ((offset <= 127) ? 1 : 4);
  return size+3+offset_size;
}

// Helper for XMM registers.  Extra opcode bits, limited syntax.
static int impl_x_helper( CodeBuffer *cbuf, bool do_size, bool is_load,
                         int offset, int reg_lo, int reg_hi, int size, outputStream* st ) {
  if( cbuf ) {
    if( reg_lo+1 == reg_hi ) { // double move?
      if( is_load && !UseXmmLoadAndClearUpper )
        emit_opcode(*cbuf, 0x66 ); // use 'movlpd' for load
      else
        emit_opcode(*cbuf, 0xF2 ); // use 'movsd' otherwise
    } else {
      emit_opcode(*cbuf, 0xF3 );
    }
    emit_opcode(*cbuf, 0x0F );
    if( reg_lo+1 == reg_hi && is_load && !UseXmmLoadAndClearUpper )
      emit_opcode(*cbuf, 0x12 );   // use 'movlpd' for load
    else
      emit_opcode(*cbuf, is_load ? 0x10 : 0x11 );
    encode_RegMem(*cbuf, Matcher::_regEncode[reg_lo], ESP_enc, 0x4, 0, offset, false);
#ifndef PRODUCT
  } else if( !do_size ) {
    if( size != 0 ) st->print("\n\t");
    if( reg_lo+1 == reg_hi ) { // double move?
      if( is_load ) st->print("%s %s,[ESP + #%d]",
                               UseXmmLoadAndClearUpper ? "MOVSD " : "MOVLPD",
                               Matcher::regName[reg_lo], offset);
      else          st->print("MOVSD  [ESP + #%d],%s",
                               offset, Matcher::regName[reg_lo]);
    } else {
      if( is_load ) st->print("MOVSS  %s,[ESP + #%d]",
                               Matcher::regName[reg_lo], offset);
      else          st->print("MOVSS  [ESP + #%d],%s",
                               offset, Matcher::regName[reg_lo]);
    }
#endif
  }
  int offset_size = (offset == 0) ? 0 : ((offset <= 127) ? 1 : 4);
  return size+5+offset_size;
}


static int impl_movx_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                            int src_hi, int dst_hi, int size, outputStream* st ) {
  if( UseXmmRegToRegMoveAll ) {//Use movaps,movapd to move between xmm registers
    if( cbuf ) {
      if( (src_lo+1 == src_hi && dst_lo+1 == dst_hi) ) {
        emit_opcode(*cbuf, 0x66 );
      }
      emit_opcode(*cbuf, 0x0F );
      emit_opcode(*cbuf, 0x28 );
      emit_rm    (*cbuf, 0x3, Matcher::_regEncode[dst_lo], Matcher::_regEncode[src_lo] );
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      if( src_lo+1 == src_hi && dst_lo+1 == dst_hi ) { // double move?
        st->print("MOVAPD %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      } else {
        st->print("MOVAPS %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      }
#endif
    }
    return size + ((src_lo+1 == src_hi && dst_lo+1 == dst_hi) ? 4 : 3);
  } else {
    if( cbuf ) {
      emit_opcode(*cbuf, (src_lo+1 == src_hi && dst_lo+1 == dst_hi) ? 0xF2 : 0xF3 );
      emit_opcode(*cbuf, 0x0F );
      emit_opcode(*cbuf, 0x10 );
      emit_rm    (*cbuf, 0x3, Matcher::_regEncode[dst_lo], Matcher::_regEncode[src_lo] );
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      if( src_lo+1 == src_hi && dst_lo+1 == dst_hi ) { // double move?
        st->print("MOVSD  %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      } else {
        st->print("MOVSS  %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      }
#endif
    }
    return size+4;
  }
}

static int impl_mov_helper( CodeBuffer *cbuf, bool do_size, int src, int dst, int size, outputStream* st ) {
  if( cbuf ) {
    emit_opcode(*cbuf, 0x8B );
    emit_rm    (*cbuf, 0x3, Matcher::_regEncode[dst], Matcher::_regEncode[src] );
#ifndef PRODUCT
  } else if( !do_size ) {
    if( size != 0 ) st->print("\n\t");
    st->print("MOV    %s,%s",Matcher::regName[dst],Matcher::regName[src]);
#endif
  }
  return size+2;
}

static int impl_fp_store_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int src_hi, int dst_lo, int dst_hi,
                                 int offset, int size, outputStream* st ) {
  if( src_lo != FPR1L_num ) {      // Move value to top of FP stack, if not already there
    if( cbuf ) {
      emit_opcode( *cbuf, 0xD9 );  // FLD (i.e., push it)
      emit_d8( *cbuf, 0xC0-1+Matcher::_regEncode[src_lo] );
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("FLD    %s",Matcher::regName[src_lo]);
#endif
    }
    size += 2;
  }

  int st_op = (src_lo != FPR1L_num) ? EBX_num /*store & pop*/ : EDX_num /*store no pop*/;
  const char *op_str;
  int op;
  if( src_lo+1 == src_hi && dst_lo+1 == dst_hi ) { // double store?
    op_str = (src_lo != FPR1L_num) ? "FSTP_D" : "FST_D ";
    op = 0xDD;
  } else {                   // 32-bit store
    op_str = (src_lo != FPR1L_num) ? "FSTP_S" : "FST_S ";
    op = 0xD9;
    assert( !OptoReg::is_valid(src_hi) && !OptoReg::is_valid(dst_hi), "no non-adjacent float-stores" );
  }

  return impl_helper(cbuf,do_size,false,offset,st_op,op,op_str,size, st);
}

uint MachSpillCopyNode::implementation( CodeBuffer *cbuf, PhaseRegAlloc *ra_, bool do_size, outputStream* st ) const {
  // Get registers to move
  OptoReg::Name src_second = ra_->get_reg_second(in(1));
  OptoReg::Name src_first = ra_->get_reg_first(in(1));
  OptoReg::Name dst_second = ra_->get_reg_second(this );
  OptoReg::Name dst_first = ra_->get_reg_first(this );

  enum RC src_second_rc = rc_class(src_second);
  enum RC src_first_rc = rc_class(src_first);
  enum RC dst_second_rc = rc_class(dst_second);
  enum RC dst_first_rc = rc_class(dst_first);

  assert( OptoReg::is_valid(src_first) && OptoReg::is_valid(dst_first), "must move at least 1 register" );

  // Generate spill code!
  int size = 0;

  if( src_first == dst_first && src_second == dst_second )
    return size;            // Self copy, no move

  // --------------------------------------
  // Check for mem-mem move.  push/pop to move.
  if( src_first_rc == rc_stack && dst_first_rc == rc_stack ) {
    if( src_second == dst_first ) { // overlapping stack copy ranges
      assert( src_second_rc == rc_stack && dst_second_rc == rc_stack, "we only expect a stk-stk copy here" );
      size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_second),ESI_num,0xFF,"PUSH  ",size, st);
      size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_second),EAX_num,0x8F,"POP   ",size, st);
      src_second_rc = dst_second_rc = rc_bad;  // flag as already moved the second bits
    }
    // move low bits
    size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_first),ESI_num,0xFF,"PUSH  ",size, st);
    size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_first),EAX_num,0x8F,"POP   ",size, st);
    if( src_second_rc == rc_stack && dst_second_rc == rc_stack ) { // mov second bits
      size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_second),ESI_num,0xFF,"PUSH  ",size, st);
      size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_second),EAX_num,0x8F,"POP   ",size, st);
    }
    return size;
  }

  // --------------------------------------
  // Check for integer reg-reg copy
  if( src_first_rc == rc_int && dst_first_rc == rc_int )
    size = impl_mov_helper(cbuf,do_size,src_first,dst_first,size, st);

  // Check for integer store
  if( src_first_rc == rc_int && dst_first_rc == rc_stack )
    size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_first),src_first,0x89,"MOV ",size, st);

  // Check for integer load
  if( dst_first_rc == rc_int && src_first_rc == rc_stack )
    size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_first),dst_first,0x8B,"MOV ",size, st);

  // --------------------------------------
  // Check for float reg-reg copy
  if( src_first_rc == rc_float && dst_first_rc == rc_float ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad) ||
            (src_first+1 == src_second && dst_first+1 == dst_second), "no non-adjacent float-moves" );
    if( cbuf ) {

      // Note the mucking with the register encode to compensate for the 0/1
      // indexing issue mentioned in a comment in the reg_def sections
      // for FPR registers many lines above here.

      if( src_first != FPR1L_num ) {
        emit_opcode  (*cbuf, 0xD9 );           // FLD    ST(i)
        emit_d8      (*cbuf, 0xC0+Matcher::_regEncode[src_first]-1 );
        emit_opcode  (*cbuf, 0xDD );           // FSTP   ST(i)
        emit_d8      (*cbuf, 0xD8+Matcher::_regEncode[dst_first] );
     } else {
        emit_opcode  (*cbuf, 0xDD );           // FST    ST(i)
        emit_d8      (*cbuf, 0xD0+Matcher::_regEncode[dst_first]-1 );
     }
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      if( src_first != FPR1L_num ) st->print("FLD    %s\n\tFSTP   %s",Matcher::regName[src_first],Matcher::regName[dst_first]);
      else                      st->print(             "FST    %s",                            Matcher::regName[dst_first]);
#endif
    }
    return size + ((src_first != FPR1L_num) ? 2+2 : 2);
  }

  // Check for float store
  if( src_first_rc == rc_float && dst_first_rc == rc_stack ) {
    return impl_fp_store_helper(cbuf,do_size,src_first,src_second,dst_first,dst_second,ra_->reg2offset(dst_first),size, st);
  }

  // Check for float load
  if( dst_first_rc == rc_float && src_first_rc == rc_stack ) {
    int offset = ra_->reg2offset(src_first);
    const char *op_str;
    int op;
    if( src_first+1 == src_second && dst_first+1 == dst_second ) { // double load?
      op_str = "FLD_D";
      op = 0xDD;
    } else {                   // 32-bit load
      op_str = "FLD_S";
      op = 0xD9;
      assert( src_second_rc == rc_bad && dst_second_rc == rc_bad, "no non-adjacent float-loads" );
    }
    if( cbuf ) {
      emit_opcode  (*cbuf, op );
      encode_RegMem(*cbuf, 0x0, ESP_enc, 0x4, 0, offset, false);
      emit_opcode  (*cbuf, 0xDD );           // FSTP   ST(i)
      emit_d8      (*cbuf, 0xD8+Matcher::_regEncode[dst_first] );
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("%s  ST,[ESP + #%d]\n\tFSTP   %s",op_str, offset,Matcher::regName[dst_first]);
#endif
    }
    int offset_size = (offset == 0) ? 0 : ((offset <= 127) ? 1 : 4);
    return size + 3+offset_size+2;
  }

  // Check for xmm reg-reg copy
  if( src_first_rc == rc_xmm && dst_first_rc == rc_xmm ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad) ||
            (src_first+1 == src_second && dst_first+1 == dst_second),
            "no non-adjacent float-moves" );
    return impl_movx_helper(cbuf,do_size,src_first,dst_first,src_second, dst_second, size, st);
  }

  // Check for xmm store
  if( src_first_rc == rc_xmm && dst_first_rc == rc_stack ) {
    return impl_x_helper(cbuf,do_size,false,ra_->reg2offset(dst_first),src_first, src_second, size, st);
  }

  // Check for float xmm load
  if( dst_first_rc == rc_xmm && src_first_rc == rc_stack ) {
    return impl_x_helper(cbuf,do_size,true ,ra_->reg2offset(src_first),dst_first, dst_second, size, st);
  }

  // Copy from float reg to xmm reg
  if( dst_first_rc == rc_xmm && src_first_rc == rc_float ) {
    // copy to the top of stack from floating point reg
    // and use LEA to preserve flags
    if( cbuf ) {
      emit_opcode(*cbuf,0x8D);  // LEA  ESP,[ESP-8]
      emit_rm(*cbuf, 0x1, ESP_enc, 0x04);
      emit_rm(*cbuf, 0x0, 0x04, ESP_enc);
      emit_d8(*cbuf,0xF8);
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("LEA    ESP,[ESP-8]");
#endif
    }
    size += 4;

    size = impl_fp_store_helper(cbuf,do_size,src_first,src_second,dst_first,dst_second,0,size, st);

    // Copy from the temp memory to the xmm reg.
    size = impl_x_helper(cbuf,do_size,true ,0,dst_first, dst_second, size, st);

    if( cbuf ) {
      emit_opcode(*cbuf,0x8D);  // LEA  ESP,[ESP+8]
      emit_rm(*cbuf, 0x1, ESP_enc, 0x04);
      emit_rm(*cbuf, 0x0, 0x04, ESP_enc);
      emit_d8(*cbuf,0x08);
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("LEA    ESP,[ESP+8]");
#endif
    }
    size += 4;
    return size;
  }

  assert( size > 0, "missed a case" );

  // --------------------------------------------------------------------
  // Check for second bits still needing moving.
  if( src_second == dst_second )
    return size;               // Self copy; no move
  assert( src_second_rc != rc_bad && dst_second_rc != rc_bad, "src_second & dst_second cannot be Bad" );

  // Check for second word int-int move
  if( src_second_rc == rc_int && dst_second_rc == rc_int )
    return impl_mov_helper(cbuf,do_size,src_second,dst_second,size, st);

  // Check for second word integer store
  if( src_second_rc == rc_int && dst_second_rc == rc_stack )
    return impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_second),src_second,0x89,"MOV ",size, st);

  // Check for second word integer load
  if( dst_second_rc == rc_int && src_second_rc == rc_stack )
    return impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_second),dst_second,0x8B,"MOV ",size, st);


  Unimplemented();
}

#ifndef PRODUCT
void MachSpillCopyNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  implementation( NULL, ra_, false, st );
}
#endif

void MachSpillCopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  implementation( &cbuf, ra_, false, NULL );
}

uint MachSpillCopyNode::size(PhaseRegAlloc *ra_) const {
  return implementation( NULL, ra_, true, NULL );
}

//=============================================================================
#ifndef PRODUCT
void MachNopNode::format( PhaseRegAlloc *, outputStream* st ) const {
  st->print("NOP \t# %d bytes pad for loops and calls", _count);
}
#endif

void MachNopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc * ) const {
  MacroAssembler _masm(&cbuf);
  __ nop(_count);
}

uint MachNopNode::size(PhaseRegAlloc *) const {
  return _count;
}


//=============================================================================
#ifndef PRODUCT
void BoxLockNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_reg_first(this);
  st->print("LEA    %s,[ESP + #%d]",Matcher::regName[reg],offset);
}
#endif

void BoxLockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_encode(this);
  if( offset >= 128 ) {
    emit_opcode(cbuf, 0x8D);      // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x2, reg, 0x04);
    emit_rm(cbuf, 0x0, 0x04, ESP_enc);
    emit_d32(cbuf, offset);
  }
  else {
    emit_opcode(cbuf, 0x8D);      // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x1, reg, 0x04);
    emit_rm(cbuf, 0x0, 0x04, ESP_enc);
    emit_d8(cbuf, offset);
  }
}

uint BoxLockNode::size(PhaseRegAlloc *ra_) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  if( offset >= 128 ) {
    return 7;
  }
  else {
    return 4;
  }
}

//=============================================================================

// emit call stub, compiled java to interpreter
void emit_java_to_interp(CodeBuffer &cbuf ) {
  // Stub is fixed up when the corresponding call is converted from calling
  // compiled code to calling interpreted code.
  // mov rbx,0
  // jmp -1

  address mark = cbuf.inst_mark();  // get mark within main instrs section

  // Note that the code buffer's inst_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a stub.
  MacroAssembler _masm(&cbuf);

  address base =
  __ start_a_stub(Compile::MAX_stubs_size);
  if (base == NULL)  return;  // CodeBuffer::expand failed
  // static stub relocation stores the instruction address of the call
  __ relocate(static_stub_Relocation::spec(mark), RELOC_IMM32);
  // static stub relocation also tags the methodOop in the code-stream.
  __ movoop(rbx, (jobject)NULL);  // method is zapped till fixup time
  // This is recognized as unresolved by relocs/nativeInst/ic code
  __ jump(RuntimeAddress(__ pc()));

  __ end_a_stub();
  // Update current stubs pointer and restore code_end.
}
// size of call stub, compiled java to interpretor
uint size_java_to_interp() {
  return 10;  // movl; jmp
}
// relocation entries for call stub, compiled java to interpretor
uint reloc_java_to_interp() {
  return 4;  // 3 in emit_java_to_interp + 1 in Java_Static_Call
}

//=============================================================================
#ifndef PRODUCT
void MachUEPNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  st->print_cr(  "CMP    EAX,[ECX+4]\t# Inline cache check");
  st->print_cr("\tJNE    SharedRuntime::handle_ic_miss_stub");
  st->print_cr("\tNOP");
  st->print_cr("\tNOP");
  if( !OptoBreakpoint )
    st->print_cr("\tNOP");
}
#endif

void MachUEPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  MacroAssembler masm(&cbuf);
#ifdef ASSERT
  uint code_size = cbuf.code_size();
#endif
  masm.cmpptr(rax, Address(rcx, oopDesc::klass_offset_in_bytes()));
  masm.jump_cc(Assembler::notEqual,
               RuntimeAddress(SharedRuntime::get_ic_miss_stub()));
  /* WARNING these NOPs are critical so that verified entry point is properly
     aligned for patching by NativeJump::patch_verified_entry() */
  int nops_cnt = 2;
  if( !OptoBreakpoint ) // Leave space for int3
     nops_cnt += 1;
  masm.nop(nops_cnt);

  assert(cbuf.code_size() - code_size == size(ra_), "checking code size of inline cache node");
}

uint MachUEPNode::size(PhaseRegAlloc *ra_) const {
  return OptoBreakpoint ? 11 : 12;
}


//=============================================================================
uint size_exception_handler() {
  // NativeCall instruction size is the same as NativeJump.
  // exception handler starts out as jump and can be patched to
  // a call be deoptimization.  (4932387)
  // Note that this value is also credited (in output.cpp) to
  // the size of the code section.
  return NativeJump::instruction_size;
}

// Emit exception handler code.  Stuff framesize into a register
// and call a VM stub routine.
int emit_exception_handler(CodeBuffer& cbuf) {

  // Note that the code buffer's inst_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  MacroAssembler _masm(&cbuf);
  address base =
  __ start_a_stub(size_exception_handler());
  if (base == NULL)  return 0;  // CodeBuffer::expand failed
  int offset = __ offset();
  __ jump(RuntimeAddress(OptoRuntime::exception_blob()->instructions_begin()));
  assert(__ offset() - offset <= (int) size_exception_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

uint size_deopt_handler() {
  // NativeCall instruction size is the same as NativeJump.
  // exception handler starts out as jump and can be patched to
  // a call be deoptimization.  (4932387)
  // Note that this value is also credited (in output.cpp) to
  // the size of the code section.
  return 5 + NativeJump::instruction_size; // pushl(); jmp;
}

// Emit deopt handler code.
int emit_deopt_handler(CodeBuffer& cbuf) {

  // Note that the code buffer's inst_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  MacroAssembler _masm(&cbuf);
  address base =
  __ start_a_stub(size_exception_handler());
  if (base == NULL)  return 0;  // CodeBuffer::expand failed
  int offset = __ offset();
  InternalAddress here(__ pc());
  __ pushptr(here.addr());

  __ jump(RuntimeAddress(SharedRuntime::deopt_blob()->unpack()));
  assert(__ offset() - offset <= (int) size_deopt_handler(), "overflow");
  __ end_a_stub();
  return offset;
}


static void emit_double_constant(CodeBuffer& cbuf, double x) {
  int mark = cbuf.insts()->mark_off();
  MacroAssembler _masm(&cbuf);
  address double_address = __ double_constant(x);
  cbuf.insts()->set_mark_off(mark);  // preserve mark across masm shift
  emit_d32_reloc(cbuf,
                 (int)double_address,
                 internal_word_Relocation::spec(double_address),
                 RELOC_DISP32);
}

static void emit_float_constant(CodeBuffer& cbuf, float x) {
  int mark = cbuf.insts()->mark_off();
  MacroAssembler _masm(&cbuf);
  address float_address = __ float_constant(x);
  cbuf.insts()->set_mark_off(mark);  // preserve mark across masm shift
  emit_d32_reloc(cbuf,
                 (int)float_address,
                 internal_word_Relocation::spec(float_address),
                 RELOC_DISP32);
}


int Matcher::regnum_to_fpu_offset(int regnum) {
  return regnum - 32; // The FP registers are in the second chunk
}

bool is_positive_zero_float(jfloat f) {
  return jint_cast(f) == jint_cast(0.0F);
}

bool is_positive_one_float(jfloat f) {
  return jint_cast(f) == jint_cast(1.0F);
}

bool is_positive_zero_double(jdouble d) {
  return jlong_cast(d) == jlong_cast(0.0);
}

bool is_positive_one_double(jdouble d) {
  return jlong_cast(d) == jlong_cast(1.0);
}

// This is UltraSparc specific, true just means we have fast l2f conversion
const bool Matcher::convL2FSupported(void) {
  return true;
}

// Vector width in bytes
const uint Matcher::vector_width_in_bytes(void) {
  return UseSSE >= 2 ? 8 : 0;
}

// Vector ideal reg
const uint Matcher::vector_ideal_reg(void) {
  return Op_RegD;
}

// Is this branch offset short enough that a short branch can be used?
//
// NOTE: If the platform does not provide any short branch variants, then
//       this method should return false for offset 0.
bool Matcher::is_short_branch_offset(int rule, int offset) {
  // the short version of jmpConUCF2 contains multiple branches,
  // making the reach slightly less
  if (rule == jmpConUCF2_rule)
    return (-126 <= offset && offset <= 125);
  return (-128 <= offset && offset <= 127);
}

const bool Matcher::isSimpleConstant64(jlong value) {
  // Will one (StoreL ConL) be cheaper than two (StoreI ConI)?.
  return false;
}

// The ecx parameter to rep stos for the ClearArray node is in dwords.
const bool Matcher::init_array_count_is_in_bytes = false;

// Threshold size for cleararray.
const int Matcher::init_array_short_size = 8 * BytesPerLong;

// Should the Matcher clone shifts on addressing modes, expecting them to
// be subsumed into complex addressing expressions or compute them into
// registers?  True for Intel but false for most RISCs
const bool Matcher::clone_shift_expressions = true;

// Is it better to copy float constants, or load them directly from memory?
// Intel can load a float constant from a direct address, requiring no
// extra registers.  Most RISCs will have to materialize an address into a
// register first, so they would do better to copy the constant from stack.
const bool Matcher::rematerialize_float_constants = true;

// If CPU can load and store mis-aligned doubles directly then no fixup is
// needed.  Else we split the double into 2 integer pieces and move it
// piece-by-piece.  Only happens when passing doubles into C code as the
// Java calling convention forces doubles to be aligned.
const bool Matcher::misaligned_doubles_ok = true;


void Matcher::pd_implicit_null_fixup(MachNode *node, uint idx) {
  // Get the memory operand from the node
  uint numopnds = node->num_opnds();        // Virtual call for number of operands
  uint skipped  = node->oper_input_base();  // Sum of leaves skipped so far
  assert( idx >= skipped, "idx too low in pd_implicit_null_fixup" );
  uint opcnt     = 1;                 // First operand
  uint num_edges = node->_opnds[1]->num_edges(); // leaves for first operand
  while( idx >= skipped+num_edges ) {
    skipped += num_edges;
    opcnt++;                          // Bump operand count
    assert( opcnt < numopnds, "Accessing non-existent operand" );
    num_edges = node->_opnds[opcnt]->num_edges(); // leaves for next operand
  }

  MachOper *memory = node->_opnds[opcnt];
  MachOper *new_memory = NULL;
  switch (memory->opcode()) {
  case DIRECT:
  case INDOFFSET32X:
    // No transformation necessary.
    return;
  case INDIRECT:
    new_memory = new (C) indirect_win95_safeOper( );
    break;
  case INDOFFSET8:
    new_memory = new (C) indOffset8_win95_safeOper(memory->disp(NULL, NULL, 0));
    break;
  case INDOFFSET32:
    new_memory = new (C) indOffset32_win95_safeOper(memory->disp(NULL, NULL, 0));
    break;
  case INDINDEXOFFSET:
    new_memory = new (C) indIndexOffset_win95_safeOper(memory->disp(NULL, NULL, 0));
    break;
  case INDINDEXSCALE:
    new_memory = new (C) indIndexScale_win95_safeOper(memory->scale());
    break;
  case INDINDEXSCALEOFFSET:
    new_memory = new (C) indIndexScaleOffset_win95_safeOper(memory->scale(), memory->disp(NULL, NULL, 0));
    break;
  case LOAD_LONG_INDIRECT:
  case LOAD_LONG_INDOFFSET32:
    // Does not use EBP as address register, use { EDX, EBX, EDI, ESI}
    return;
  default:
    assert(false, "unexpected memory operand in pd_implicit_null_fixup()");
    return;
  }
  node->_opnds[opcnt] = new_memory;
}

// Advertise here if the CPU requires explicit rounding operations
// to implement the UseStrictFP mode.
const bool Matcher::strict_fp_requires_explicit_rounding = true;

// Do floats take an entire double register or just half?
const bool Matcher::float_in_double = true;
// Do ints take an entire long register or just half?
const bool Matcher::int_in_long = false;

// Return whether or not this register is ever used as an argument.  This
// function is used on startup to build the trampoline stubs in generateOptoStub.
// Registers not mentioned will be killed by the VM call in the trampoline, and
// arguments in those registers not be available to the callee.
bool Matcher::can_be_java_arg( int reg ) {
  if(  reg == ECX_num   || reg == EDX_num   ) return true;
  if( (reg == XMM0a_num || reg == XMM1a_num) && UseSSE>=1 ) return true;
  if( (reg == XMM0b_num || reg == XMM1b_num) && UseSSE>=2 ) return true;
  return false;
}

bool Matcher::is_spillable_arg( int reg ) {
  return can_be_java_arg(reg);
}

// Register for DIVI projection of divmodI
RegMask Matcher::divI_proj_mask() {
  return EAX_REG_mask;
}

// Register for MODI projection of divmodI
RegMask Matcher::modI_proj_mask() {
  return EDX_REG_mask;
}

// Register for DIVL projection of divmodL
RegMask Matcher::divL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODL projection of divmodL
RegMask Matcher::modL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}



//SourceForm


// emit an interrupt that is caught by the debugger
void emit_break(CodeBuffer &cbuf) {

  // Debugger doesn't really catch this but best we can do so far QQQ
  MacroAssembler* masm = new MacroAssembler(&cbuf);
  masm->call(RuntimeAddress(CAST_FROM_FN_PTR(address, os::breakpoint)));
}

void MachBreakpointNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  emit_break(cbuf);
}


uint MachBreakpointNode::size(PhaseRegAlloc *ra_) const {
  return 5;
}



#ifndef PRODUCT
void Compile::adlc_verification() {

  // Following assertions generated from definition section
}
#endif

// Map from machine-independent register number to register_save_policy
const        char register_save_policy[] = {
  'C',
  'C',
  'N',
  'C',
  'C',
  'C',
  'C',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C' // no trailing comma
};

// Map from machine-independent register number to c_reg_save_policy
const        char c_reg_save_policy[] = {
  'C',
  'E',
  'E',
  'E',
  'C',
  'C',
  'E',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C' // no trailing comma
};

// Map from machine-independent register number to register_save_type
const        int register_save_type[] = {
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  0 // no trailing comma
};


const        int   reduceOp[] = {
  0, // 0
  0, // 1
  0, // 2
  0, // 3
  0, // 4
  0, // 5
  0, // 6
  0, // 7
  immI_rule, // 8
  immI0_rule, // 9
  immI1_rule, // 10
  immI_M1_rule, // 11
  immI2_rule, // 12
  immI8_rule, // 13
  immI16_rule, // 14
  immI_32_rule, // 15
  immI_1_31_rule, // 16
  immI_32_63_rule, // 17
  immI_1_rule, // 18
  immI_2_rule, // 19
  immI_3_rule, // 20
  immP_rule, // 21
  immP0_rule, // 22
  immL_rule, // 23
  immL0_rule, // 24
  immL_M1_rule, // 25
  immL_127_rule, // 26
  immL_32bits_rule, // 27
  immL32_rule, // 28
  immD0_rule, // 29
  immD1_rule, // 30
  immD_rule, // 31
  immXD_rule, // 32
  immXD0_rule, // 33
  immF0_rule, // 34
  immF_rule, // 35
  immXF_rule, // 36
  immXF0_rule, // 37
  immI_16_rule, // 38
  immI_24_rule, // 39
  immI_255_rule, // 40
  eRegI_rule, // 41
  xRegI_rule, // 42
  eAXRegI_rule, // 43
  eBXRegI_rule, // 44
  eCXRegI_rule, // 45
  eDXRegI_rule, // 46
  eDIRegI_rule, // 47
  naxRegI_rule, // 48
  nadxRegI_rule, // 49
  ncxRegI_rule, // 50
  eSIRegI_rule, // 51
  anyRegP_rule, // 52
  eRegP_rule, // 53
  eRegP_no_EBP_rule, // 54
  naxRegP_rule, // 55
  nabxRegP_rule, // 56
  pRegP_rule, // 57
  eAXRegP_rule, // 58
  eBXRegP_rule, // 59
  eCXRegP_rule, // 60
  eSIRegP_rule, // 61
  eDIRegP_rule, // 62
  eBPRegP_rule, // 63
  eRegL_rule, // 64
  eADXRegL_rule, // 65
  eBCXRegL_rule, // 66
  eADXRegL_low_only_rule, // 67
  eFlagsReg_rule, // 68
  eFlagsRegU_rule, // 69
  eFlagsRegUCF_rule, // 70
  flagsReg_long_LTGE_rule, // 71
  flagsReg_long_EQNE_rule, // 72
  flagsReg_long_LEGT_rule, // 73
  regD_rule, // 74
  regDPR1_rule, // 75
  regDPR2_rule, // 76
  regnotDPR1_rule, // 77
  regXD_rule, // 78
  regXD6_rule, // 79
  regXD7_rule, // 80
  regF_rule, // 81
  regFPR1_rule, // 82
  regX_rule, // 83
  direct_rule, // 84
  indirect_rule, // 85
  indOffset8_rule, // 86
  indOffset32_rule, // 87
  indOffset32X_rule, // 88
  indIndexOffset_rule, // 89
  indIndex_rule, // 90
  indIndexScale_rule, // 91
  indIndexScaleOffset_rule, // 92
  load_long_RegP_rule, // 93
  load_long_indirect_rule, // 94
  load_long_indOffset32_rule, // 95
  stackSlotP_rule, // 96
  stackSlotI_rule, // 97
  stackSlotF_rule, // 98
  stackSlotD_rule, // 99
  stackSlotL_rule, // 100
  indirect_win95_safe_rule, // 101
  indOffset8_win95_safe_rule, // 102
  indOffset32_win95_safe_rule, // 103
  indIndexOffset_win95_safe_rule, // 104
  indIndexScale_win95_safe_rule, // 105
  indIndexScaleOffset_win95_safe_rule, // 106
  cmpOp_rule, // 107
  cmpOpU_rule, // 108
  cmpOpUCF_rule, // 109
  cmpOpUCF2_rule, // 110
  cmpOp_fcmov_rule, // 111
  cmpOp_commute_rule, // 112
  // last operand
  load_long_memory_rule, // 113
  memory_rule, // 114
  long_memory_rule, // 115
  // last operand class
  _AddP_eRegP_eRegI_rule, // 116
  _LShiftI_eRegI_immI2_rule, // 117
  _AddP_eRegP__LShiftI_eRegI_immI2_rule, // 118
  _AddP_eRegP_no_EBP_eRegI_rule, // 119
  _AddP_eRegP_no_EBP__LShiftI_eRegI_immI2_rule, // 120
  _LoadB_memory__rule, // 121
  _RoundDouble_regDPR1__rule, // 122
  _RoundFloat_regFPR1__rule, // 123
  _ConvD2F_regDPR1__rule, // 124
  _Binary_cmpOp_eFlagsReg_rule, // 125
  _Binary_eRegI_eRegI_rule, // 126
  _Binary_cmpOpU_eFlagsRegU_rule, // 127
  _Binary_cmpOpUCF_eFlagsRegUCF_rule, // 128
  _LoadI_memory__rule, // 129
  _Binary_eRegI__LoadI_memory__rule, // 130
  _Binary_eRegP_eRegP_rule, // 131
  _Binary_cmpOp_fcmov_eFlagsRegU_rule, // 132
  _Binary_regDPR1_regD_rule, // 133
  _Binary_regFPR1_regF_rule, // 134
  _Binary_regD_regD_rule, // 135
  _Binary_regF_regF_rule, // 136
  _Binary_regX_regX_rule, // 137
  _Binary_regXD_regXD_rule, // 138
  _Binary_eRegL_eRegL_rule, // 139
  _AddI__LoadI_memory__eRegI_rule, // 140
  _AddI_eRegI__LoadI_memory__rule, // 141
  _AddI__LoadI_memory__immI_rule, // 142
  _AddI__LoadI_memory__immI1_rule, // 143
  _AddI__LoadI_memory__immI_M1_rule, // 144
  _Binary_eAXRegP_eRegP_rule, // 145
  _Binary_eAXRegI_eRegI_rule, // 146
  _Binary_eADXRegL_eBCXRegL_rule, // 147
  _Binary_eAXRegP_eCXRegP_rule, // 148
  _Binary_eAXRegI_eCXRegI_rule, // 149
  _SubI__LoadI_memory__eRegI_rule, // 150
  _SubI_immI0_eRegI_rule, // 151
  _ConvI2L_nadxRegI__rule, // 152
  _MulL__ConvI2L_nadxRegI__eADXRegL_low_only_rule, // 153
  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_rule, // 154
  _MulL_eADXRegL_low_only__ConvI2L_nadxRegI__rule, // 155
  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_rule, // 156
  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_63_rule, // 157
  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_63_rule, // 158
  _ConvI2L_eAXRegI__rule, // 159
  _AndL__ConvI2L_eAXRegI__immL_32bits_rule, // 160
  _AndL__ConvI2L_nadxRegI__immL_32bits_rule, // 161
  _RShiftI__LoadI_memory__immI1_rule, // 162
  _RShiftI__LoadI_memory__immI8_rule, // 163
  _LShiftI_xRegI_immI_24_rule, // 164
  _LShiftI_xRegI_immI_16_rule, // 165
  _AndI__LoadI_memory__eRegI_rule, // 166
  _AndI_eRegI__LoadI_memory__rule, // 167
  _AndI__LoadI_memory__immI_rule, // 168
  _CastP2X_eRegP__rule, // 169
  _OrI__LoadI_memory__eRegI_rule, // 170
  _OrI_eRegI__LoadI_memory__rule, // 171
  _OrI__LoadI_memory__immI_rule, // 172
  _LShiftI_eRegI_immI1_rule, // 173
  _URShiftI_eRegI_immI_M1_rule, // 174
  _LShiftI_eRegI_immI8_rule, // 175
  _URShiftI_eRegI_immI8_rule, // 176
  _LShiftI_ncxRegI_eCXRegI_rule, // 177
  _SubI_immI0_eCXRegI_rule, // 178
  _URShiftI_ncxRegI__SubI_immI0_eCXRegI_rule, // 179
  _SubI_immI_32_eCXRegI_rule, // 180
  _URShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule, // 181
  _URShiftI_eRegI_immI1_rule, // 182
  _LShiftI_eRegI_immI_M1_rule, // 183
  _URShiftI_ncxRegI_eCXRegI_rule, // 184
  _LShiftI_ncxRegI__SubI_immI0_eCXRegI_rule, // 185
  _LShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule, // 186
  _XorI__LoadI_memory__eRegI_rule, // 187
  _XorI_eRegI__LoadI_memory__rule, // 188
  _XorI__LoadI_memory__immI_rule, // 189
  _CmpLTMask_ncxRegI_ncxRegI_rule, // 190
  _AndI__CmpLTMask_ncxRegI_ncxRegI_ncxRegI_rule, // 191
  _SubI_ncxRegI_ncxRegI_rule, // 192
  _AndI_ncxRegI__CmpLTMask_ncxRegI_ncxRegI_rule, // 193
  _LoadL_load_long_memory__rule, // 194
  _LoadD_memory__rule, // 195
  _SubD_regD_regD_rule, // 196
  _AddD_regD_regD_rule, // 197
  _AddD__LoadD_memory__regD_rule, // 198
  _RoundDouble__AddD__LoadD_memory__regD__rule, // 199
  _AddD_regD__LoadD_memory__rule, // 200
  _RoundDouble__AddD_regD__LoadD_memory___rule, // 201
  _AddD_regD_immD_rule, // 202
  _MulD_regD_regD_rule, // 203
  _DivD_regD_regD_rule, // 204
  _LoadF_memory__rule, // 205
  _ConvF2D_regX__rule, // 206
  _SqrtD__ConvF2D_regX___rule, // 207
  _ConvF2D__LoadF_memory___rule, // 208
  _SqrtD__ConvF2D__LoadF_memory____rule, // 209
  _MulF__LoadF_memory__regF_rule, // 210
  _MulF_regF__LoadF_memory__rule, // 211
  _MulF_regF_regF_rule, // 212
  _SubF_regF_regF_rule, // 213
  _ConvI2L_eRegI__rule, // 214
  _AndI_eRegI_immI_rule, // 215
  _AndI_eRegI_memory_rule, // 216
  _LoadP_memory__rule, // 217
  _PartialSubtypeCheck_eSIRegP_eAXRegP_rule, // 218
  _Binary_cmpOp_flagsReg_long_LTGE_rule, // 219
  _Binary_eRegL__LoadL_load_long_memory__rule, // 220
  _Binary_cmpOp_flagsReg_long_EQNE_rule, // 221
  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule, // 222
  // last internally defined operand
  stackSlotI_rule, // 223
  stackSlotP_rule, // 224
  stackSlotL_rule, // 225
  eRegP_rule, // 226
  eRegP_rule, // 227
  eRegP_rule, // 228
  eRegP_rule, // 229
  eRegP_rule, // 230
  eRegI_rule, // 231
  eRegI_rule, // 232
  eRegP_rule, // 233
  eRegL_rule, // 234
  eRegL_rule, // 235
  regF_rule, // 236
  regX_rule, // 237
  regX_rule, // 238
  regD_rule, // 239
  regXD_rule, // 240
  regXD_rule, // 241
  eRegI_rule, // 242
  eRegL_rule, // 243
  eRegP_rule, // 244
  regF_rule, // 245
  regD_rule, // 246
  eADXRegL_low_only_rule, // 247
  eAXRegP_rule, // 248
  eRegI_rule, // 249
  eRegP_rule, // 250
  eRegP_rule, // 251
  eRegI_rule, // 252
  eFlagsReg_rule, // 253
  eFlagsReg_rule, // 254
  eFlagsReg_rule, // 255
  0, // 256
  0, // 257
  eFlagsRegU_rule, // 258
  eFlagsRegUCF_rule, // 259
  eFlagsRegU_rule, // 260
  eFlagsRegU_rule, // 261
  eFlagsRegUCF_rule, // 262
  eFlagsRegU_rule, // 263
  eFlagsRegUCF_rule, // 264
  regDPR1_rule, // 265
  regXD_rule, // 266
  regDPR1_rule, // 267
  regXD_rule, // 268
  eFlagsRegU_rule, // 269
  eFlagsRegUCF_rule, // 270
  eFlagsRegU_rule, // 271
  eFlagsRegU_rule, // 272
  eFlagsRegUCF_rule, // 273
  eFlagsRegU_rule, // 274
  eFlagsRegUCF_rule, // 275
  regFPR1_rule, // 276
  regX_rule, // 277
  regFPR1_rule, // 278
  regX_rule, // 279
  eRegI_rule, // 280
  eRegI_rule, // 281
  regF_rule, // 282
  regX_rule, // 283
  regX_rule, // 284
  eRegL_rule, // 285
  regD_rule, // 286
  regXD_rule, // 287
  regXD_rule, // 288
  eFlagsReg_rule, // 289
  eFlagsReg_rule, // 290
  eFlagsReg_rule, // 291
  eFlagsReg_rule, // 292
  eFlagsReg_rule, // 293
  eFlagsReg_rule, // 294
  eFlagsRegU_rule, // 295
  eFlagsRegU_rule, // 296
  eFlagsRegU_rule, // 297
  eFlagsRegU_rule, // 298
  eFlagsRegU_rule, // 299
  eFlagsRegU_rule, // 300
  eFlagsRegU_rule, // 301
  eFlagsRegU_rule, // 302
  eFlagsReg_rule, // 303
  eFlagsReg_rule, // 304
  eFlagsReg_rule, // 305
  flagsReg_long_LTGE_rule, // 306
  flagsReg_long_LTGE_rule, // 307
  flagsReg_long_EQNE_rule, // 308
  flagsReg_long_EQNE_rule, // 309
  flagsReg_long_LEGT_rule, // 310
  flagsReg_long_LEGT_rule, // 311
  eFlagsReg_rule, // 312
  eFlagsReg_rule, // 313
  Universe_rule, // 314
  eRegI_rule, // 315
  eRegL_rule, // 316
  xRegI_rule, // 317
  xRegI_rule, // 318
  eRegI_rule, // 319
  eRegI_rule, // 320
  eRegL_rule, // 321
  stackSlotL_rule, // 322
  stackSlotL_rule, // 323
  eRegL_rule, // 324
  eRegI_rule, // 325
  eRegP_rule, // 326
  eRegP_rule, // 327
  eRegI_rule, // 328
  regD_rule, // 329
  regXD_rule, // 330
  regXD_rule, // 331
  regX_rule, // 332
  regF_rule, // 333
  regXD_rule, // 334
  regXD_rule, // 335
  regXD_rule, // 336
  regXD_rule, // 337
  regXD_rule, // 338
  Universe_rule, // 339
  Universe_rule, // 340
  Universe_rule, // 341
  Universe_rule, // 342
  Universe_rule, // 343
  Universe_rule, // 344
  Universe_rule, // 345
  Universe_rule, // 346
  Universe_rule, // 347
  Universe_rule, // 348
  Universe_rule, // 349
  Universe_rule, // 350
  Universe_rule, // 351
  Universe_rule, // 352
  Universe_rule, // 353
  Universe_rule, // 354
  Universe_rule, // 355
  Universe_rule, // 356
  Universe_rule, // 357
  Universe_rule, // 358
  Universe_rule, // 359
  Universe_rule, // 360
  Universe_rule, // 361
  Universe_rule, // 362
  Universe_rule, // 363
  Universe_rule, // 364
  Universe_rule, // 365
  Universe_rule, // 366
  Universe_rule, // 367
  Universe_rule, // 368
  Universe_rule, // 369
  Universe_rule, // 370
  Universe_rule, // 371
  Universe_rule, // 372
  Universe_rule, // 373
  Universe_rule, // 374
  Universe_rule, // 375
  Universe_rule, // 376
  Universe_rule, // 377
  Universe_rule, // 378
  Universe_rule, // 379
  Universe_rule, // 380
  eRegI_rule, // 381
  eRegI_rule, // 382
  eRegI_rule, // 383
  eRegI_rule, // 384
  eRegI_rule, // 385
  eRegI_rule, // 386
  eRegP_rule, // 387
  eRegP_rule, // 388
  eRegP_rule, // 389
  eRegP_rule, // 390
  regDPR1_rule, // 391
  regFPR1_rule, // 392
  regD_rule, // 393
  regF_rule, // 394
  regX_rule, // 395
  regXD_rule, // 396
  regX_rule, // 397
  regX_rule, // 398
  regXD_rule, // 399
  regXD_rule, // 400
  eRegL_rule, // 401
  eRegL_rule, // 402
  eRegL_rule, // 403
  eRegI_rule, // 404
  eRegI_rule, // 405
  eRegI_rule, // 406
  eRegI_rule, // 407
  eRegP_rule, // 408
  eRegI_rule, // 409
  eRegP_rule, // 410
  eRegP_rule, // 411
  eRegI_rule, // 412
  eRegI_rule, // 413
  Universe_rule, // 414
  Universe_rule, // 415
  Universe_rule, // 416
  Universe_rule, // 417
  Universe_rule, // 418
  eRegP_rule, // 419
  stackSlotL_rule, // 420
  stackSlotL_rule, // 421
  eRegL_rule, // 422
  eRegI_rule, // 423
  eRegI_rule, // 424
  eRegI_rule, // 425
  eRegI_rule, // 426
  eRegI_rule, // 427
  eRegI_rule, // 428
  Universe_rule, // 429
  eRegP_rule, // 430
  eRegI_rule, // 431
  eRegI_rule, // 432
  eRegI_rule, // 433
  eDXRegI_rule, // 434
  eDXRegI_rule, // 435
  eDXRegI_rule, // 436
  eDXRegI_rule, // 437
  eRegI_rule, // 438
  eRegI_rule, // 439
  eRegI_rule, // 440
  eADXRegL_rule, // 441
  eADXRegL_rule, // 442
  eADXRegL_rule, // 443
  eADXRegL_rule, // 444
  eADXRegL_rule, // 445
  eADXRegL_rule, // 446
  eAXRegI_rule, // 447
  eADXRegL_rule, // 448
  Universe_rule, // 449
  eDXRegI_rule, // 450
  eADXRegL_rule, // 451
  eRegI_rule, // 452
  eRegI_rule, // 453
  eRegI_rule, // 454
  eRegI_rule, // 455
  Universe_rule, // 456
  eRegI_rule, // 457
  Universe_rule, // 458
  eRegI_rule, // 459
  eRegI_rule, // 460
  eRegI_rule, // 461
  eRegI_rule, // 462
  eRegI_rule, // 463
  eRegI_rule, // 464
  eRegI_rule, // 465
  eRegI_rule, // 466
  eRegI_rule, // 467
  eRegI_rule, // 468
  Universe_rule, // 469
  Universe_rule, // 470
  Universe_rule, // 471
  eRegI_rule, // 472
  eRegI_rule, // 473
  eRegI_rule, // 474
  eRegI_rule, // 475
  eRegI_rule, // 476
  eRegI_rule, // 477
  Universe_rule, // 478
  Universe_rule, // 479
  Universe_rule, // 480
  0, // 481
  0, // 482
  0, // 483
  eRegI_rule, // 484
  eRegI_rule, // 485
  eRegI_rule, // 486
  eRegI_rule, // 487
  ncxRegI_rule, // 488
  ncxRegI_rule, // 489
  ncxRegI_rule, // 490
  ncxRegI_rule, // 491
  0, // 492
  0, // 493
  0, // 494
  eRegI_rule, // 495
  eRegI_rule, // 496
  eRegI_rule, // 497
  eRegI_rule, // 498
  ncxRegI_rule, // 499
  ncxRegI_rule, // 500
  ncxRegI_rule, // 501
  ncxRegI_rule, // 502
  eRegI_rule, // 503
  eRegI_rule, // 504
  eRegI_rule, // 505
  eRegI_rule, // 506
  eRegI_rule, // 507
  Universe_rule, // 508
  Universe_rule, // 509
  Universe_rule, // 510
  0, // 511
  eRegI_rule, // 512
  0, // 513
  eRegI_rule, // 514
  eCXRegI_rule, // 515
  eRegI_rule, // 516
  ncxRegI_rule, // 517
  ncxRegI_rule, // 518
  ncxRegI_rule, // 519
  ncxRegI_rule, // 520
  eRegL_rule, // 521
  eRegL_rule, // 522
  eRegL_rule, // 523
  eRegL_rule, // 524
  eRegL_rule, // 525
  eRegL_rule, // 526
  eRegL_rule, // 527
  eRegL_rule, // 528
  eRegL_rule, // 529
  eRegL_rule, // 530
  eRegL_rule, // 531
  eRegL_rule, // 532
  eRegL_rule, // 533
  eRegL_rule, // 534
  eRegL_rule, // 535
  eRegL_rule, // 536
  eRegL_rule, // 537
  eRegL_rule, // 538
  eRegL_rule, // 539
  eRegL_rule, // 540
  eRegL_rule, // 541
  eRegL_rule, // 542
  eRegL_rule, // 543
  eRegL_rule, // 544
  eRegL_rule, // 545
  eRegL_rule, // 546
  eRegL_rule, // 547
  eRegL_rule, // 548
  eRegL_rule, // 549
  eRegL_rule, // 550
  eRegL_rule, // 551
  eRegL_rule, // 552
  eRegL_rule, // 553
  eRegI_rule, // 554
  eRegI_rule, // 555
  eRegI_rule, // 556
  eRegI_rule, // 557
  regD_rule, // 558
  stackSlotD_rule, // 559
  regD_rule, // 560
  regD_rule, // 561
  stackSlotD_rule, // 562
  regD_rule, // 563
  regD_rule, // 564
  Universe_rule, // 565
  Universe_rule, // 566
  regD_rule, // 567
  regD_rule, // 568
  stackSlotD_rule, // 569
  regXD_rule, // 570
  regXD_rule, // 571
  regXD_rule, // 572
  regXD_rule, // 573
  regXD_rule, // 574
  regXD_rule, // 575
  regXD_rule, // 576
  regXD_rule, // 577
  regXD_rule, // 578
  regXD_rule, // 579
  regXD_rule, // 580
  regXD_rule, // 581
  regXD_rule, // 582
  regXD_rule, // 583
  regD_rule, // 584
  regDPR1_rule, // 585
  regD_rule, // 586
  regD_rule, // 587
  regD_rule, // 588
  regD_rule, // 589
  regD_rule, // 590
  regD_rule, // 591
  regD_rule, // 592
  regD_rule, // 593
  regD_rule, // 594
  regDPR1_rule, // 595
  stackSlotD_rule, // 596
  regD_rule, // 597
  regXD_rule, // 598
  regDPR1_rule, // 599
  regXD_rule, // 600
  regDPR1_rule, // 601
  regXD_rule, // 602
  regDPR1_rule, // 603
  regXD_rule, // 604
  regD_rule, // 605
  regXD_rule, // 606
  regD_rule, // 607
  regDPR1_rule, // 608
  regXD_rule, // 609
  regDPR1_rule, // 610
  regXD_rule, // 611
  regDPR1_rule, // 612
  regXD_rule, // 613
  regDPR1_rule, // 614
  regXD_rule, // 615
  eRegI_rule, // 616
  eRegI_rule, // 617
  eRegI_rule, // 618
  eRegI_rule, // 619
  stackSlotF_rule, // 620
  regF_rule, // 621
  stackSlotF_rule, // 622
  regF_rule, // 623
  regX_rule, // 624
  regX_rule, // 625
  regX_rule, // 626
  regX_rule, // 627
  regX_rule, // 628
  regX_rule, // 629
  regX_rule, // 630
  regX_rule, // 631
  regX_rule, // 632
  regX_rule, // 633
  regX_rule, // 634
  regX_rule, // 635
  regX_rule, // 636
  regX_rule, // 637
  regX_rule, // 638
  regX_rule, // 639
  regXD_rule, // 640
  regXD_rule, // 641
  stackSlotF_rule, // 642
  stackSlotF_rule, // 643
  regF_rule, // 644
  regF_rule, // 645
  stackSlotF_rule, // 646
  stackSlotF_rule, // 647
  stackSlotF_rule, // 648
  stackSlotF_rule, // 649
  stackSlotF_rule, // 650
  regF_rule, // 651
  stackSlotF_rule, // 652
  regF_rule, // 653
  stackSlotF_rule, // 654
  stackSlotF_rule, // 655
  regF_rule, // 656
  regF_rule, // 657
  stackSlotF_rule, // 658
  stackSlotF_rule, // 659
  regF_rule, // 660
  regF_rule, // 661
  regF_rule, // 662
  regF_rule, // 663
  regF_rule, // 664
  regF_rule, // 665
  regF_rule, // 666
  regF_rule, // 667
  regF_rule, // 668
  regF_rule, // 669
  stackSlotF_rule, // 670
  regF_rule, // 671
  stackSlotF_rule, // 672
  regF_rule, // 673
  regX_rule, // 674
  stackSlotF_rule, // 675
  stackSlotD_rule, // 676
  stackSlotF_rule, // 677
  regX_rule, // 678
  regX_rule, // 679
  regD_rule, // 680
  stackSlotD_rule, // 681
  regD_rule, // 682
  regXD_rule, // 683
  eAXRegI_rule, // 684
  eAXRegI_rule, // 685
  eADXRegL_rule, // 686
  eADXRegL_rule, // 687
  eAXRegI_rule, // 688
  eAXRegI_rule, // 689
  eADXRegL_rule, // 690
  eADXRegL_rule, // 691
  regD_rule, // 692
  regXD_rule, // 693
  regXD_rule, // 694
  regXD_rule, // 695
  regD_rule, // 696
  regF_rule, // 697
  stackSlotF_rule, // 698
  stackSlotF_rule, // 699
  regF_rule, // 700
  regF_rule, // 701
  regX_rule, // 702
  regX_rule, // 703
  eRegL_rule, // 704
  eRegL_rule, // 705
  eRegL_rule, // 706
  stackSlotD_rule, // 707
  regXD_rule, // 708
  regX_rule, // 709
  stackSlotF_rule, // 710
  eRegI_rule, // 711
  stackSlotI_rule, // 712
  stackSlotI_rule, // 713
  stackSlotF_rule, // 714
  stackSlotL_rule, // 715
  stackSlotL_rule, // 716
  eRegL_rule, // 717
  stackSlotD_rule, // 718
  regXD_rule, // 719
  regXD_rule, // 720
  regXD_rule, // 721
  regXD_rule, // 722
  regXD_rule, // 723
  regXD_rule, // 724
  regXD_rule, // 725
  regXD_rule, // 726
  regXD_rule, // 727
  regXD_rule, // 728
  regXD_rule, // 729
  regXD_rule, // 730
  regXD_rule, // 731
  regXD_rule, // 732
  regXD_rule, // 733
  regXD_rule, // 734
  Universe_rule, // 735
  eCXRegI_rule, // 736
  eCXRegI_rule, // 737
  eRegI_rule, // 738
  eRegI_rule, // 739
  Universe_rule, // 740
  Universe_rule, // 741
  Universe_rule, // 742
  Universe_rule, // 743
  Universe_rule, // 744
  Universe_rule, // 745
  Universe_rule, // 746
  Universe_rule, // 747
  Universe_rule, // 748
  eDIRegP_rule, // 749
  Universe_rule, // 750
  Universe_rule, // 751
  Universe_rule, // 752
  Universe_rule, // 753
  Universe_rule, // 754
  Universe_rule, // 755
  Universe_rule, // 756
  Universe_rule, // 757
  eSIRegI_rule, // 758
  Universe_rule, // 759
  eRegL_rule, // 760
  eRegL_rule, // 761
  eRegI_rule, // 762
  eRegI_rule, // 763
  eRegP_rule, // 764
  regD_rule, // 765
  regXD_rule, // 766
  regF_rule, // 767
  regX_rule, // 768
  Universe_rule, // 769
  eRegL_rule, // 770
  eRegL_rule, // 771
  eRegI_rule, // 772
  eRegI_rule, // 773
  eRegP_rule, // 774
  regD_rule, // 775
  regXD_rule, // 776
  regF_rule, // 777
  regX_rule, // 778
  Universe_rule, // 779
  eRegL_rule, // 780
  eRegL_rule, // 781
  eRegI_rule, // 782
  eRegI_rule, // 783
  eRegP_rule, // 784
  regD_rule, // 785
  regXD_rule, // 786
  regF_rule, // 787
  regX_rule, // 788
  Universe_rule, // 789
  Universe_rule, // 790
  Universe_rule, // 791
  Universe_rule, // 792
  Universe_rule, // 793
  Universe_rule, // 794
  Universe_rule, // 795
  Universe_rule, // 796
  eAXRegP_rule, // 797
  Universe_rule, // 798
  eRegP_rule, // 799
  eRegP_rule, // 800
  Universe_rule, // 801
  // last instruction
  0 // no trailing comma
};

const        int   leftOp[] = {
  0, // 0
  0, // 1
  0, // 2
  0, // 3
  0, // 4
  0, // 5
  0, // 6
  0, // 7
  0, // 8
  0, // 9
  0, // 10
  0, // 11
  0, // 12
  0, // 13
  0, // 14
  0, // 15
  0, // 16
  0, // 17
  0, // 18
  0, // 19
  0, // 20
  0, // 21
  0, // 22
  0, // 23
  0, // 24
  0, // 25
  0, // 26
  0, // 27
  0, // 28
  0, // 29
  0, // 30
  0, // 31
  0, // 32
  0, // 33
  0, // 34
  0, // 35
  0, // 36
  0, // 37
  0, // 38
  0, // 39
  0, // 40
  0, // 41
  0, // 42
  0, // 43
  0, // 44
  0, // 45
  0, // 46
  0, // 47
  0, // 48
  0, // 49
  0, // 50
  0, // 51
  0, // 52
  0, // 53
  0, // 54
  0, // 55
  0, // 56
  0, // 57
  0, // 58
  0, // 59
  0, // 60
  0, // 61
  0, // 62
  0, // 63
  0, // 64
  0, // 65
  0, // 66
  0, // 67
  0, // 68
  0, // 69
  0, // 70
  0, // 71
  0, // 72
  0, // 73
  0, // 74
  0, // 75
  0, // 76
  0, // 77
  0, // 78
  0, // 79
  0, // 80
  0, // 81
  0, // 82
  0, // 83
  0, // 84
  0, // 85
  eRegP_rule, // 86
  eRegP_rule, // 87
  immP_rule, // 88
  _AddP_eRegP_eRegI_rule, // 89
  eRegP_rule, // 90
  eRegP_rule, // 91
  _AddP_eRegP__LShiftI_eRegI_immI2_rule, // 92
  0, // 93
  0, // 94
  load_long_RegP_rule, // 95
  0, // 96
  0, // 97
  0, // 98
  0, // 99
  0, // 100
  0, // 101
  eRegP_no_EBP_rule, // 102
  eRegP_no_EBP_rule, // 103
  _AddP_eRegP_no_EBP_eRegI_rule, // 104
  eRegP_no_EBP_rule, // 105
  _AddP_eRegP_no_EBP__LShiftI_eRegI_immI2_rule, // 106
  0, // 107
  0, // 108
  0, // 109
  0, // 110
  0, // 111
  0, // 112
  // last operand
  0, // 113
  0, // 114
  0, // 115
  // last operand class
  eRegP_rule, // 116
  eRegI_rule, // 117
  eRegP_rule, // 118
  eRegP_no_EBP_rule, // 119
  eRegP_no_EBP_rule, // 120
  memory_rule, // 121
  regDPR1_rule, // 122
  regFPR1_rule, // 123
  regDPR1_rule, // 124
  cmpOp_rule, // 125
  eRegI_rule, // 126
  cmpOpU_rule, // 127
  cmpOpUCF_rule, // 128
  memory_rule, // 129
  eRegI_rule, // 130
  eRegP_rule, // 131
  cmpOp_fcmov_rule, // 132
  regDPR1_rule, // 133
  regFPR1_rule, // 134
  regD_rule, // 135
  regF_rule, // 136
  regX_rule, // 137
  regXD_rule, // 138
  eRegL_rule, // 139
  _LoadI_memory__rule, // 140
  eRegI_rule, // 141
  _LoadI_memory__rule, // 142
  _LoadI_memory__rule, // 143
  _LoadI_memory__rule, // 144
  eAXRegP_rule, // 145
  eAXRegI_rule, // 146
  eADXRegL_rule, // 147
  eAXRegP_rule, // 148
  eAXRegI_rule, // 149
  _LoadI_memory__rule, // 150
  immI0_rule, // 151
  nadxRegI_rule, // 152
  _ConvI2L_nadxRegI__rule, // 153
  _MulL__ConvI2L_nadxRegI__eADXRegL_low_only_rule, // 154
  eADXRegL_low_only_rule, // 155
  _MulL_eADXRegL_low_only__ConvI2L_nadxRegI__rule, // 156
  _MulL__ConvI2L_nadxRegI__eADXRegL_low_only_rule, // 157
  _MulL_eADXRegL_low_only__ConvI2L_nadxRegI__rule, // 158
  eAXRegI_rule, // 159
  _ConvI2L_eAXRegI__rule, // 160
  _ConvI2L_nadxRegI__rule, // 161
  _LoadI_memory__rule, // 162
  _LoadI_memory__rule, // 163
  xRegI_rule, // 164
  xRegI_rule, // 165
  _LoadI_memory__rule, // 166
  eRegI_rule, // 167
  _LoadI_memory__rule, // 168
  eRegP_rule, // 169
  _LoadI_memory__rule, // 170
  eRegI_rule, // 171
  _LoadI_memory__rule, // 172
  eRegI_rule, // 173
  eRegI_rule, // 174
  eRegI_rule, // 175
  eRegI_rule, // 176
  ncxRegI_rule, // 177
  immI0_rule, // 178
  ncxRegI_rule, // 179
  immI_32_rule, // 180
  ncxRegI_rule, // 181
  eRegI_rule, // 182
  eRegI_rule, // 183
  ncxRegI_rule, // 184
  ncxRegI_rule, // 185
  ncxRegI_rule, // 186
  _LoadI_memory__rule, // 187
  eRegI_rule, // 188
  _LoadI_memory__rule, // 189
  ncxRegI_rule, // 190
  _CmpLTMask_ncxRegI_ncxRegI_rule, // 191
  ncxRegI_rule, // 192
  ncxRegI_rule, // 193
  load_long_memory_rule, // 194
  memory_rule, // 195
  regD_rule, // 196
  regD_rule, // 197
  _LoadD_memory__rule, // 198
  _AddD__LoadD_memory__regD_rule, // 199
  regD_rule, // 200
  _AddD_regD__LoadD_memory__rule, // 201
  regD_rule, // 202
  regD_rule, // 203
  regD_rule, // 204
  memory_rule, // 205
  regX_rule, // 206
  _ConvF2D_regX__rule, // 207
  _LoadF_memory__rule, // 208
  _ConvF2D__LoadF_memory___rule, // 209
  _LoadF_memory__rule, // 210
  regF_rule, // 211
  regF_rule, // 212
  regF_rule, // 213
  eRegI_rule, // 214
  eRegI_rule, // 215
  eRegI_rule, // 216
  memory_rule, // 217
  eSIRegP_rule, // 218
  cmpOp_rule, // 219
  eRegL_rule, // 220
  cmpOp_rule, // 221
  cmpOp_commute_rule, // 222
  // last internally defined operand
  eRegI_rule, // 223
  eRegP_rule, // 224
  eRegL_rule, // 225
  indOffset8_rule, // 226
  indOffset32_rule, // 227
  indIndexOffset_rule, // 228
  indIndexScale_rule, // 229
  indIndexScaleOffset_rule, // 230
  immI_rule, // 231
  immI0_rule, // 232
  immP_rule, // 233
  immL_rule, // 234
  immL0_rule, // 235
  immF_rule, // 236
  immXF_rule, // 237
  immXF0_rule, // 238
  immD_rule, // 239
  immXD_rule, // 240
  immXD0_rule, // 241
  stackSlotI_rule, // 242
  stackSlotL_rule, // 243
  stackSlotP_rule, // 244
  stackSlotF_rule, // 245
  stackSlotD_rule, // 246
  immL32_rule, // 247
  eAXRegI_rule, // 248
  eRegP_rule, // 249
  eRegP_rule, // 250
  eRegP_rule, // 251
  eRegI_rule, // 252
  memory_rule, // 253
  memory_rule, // 254
  memory_rule, // 255
  0, // 256
  0, // 257
  regD_rule, // 258
  regD_rule, // 259
  regD_rule, // 260
  regXD_rule, // 261
  regXD_rule, // 262
  regXD_rule, // 263
  regXD_rule, // 264
  regDPR1_rule, // 265
  regXD_rule, // 266
  regDPR1_rule, // 267
  regXD_rule, // 268
  regF_rule, // 269
  regF_rule, // 270
  regF_rule, // 271
  regX_rule, // 272
  regX_rule, // 273
  regX_rule, // 274
  regX_rule, // 275
  regFPR1_rule, // 276
  regX_rule, // 277
  regFPR1_rule, // 278
  regX_rule, // 279
  stackSlotF_rule, // 280
  regX_rule, // 281
  stackSlotI_rule, // 282
  stackSlotI_rule, // 283
  eRegI_rule, // 284
  stackSlotD_rule, // 285
  stackSlotL_rule, // 286
  stackSlotL_rule, // 287
  stackSlotL_rule, // 288
  eRegI_rule, // 289
  eRegI_rule, // 290
  eRegI_rule, // 291
  eRegI_rule, // 292
  _AndI_eRegI_immI_rule, // 293
  _AndI_eRegI_memory_rule, // 294
  eRegI_rule, // 295
  eRegI_rule, // 296
  eRegI_rule, // 297
  eRegI_rule, // 298
  eRegP_rule, // 299
  eRegP_rule, // 300
  eRegP_rule, // 301
  eRegP_rule, // 302
  eRegP_rule, // 303
  _LoadP_memory__rule, // 304
  _PartialSubtypeCheck_eSIRegP_eAXRegP_rule, // 305
  eRegL_rule, // 306
  eRegL_rule, // 307
  eRegL_rule, // 308
  eRegL_rule, // 309
  eRegL_rule, // 310
  eRegL_rule, // 311
  eRegP_rule, // 312
  eRegP_rule, // 313
  0, // 314
  eRegI_rule, // 315
  eRegL_rule, // 316
  memory_rule, // 317
  _LoadB_memory__rule, // 318
  memory_rule, // 319
  memory_rule, // 320
  load_long_memory_rule, // 321
  memory_rule, // 322
  memory_rule, // 323
  memory_rule, // 324
  memory_rule, // 325
  memory_rule, // 326
  memory_rule, // 327
  memory_rule, // 328
  memory_rule, // 329
  memory_rule, // 330
  memory_rule, // 331
  memory_rule, // 332
  memory_rule, // 333
  memory_rule, // 334
  memory_rule, // 335
  memory_rule, // 336
  memory_rule, // 337
  memory_rule, // 338
  memory_rule, // 339
  memory_rule, // 340
  memory_rule, // 341
  memory_rule, // 342
  memory_rule, // 343
  memory_rule, // 344
  memory_rule, // 345
  memory_rule, // 346
  memory_rule, // 347
  memory_rule, // 348
  memory_rule, // 349
  memory_rule, // 350
  memory_rule, // 351
  long_memory_rule, // 352
  memory_rule, // 353
  memory_rule, // 354
  memory_rule, // 355
  memory_rule, // 356
  memory_rule, // 357
  memory_rule, // 358
  memory_rule, // 359
  memory_rule, // 360
  memory_rule, // 361
  memory_rule, // 362
  memory_rule, // 363
  memory_rule, // 364
  memory_rule, // 365
  memory_rule, // 366
  memory_rule, // 367
  memory_rule, // 368
  memory_rule, // 369
  memory_rule, // 370
  memory_rule, // 371
  memory_rule, // 372
  memory_rule, // 373
  memory_rule, // 374
  0, // 375
  0, // 376
  0, // 377
  0, // 378
  0, // 379
  0, // 380
  _Binary_cmpOp_eFlagsReg_rule, // 381
  _Binary_cmpOpU_eFlagsRegU_rule, // 382
  _Binary_cmpOpUCF_eFlagsRegUCF_rule, // 383
  _Binary_cmpOp_eFlagsReg_rule, // 384
  _Binary_cmpOpU_eFlagsRegU_rule, // 385
  _Binary_cmpOpUCF_eFlagsRegUCF_rule, // 386
  _Binary_cmpOp_eFlagsReg_rule, // 387
  _Binary_cmpOp_eFlagsReg_rule, // 388
  _Binary_cmpOpU_eFlagsRegU_rule, // 389
  _Binary_cmpOpUCF_eFlagsRegUCF_rule, // 390
  _Binary_cmpOp_fcmov_eFlagsRegU_rule, // 391
  _Binary_cmpOp_fcmov_eFlagsRegU_rule, // 392
  _Binary_cmpOp_eFlagsReg_rule, // 393
  _Binary_cmpOp_eFlagsReg_rule, // 394
  _Binary_cmpOp_eFlagsReg_rule, // 395
  _Binary_cmpOp_eFlagsReg_rule, // 396
  _Binary_cmpOpU_eFlagsRegU_rule, // 397
  _Binary_cmpOpUCF_eFlagsRegUCF_rule, // 398
  _Binary_cmpOpU_eFlagsRegU_rule, // 399
  _Binary_cmpOpUCF_eFlagsRegUCF_rule, // 400
  _Binary_cmpOp_eFlagsReg_rule, // 401
  _Binary_cmpOpU_eFlagsRegU_rule, // 402
  _Binary_cmpOpUCF_eFlagsRegUCF_rule, // 403
  eRegI_rule, // 404
  eRegI_rule, // 405
  eRegI_rule, // 406
  eRegI_rule, // 407
  eRegP_rule, // 408
  eRegI_rule, // 409
  eRegP_rule, // 410
  eRegP_rule, // 411
  eRegI_rule, // 412
  _LoadI_memory__rule, // 413
  memory_rule, // 414
  memory_rule, // 415
  memory_rule, // 416
  memory_rule, // 417
  memory_rule, // 418
  memory_rule, // 419
  load_long_memory_rule, // 420
  load_long_memory_rule, // 421
  load_long_memory_rule, // 422
  eSIRegP_rule, // 423
  pRegP_rule, // 424
  pRegP_rule, // 425
  eRegI_rule, // 426
  eRegI_rule, // 427
  eRegI_rule, // 428
  memory_rule, // 429
  eRegP_rule, // 430
  immI0_rule, // 431
  eRegI_rule, // 432
  eRegI_rule, // 433
  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_rule, // 434
  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_rule, // 435
  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_63_rule, // 436
  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_63_rule, // 437
  _LoadI_memory__rule, // 438
  eRegI_rule, // 439
  _LoadI_memory__rule, // 440
  _ConvI2L_eAXRegI__rule, // 441
  _ConvI2L_nadxRegI__rule, // 442
  _AndL__ConvI2L_eAXRegI__immL_32bits_rule, // 443
  _AndL__ConvI2L_nadxRegI__immL_32bits_rule, // 444
  eADXRegL_rule, // 445
  eADXRegL_rule, // 446
  eAXRegI_rule, // 447
  eRegL_rule, // 448
  eAXRegI_rule, // 449
  eAXRegI_rule, // 450
  eRegL_rule, // 451
  eRegI_rule, // 452
  eRegI_rule, // 453
  eRegI_rule, // 454
  eRegI_rule, // 455
  memory_rule, // 456
  eRegI_rule, // 457
  memory_rule, // 458
  eRegI_rule, // 459
  eRegI_rule, // 460
  eRegI_rule, // 461
  _LShiftI_xRegI_immI_24_rule, // 462
  _LShiftI_xRegI_immI_16_rule, // 463
  eRegI_rule, // 464
  eRegI_rule, // 465
  eRegI_rule, // 466
  eRegI_rule, // 467
  _LoadI_memory__rule, // 468
  memory_rule, // 469
  memory_rule, // 470
  memory_rule, // 471
  eRegI_rule, // 472
  eRegI_rule, // 473
  _CastP2X_eRegP__rule, // 474
  eRegI_rule, // 475
  eRegI_rule, // 476
  _LoadI_memory__rule, // 477
  memory_rule, // 478
  memory_rule, // 479
  memory_rule, // 480
  0, // 481
  0, // 482
  0, // 483
  _LShiftI_eRegI_immI1_rule, // 484
  _URShiftI_eRegI_immI_M1_rule, // 485
  _LShiftI_eRegI_immI8_rule, // 486
  _URShiftI_eRegI_immI8_rule, // 487
  _LShiftI_ncxRegI_eCXRegI_rule, // 488
  _URShiftI_ncxRegI__SubI_immI0_eCXRegI_rule, // 489
  _LShiftI_ncxRegI_eCXRegI_rule, // 490
  _URShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule, // 491
  0, // 492
  0, // 493
  0, // 494
  _URShiftI_eRegI_immI1_rule, // 495
  _LShiftI_eRegI_immI_M1_rule, // 496
  _URShiftI_eRegI_immI8_rule, // 497
  _LShiftI_eRegI_immI8_rule, // 498
  _URShiftI_ncxRegI_eCXRegI_rule, // 499
  _LShiftI_ncxRegI__SubI_immI0_eCXRegI_rule, // 500
  _URShiftI_ncxRegI_eCXRegI_rule, // 501
  _LShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule, // 502
  eRegI_rule, // 503
  eRegI_rule, // 504
  eRegI_rule, // 505
  eRegI_rule, // 506
  _LoadI_memory__rule, // 507
  memory_rule, // 508
  memory_rule, // 509
  memory_rule, // 510
  0, // 511
  eRegI_rule, // 512
  0, // 513
  eRegP_rule, // 514
  ncxRegI_rule, // 515
  eRegI_rule, // 516
  _AndI__CmpLTMask_ncxRegI_ncxRegI_ncxRegI_rule, // 517
  _SubI_ncxRegI_ncxRegI_rule, // 518
  _AndI_ncxRegI__CmpLTMask_ncxRegI_ncxRegI_rule, // 519
  _SubI_ncxRegI_ncxRegI_rule, // 520
  eRegL_rule, // 521
  eRegL_rule, // 522
  eRegL_rule, // 523
  _LoadL_load_long_memory__rule, // 524
  eRegL_rule, // 525
  eRegL_rule, // 526
  eRegL_rule, // 527
  immL0_rule, // 528
  eRegL_rule, // 529
  eRegL_rule, // 530
  eRegL_rule, // 531
  _LoadL_load_long_memory__rule, // 532
  eRegL_rule, // 533
  eRegL_rule, // 534
  eRegL_rule, // 535
  _LoadL_load_long_memory__rule, // 536
  eRegL_rule, // 537
  eRegL_rule, // 538
  eRegL_rule, // 539
  eRegL_rule, // 540
  _LoadL_load_long_memory__rule, // 541
  eRegL_rule, // 542
  eRegL_rule, // 543
  eRegL_rule, // 544
  eRegL_rule, // 545
  eRegL_rule, // 546
  eRegL_rule, // 547
  eRegL_rule, // 548
  eRegL_rule, // 549
  eRegL_rule, // 550
  eRegL_rule, // 551
  eRegL_rule, // 552
  eRegL_rule, // 553
  regD_rule, // 554
  regD_rule, // 555
  regXD_rule, // 556
  regXD_rule, // 557
  regD_rule, // 558
  _SubD_regD_regD_rule, // 559
  regD_rule, // 560
  regD_rule, // 561
  _AddD_regD_regD_rule, // 562
  regD_rule, // 563
  _LoadD_memory__rule, // 564
  memory_rule, // 565
  memory_rule, // 566
  regD_rule, // 567
  regD_rule, // 568
  _AddD_regD_immD_rule, // 569
  regXD_rule, // 570
  regXD_rule, // 571
  regXD_rule, // 572
  _LoadD_memory__rule, // 573
  regXD_rule, // 574
  regXD_rule, // 575
  regXD_rule, // 576
  regXD_rule, // 577
  regXD_rule, // 578
  regXD_rule, // 579
  _LoadD_memory__rule, // 580
  regXD_rule, // 581
  regXD_rule, // 582
  regXD_rule, // 583
  regD_rule, // 584
  regDPR1_rule, // 585
  regD_rule, // 586
  regD_rule, // 587
  _LoadD_memory__rule, // 588
  regD_rule, // 589
  _LoadD_memory__rule, // 590
  _MulD_regD_regD_rule, // 591
  regD_rule, // 592
  _MulD_regD_regD_rule, // 593
  regD_rule, // 594
  regDPR1_rule, // 595
  _DivD_regD_regD_rule, // 596
  regD_rule, // 597
  regXD_rule, // 598
  regDPR1_rule, // 599
  regXD_rule, // 600
  regDPR1_rule, // 601
  regXD_rule, // 602
  regDPR1_rule, // 603
  regXD_rule, // 604
  regD_rule, // 605
  regXD_rule, // 606
  regD_rule, // 607
  regD_rule, // 608
  regXD_rule, // 609
  regDPR1_rule, // 610
  regXD_rule, // 611
  regDPR1_rule, // 612
  regXD_rule, // 613
  regDPR1_rule, // 614
  regXD_rule, // 615
  regF_rule, // 616
  regF_rule, // 617
  regX_rule, // 618
  regX_rule, // 619
  regF_rule, // 620
  regF_rule, // 621
  regF_rule, // 622
  regF_rule, // 623
  regX_rule, // 624
  regX_rule, // 625
  regX_rule, // 626
  _LoadF_memory__rule, // 627
  regX_rule, // 628
  regX_rule, // 629
  regX_rule, // 630
  regX_rule, // 631
  regX_rule, // 632
  regX_rule, // 633
  _LoadF_memory__rule, // 634
  regX_rule, // 635
  regX_rule, // 636
  regX_rule, // 637
  _SqrtD__ConvF2D_regX___rule, // 638
  _SqrtD__ConvF2D__LoadF_memory____rule, // 639
  regXD_rule, // 640
  _LoadD_memory__rule, // 641
  regF_rule, // 642
  _LoadF_memory__rule, // 643
  regF_rule, // 644
  _LoadF_memory__rule, // 645
  memory_rule, // 646
  memory_rule, // 647
  _LoadF_memory__rule, // 648
  memory_rule, // 649
  regF_rule, // 650
  regF_rule, // 651
  regF_rule, // 652
  regF_rule, // 653
  regF_rule, // 654
  _LoadF_memory__rule, // 655
  regF_rule, // 656
  _LoadF_memory__rule, // 657
  memory_rule, // 658
  regF_rule, // 659
  regF_rule, // 660
  _LoadF_memory__rule, // 661
  regF_rule, // 662
  _MulF__LoadF_memory__regF_rule, // 663
  regF_rule, // 664
  _MulF_regF__LoadF_memory__rule, // 665
  regF_rule, // 666
  _MulF_regF_regF_rule, // 667
  regF_rule, // 668
  _SubF_regF_regF_rule, // 669
  regF_rule, // 670
  regF_rule, // 671
  regF_rule, // 672
  regF_rule, // 673
  regX_rule, // 674
  regF_rule, // 675
  regD_rule, // 676
  regD_rule, // 677
  regD_rule, // 678
  regXD_rule, // 679
  regF_rule, // 680
  regF_rule, // 681
  regX_rule, // 682
  regX_rule, // 683
  regD_rule, // 684
  regXD_rule, // 685
  regD_rule, // 686
  regXD_rule, // 687
  regF_rule, // 688
  regX_rule, // 689
  regF_rule, // 690
  regX_rule, // 691
  stackSlotI_rule, // 692
  eRegI_rule, // 693
  _LoadI_memory__rule, // 694
  eRegI_rule, // 695
  _LoadI_memory__rule, // 696
  stackSlotI_rule, // 697
  stackSlotI_rule, // 698
  _LoadI_memory__rule, // 699
  stackSlotI_rule, // 700
  _LoadI_memory__rule, // 701
  eRegI_rule, // 702
  eRegI_rule, // 703
  eRegI_rule, // 704
  _ConvI2L_eRegI__rule, // 705
  eRegL_rule, // 706
  eRegL_rule, // 707
  eRegL_rule, // 708
  eRegL_rule, // 709
  eRegL_rule, // 710
  eRegL_rule, // 711
  regF_rule, // 712
  regX_rule, // 713
  eRegI_rule, // 714
  regD_rule, // 715
  regXD_rule, // 716
  regXD_rule, // 717
  eRegL_rule, // 718
  eRegL_rule, // 719
  regXD_rule, // 720
  eRegI_rule, // 721
  immI0_rule, // 722
  regXD_rule, // 723
  eRegI_rule, // 724
  immI0_rule, // 725
  regXD_rule, // 726
  eRegI_rule, // 727
  immI0_rule, // 728
  regXD_rule, // 729
  eRegI_rule, // 730
  immI0_rule, // 731
  regXD_rule, // 732
  regX_rule, // 733
  immXF0_rule, // 734
  eCXRegI_rule, // 735
  eDIRegP_rule, // 736
  eDIRegP_rule, // 737
  eRegI_rule, // 738
  eRegI_rule, // 739
  eRegI_rule, // 740
  0, // 741
  cmpOp_rule, // 742
  cmpOp_rule, // 743
  cmpOpU_rule, // 744
  cmpOpUCF_rule, // 745
  cmpOpU_rule, // 746
  cmpOpUCF_rule, // 747
  cmpOpUCF2_rule, // 748
  eSIRegP_rule, // 749
  0, // 750
  cmpOp_rule, // 751
  cmpOp_rule, // 752
  cmpOpU_rule, // 753
  cmpOpUCF_rule, // 754
  cmpOpU_rule, // 755
  cmpOpUCF_rule, // 756
  cmpOpUCF2_rule, // 757
  eRegL_rule, // 758
  cmpOp_rule, // 759
  _Binary_cmpOp_flagsReg_long_LTGE_rule, // 760
  _Binary_cmpOp_flagsReg_long_LTGE_rule, // 761
  _Binary_cmpOp_flagsReg_long_LTGE_rule, // 762
  _Binary_cmpOp_flagsReg_long_LTGE_rule, // 763
  _Binary_cmpOp_flagsReg_long_LTGE_rule, // 764
  _Binary_cmpOp_flagsReg_long_LTGE_rule, // 765
  _Binary_cmpOp_flagsReg_long_LTGE_rule, // 766
  _Binary_cmpOp_flagsReg_long_LTGE_rule, // 767
  _Binary_cmpOp_flagsReg_long_LTGE_rule, // 768
  cmpOp_rule, // 769
  _Binary_cmpOp_flagsReg_long_EQNE_rule, // 770
  _Binary_cmpOp_flagsReg_long_EQNE_rule, // 771
  _Binary_cmpOp_flagsReg_long_EQNE_rule, // 772
  _Binary_cmpOp_flagsReg_long_EQNE_rule, // 773
  _Binary_cmpOp_flagsReg_long_EQNE_rule, // 774
  _Binary_cmpOp_flagsReg_long_EQNE_rule, // 775
  _Binary_cmpOp_flagsReg_long_EQNE_rule, // 776
  _Binary_cmpOp_flagsReg_long_EQNE_rule, // 777
  _Binary_cmpOp_flagsReg_long_EQNE_rule, // 778
  cmpOp_commute_rule, // 779
  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule, // 780
  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule, // 781
  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule, // 782
  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule, // 783
  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule, // 784
  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule, // 785
  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule, // 786
  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule, // 787
  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule, // 788
  0, // 789
  0, // 790
  0, // 791
  0, // 792
  0, // 793
  0, // 794
  eRegP_no_EBP_rule, // 795
  eRegP_no_EBP_rule, // 796
  0, // 797
  0, // 798
  0, // 799
  0, // 800
  0, // 801
  // last instruction
  0 // no trailing comma
};

const        int   rightOp[] = {
  0, // 0
  0, // 1
  0, // 2
  0, // 3
  0, // 4
  0, // 5
  0, // 6
  0, // 7
  0, // 8
  0, // 9
  0, // 10
  0, // 11
  0, // 12
  0, // 13
  0, // 14
  0, // 15
  0, // 16
  0, // 17
  0, // 18
  0, // 19
  0, // 20
  0, // 21
  0, // 22
  0, // 23
  0, // 24
  0, // 25
  0, // 26
  0, // 27
  0, // 28
  0, // 29
  0, // 30
  0, // 31
  0, // 32
  0, // 33
  0, // 34
  0, // 35
  0, // 36
  0, // 37
  0, // 38
  0, // 39
  0, // 40
  0, // 41
  0, // 42
  0, // 43
  0, // 44
  0, // 45
  0, // 46
  0, // 47
  0, // 48
  0, // 49
  0, // 50
  0, // 51
  0, // 52
  0, // 53
  0, // 54
  0, // 55
  0, // 56
  0, // 57
  0, // 58
  0, // 59
  0, // 60
  0, // 61
  0, // 62
  0, // 63
  0, // 64
  0, // 65
  0, // 66
  0, // 67
  0, // 68
  0, // 69
  0, // 70
  0, // 71
  0, // 72
  0, // 73
  0, // 74
  0, // 75
  0, // 76
  0, // 77
  0, // 78
  0, // 79
  0, // 80
  0, // 81
  0, // 82
  0, // 83
  0, // 84
  0, // 85
  immI8_rule, // 86
  immI_rule, // 87
  eRegI_rule, // 88
  immI_rule, // 89
  eRegI_rule, // 90
  _LShiftI_eRegI_immI2_rule, // 91
  immI_rule, // 92
  0, // 93
  0, // 94
  immI_rule, // 95
  0, // 96
  0, // 97
  0, // 98
  0, // 99
  0, // 100
  0, // 101
  immI8_rule, // 102
  immI_rule, // 103
  immI_rule, // 104
  _LShiftI_eRegI_immI2_rule, // 105
  immI_rule, // 106
  0, // 107
  0, // 108
  0, // 109
  0, // 110
  0, // 111
  0, // 112
  // last operand
  0, // 113
  0, // 114
  0, // 115
  // last operand class
  eRegI_rule, // 116
  immI2_rule, // 117
  _LShiftI_eRegI_immI2_rule, // 118
  eRegI_rule, // 119
  _LShiftI_eRegI_immI2_rule, // 120
  0, // 121
  0, // 122
  0, // 123
  0, // 124
  eFlagsReg_rule, // 125
  eRegI_rule, // 126
  eFlagsRegU_rule, // 127
  eFlagsRegUCF_rule, // 128
  0, // 129
  _LoadI_memory__rule, // 130
  eRegP_rule, // 131
  eFlagsRegU_rule, // 132
  regD_rule, // 133
  regF_rule, // 134
  regD_rule, // 135
  regF_rule, // 136
  regX_rule, // 137
  regXD_rule, // 138
  eRegL_rule, // 139
  eRegI_rule, // 140
  _LoadI_memory__rule, // 141
  immI_rule, // 142
  immI1_rule, // 143
  immI_M1_rule, // 144
  eRegP_rule, // 145
  eRegI_rule, // 146
  eBCXRegL_rule, // 147
  eCXRegP_rule, // 148
  eCXRegI_rule, // 149
  eRegI_rule, // 150
  eRegI_rule, // 151
  0, // 152
  eADXRegL_low_only_rule, // 153
  immI_32_rule, // 154
  _ConvI2L_nadxRegI__rule, // 155
  immI_32_rule, // 156
  immI_32_63_rule, // 157
  immI_32_63_rule, // 158
  0, // 159
  immL_32bits_rule, // 160
  immL_32bits_rule, // 161
  immI1_rule, // 162
  immI8_rule, // 163
  immI_24_rule, // 164
  immI_16_rule, // 165
  eRegI_rule, // 166
  _LoadI_memory__rule, // 167
  immI_rule, // 168
  0, // 169
  eRegI_rule, // 170
  _LoadI_memory__rule, // 171
  immI_rule, // 172
  immI1_rule, // 173
  immI_M1_rule, // 174
  immI8_rule, // 175
  immI8_rule, // 176
  eCXRegI_rule, // 177
  eCXRegI_rule, // 178
  _SubI_immI0_eCXRegI_rule, // 179
  eCXRegI_rule, // 180
  _SubI_immI_32_eCXRegI_rule, // 181
  immI1_rule, // 182
  immI_M1_rule, // 183
  eCXRegI_rule, // 184
  _SubI_immI0_eCXRegI_rule, // 185
  _SubI_immI_32_eCXRegI_rule, // 186
  eRegI_rule, // 187
  _LoadI_memory__rule, // 188
  immI_rule, // 189
  ncxRegI_rule, // 190
  ncxRegI_rule, // 191
  ncxRegI_rule, // 192
  _CmpLTMask_ncxRegI_ncxRegI_rule, // 193
  0, // 194
  0, // 195
  regD_rule, // 196
  regD_rule, // 197
  regD_rule, // 198
  0, // 199
  _LoadD_memory__rule, // 200
  0, // 201
  immD_rule, // 202
  regD_rule, // 203
  regD_rule, // 204
  0, // 205
  0, // 206
  0, // 207
  0, // 208
  0, // 209
  regF_rule, // 210
  _LoadF_memory__rule, // 211
  regF_rule, // 212
  regF_rule, // 213
  0, // 214
  immI_rule, // 215
  memory_rule, // 216
  0, // 217
  eAXRegP_rule, // 218
  flagsReg_long_LTGE_rule, // 219
  _LoadL_load_long_memory__rule, // 220
  flagsReg_long_EQNE_rule, // 221
  flagsReg_long_LEGT_rule, // 222
  // last internally defined operand
  0, // 223
  0, // 224
  0, // 225
  0, // 226
  0, // 227
  0, // 228
  0, // 229
  0, // 230
  0, // 231
  0, // 232
  0, // 233
  0, // 234
  0, // 235
  0, // 236
  0, // 237
  0, // 238
  0, // 239
  0, // 240
  0, // 241
  0, // 242
  0, // 243
  0, // 244
  0, // 245
  0, // 246
  0, // 247
  0, // 248
  0, // 249
  0, // 250
  0, // 251
  0, // 252
  _Binary_eAXRegP_eRegP_rule, // 253
  _Binary_eAXRegI_eRegI_rule, // 254
  _Binary_eADXRegL_eBCXRegL_rule, // 255
  0, // 256
  0, // 257
  regD_rule, // 258
  regD_rule, // 259
  regD_rule, // 260
  regXD_rule, // 261
  regXD_rule, // 262
  _LoadD_memory__rule, // 263
  _LoadD_memory__rule, // 264
  0, // 265
  0, // 266
  0, // 267
  0, // 268
  regF_rule, // 269
  regF_rule, // 270
  regF_rule, // 271
  regX_rule, // 272
  regX_rule, // 273
  _LoadF_memory__rule, // 274
  _LoadF_memory__rule, // 275
  0, // 276
  0, // 277
  0, // 278
  0, // 279
  0, // 280
  0, // 281
  0, // 282
  0, // 283
  0, // 284
  0, // 285
  0, // 286
  0, // 287
  0, // 288
  eRegI_rule, // 289
  immI_rule, // 290
  _LoadI_memory__rule, // 291
  immI0_rule, // 292
  immI0_rule, // 293
  immI0_rule, // 294
  eRegI_rule, // 295
  immI_rule, // 296
  _LoadI_memory__rule, // 297
  immI0_rule, // 298
  eRegP_rule, // 299
  immP_rule, // 300
  _LoadP_memory__rule, // 301
  _LoadP_memory__rule, // 302
  immP0_rule, // 303
  immI0_rule, // 304
  immP0_rule, // 305
  immL0_rule, // 306
  eRegL_rule, // 307
  immL0_rule, // 308
  eRegL_rule, // 309
  immL0_rule, // 310
  eRegL_rule, // 311
  eRegP_rule, // 312
  eAXRegP_rule, // 313
  0, // 314
  0, // 315
  0, // 316
  0, // 317
  immI_255_rule, // 318
  0, // 319
  0, // 320
  0, // 321
  0, // 322
  0, // 323
  0, // 324
  0, // 325
  0, // 326
  0, // 327
  0, // 328
  0, // 329
  0, // 330
  0, // 331
  0, // 332
  0, // 333
  0, // 334
  0, // 335
  0, // 336
  0, // 337
  0, // 338
  0, // 339
  0, // 340
  0, // 341
  0, // 342
  0, // 343
  0, // 344
  0, // 345
  0, // 346
  0, // 347
  0, // 348
  xRegI_rule, // 349
  eRegI_rule, // 350
  eRegI_rule, // 351
  eRegL_rule, // 352
  stackSlotL_rule, // 353
  stackSlotL_rule, // 354
  eRegL_rule, // 355
  anyRegP_rule, // 356
  immI_rule, // 357
  immI16_rule, // 358
  immP_rule, // 359
  immI8_rule, // 360
  regXD_rule, // 361
  regXD_rule, // 362
  regXD_rule, // 363
  immI8_rule, // 364
  regDPR1_rule, // 365
  _RoundDouble_regDPR1__rule, // 366
  regXD_rule, // 367
  regX_rule, // 368
  regXD_rule, // 369
  regFPR1_rule, // 370
  _RoundFloat_regFPR1__rule, // 371
  _ConvD2F_regDPR1__rule, // 372
  immF_rule, // 373
  immXF_rule, // 374
  0, // 375
  0, // 376
  0, // 377
  0, // 378
  0, // 379
  0, // 380
  _Binary_eRegI_eRegI_rule, // 381
  _Binary_eRegI_eRegI_rule, // 382
  _Binary_eRegI_eRegI_rule, // 383
  _Binary_eRegI__LoadI_memory__rule, // 384
  _Binary_eRegI__LoadI_memory__rule, // 385
  _Binary_eRegI__LoadI_memory__rule, // 386
  _Binary_eRegP_eRegP_rule, // 387
  _Binary_eRegP_eRegP_rule, // 388
  _Binary_eRegP_eRegP_rule, // 389
  _Binary_eRegP_eRegP_rule, // 390
  _Binary_regDPR1_regD_rule, // 391
  _Binary_regFPR1_regF_rule, // 392
  _Binary_regD_regD_rule, // 393
  _Binary_regF_regF_rule, // 394
  _Binary_regX_regX_rule, // 395
  _Binary_regXD_regXD_rule, // 396
  _Binary_regX_regX_rule, // 397
  _Binary_regX_regX_rule, // 398
  _Binary_regXD_regXD_rule, // 399
  _Binary_regXD_regXD_rule, // 400
  _Binary_eRegL_eRegL_rule, // 401
  _Binary_eRegL_eRegL_rule, // 402
  _Binary_eRegL_eRegL_rule, // 403
  eRegI_rule, // 404
  immI_rule, // 405
  immI1_rule, // 406
  immI_rule, // 407
  immI_rule, // 408
  immI_M1_rule, // 409
  eRegI_rule, // 410
  immI_rule, // 411
  _LoadI_memory__rule, // 412
  eRegI_rule, // 413
  _AddI__LoadI_memory__eRegI_rule, // 414
  _AddI_eRegI__LoadI_memory__rule, // 415
  _AddI__LoadI_memory__immI_rule, // 416
  _AddI__LoadI_memory__immI1_rule, // 417
  _AddI__LoadI_memory__immI_M1_rule, // 418
  0, // 419
  0, // 420
  0, // 421
  0, // 422
  _Binary_eADXRegL_eBCXRegL_rule, // 423
  _Binary_eAXRegP_eCXRegP_rule, // 424
  _Binary_eAXRegI_eCXRegI_rule, // 425
  eRegI_rule, // 426
  immI_rule, // 427
  _LoadI_memory__rule, // 428
  _SubI__LoadI_memory__eRegI_rule, // 429
  _SubI_immI0_eRegI_rule, // 430
  eRegI_rule, // 431
  eRegI_rule, // 432
  immI_rule, // 433
  0, // 434
  0, // 435
  0, // 436
  0, // 437
  immI_rule, // 438
  _LoadI_memory__rule, // 439
  eRegI_rule, // 440
  _ConvI2L_nadxRegI__rule, // 441
  _ConvI2L_eAXRegI__rule, // 442
  _AndL__ConvI2L_nadxRegI__immL_32bits_rule, // 443
  _AndL__ConvI2L_eAXRegI__immL_32bits_rule, // 444
  eRegL_rule, // 445
  immL_127_rule, // 446
  eCXRegI_rule, // 447
  eRegL_rule, // 448
  eCXRegI_rule, // 449
  eCXRegI_rule, // 450
  eRegL_rule, // 451
  immI1_rule, // 452
  immI8_rule, // 453
  eCXRegI_rule, // 454
  immI1_rule, // 455
  _RShiftI__LoadI_memory__immI1_rule, // 456
  immI8_rule, // 457
  _RShiftI__LoadI_memory__immI8_rule, // 458
  eCXRegI_rule, // 459
  immI1_rule, // 460
  immI8_rule, // 461
  immI_24_rule, // 462
  immI_16_rule, // 463
  eCXRegI_rule, // 464
  eRegI_rule, // 465
  immI_rule, // 466
  _LoadI_memory__rule, // 467
  eRegI_rule, // 468
  _AndI__LoadI_memory__eRegI_rule, // 469
  _AndI_eRegI__LoadI_memory__rule, // 470
  _AndI__LoadI_memory__immI_rule, // 471
  eRegI_rule, // 472
  _CastP2X_eRegP__rule, // 473
  eRegI_rule, // 474
  immI_rule, // 475
  _LoadI_memory__rule, // 476
  eRegI_rule, // 477
  _OrI__LoadI_memory__eRegI_rule, // 478
  _OrI_eRegI__LoadI_memory__rule, // 479
  _OrI__LoadI_memory__immI_rule, // 480
  0, // 481
  0, // 482
  0, // 483
  _URShiftI_eRegI_immI_M1_rule, // 484
  _LShiftI_eRegI_immI1_rule, // 485
  _URShiftI_eRegI_immI8_rule, // 486
  _LShiftI_eRegI_immI8_rule, // 487
  _URShiftI_ncxRegI__SubI_immI0_eCXRegI_rule, // 488
  _LShiftI_ncxRegI_eCXRegI_rule, // 489
  _URShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule, // 490
  _LShiftI_ncxRegI_eCXRegI_rule, // 491
  0, // 492
  0, // 493
  0, // 494
  _LShiftI_eRegI_immI_M1_rule, // 495
  _URShiftI_eRegI_immI1_rule, // 496
  _LShiftI_eRegI_immI8_rule, // 497
  _URShiftI_eRegI_immI8_rule, // 498
  _LShiftI_ncxRegI__SubI_immI0_eCXRegI_rule, // 499
  _URShiftI_ncxRegI_eCXRegI_rule, // 500
  _LShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule, // 501
  _URShiftI_ncxRegI_eCXRegI_rule, // 502
  eRegI_rule, // 503
  immI_M1_rule, // 504
  immI_rule, // 505
  _LoadI_memory__rule, // 506
  eRegI_rule, // 507
  _XorI__LoadI_memory__eRegI_rule, // 508
  _XorI_eRegI__LoadI_memory__rule, // 509
  _XorI__LoadI_memory__immI_rule, // 510
  0, // 511
  0, // 512
  0, // 513
  0, // 514
  ncxRegI_rule, // 515
  immI0_rule, // 516
  _SubI_ncxRegI_ncxRegI_rule, // 517
  _AndI__CmpLTMask_ncxRegI_ncxRegI_ncxRegI_rule, // 518
  _SubI_ncxRegI_ncxRegI_rule, // 519
  _AndI_ncxRegI__CmpLTMask_ncxRegI_ncxRegI_rule, // 520
  eRegL_rule, // 521
  immL_rule, // 522
  _LoadL_load_long_memory__rule, // 523
  eRegL_rule, // 524
  eRegL_rule, // 525
  immL_rule, // 526
  _LoadL_load_long_memory__rule, // 527
  eRegL_rule, // 528
  eRegL_rule, // 529
  immL_rule, // 530
  _LoadL_load_long_memory__rule, // 531
  eRegL_rule, // 532
  eRegL_rule, // 533
  immL_rule, // 534
  _LoadL_load_long_memory__rule, // 535
  eRegL_rule, // 536
  eRegL_rule, // 537
  immL_M1_rule, // 538
  immL_rule, // 539
  _LoadL_load_long_memory__rule, // 540
  eRegL_rule, // 541
  immI_1_rule, // 542
  immI_2_rule, // 543
  immI_3_rule, // 544
  immI_1_31_rule, // 545
  immI_32_63_rule, // 546
  eCXRegI_rule, // 547
  immI_1_31_rule, // 548
  immI_32_63_rule, // 549
  eCXRegI_rule, // 550
  immI_1_31_rule, // 551
  immI_32_63_rule, // 552
  eCXRegI_rule, // 553
  immD0_rule, // 554
  regD_rule, // 555
  regXD_rule, // 556
  _LoadD_memory__rule, // 557
  regD_rule, // 558
  0, // 559
  _LoadD_memory__rule, // 560
  regD_rule, // 561
  0, // 562
  _LoadD_memory__rule, // 563
  regD_rule, // 564
  _RoundDouble__AddD__LoadD_memory__regD__rule, // 565
  _RoundDouble__AddD_regD__LoadD_memory___rule, // 566
  immD1_rule, // 567
  immD_rule, // 568
  0, // 569
  regXD_rule, // 570
  immXD_rule, // 571
  _LoadD_memory__rule, // 572
  regXD_rule, // 573
  regXD_rule, // 574
  immXD_rule, // 575
  _LoadD_memory__rule, // 576
  regXD_rule, // 577
  immXD_rule, // 578
  _LoadD_memory__rule, // 579
  regXD_rule, // 580
  regXD_rule, // 581
  immXD_rule, // 582
  _LoadD_memory__rule, // 583
  regD_rule, // 584
  regnotDPR1_rule, // 585
  immD_rule, // 586
  _LoadD_memory__rule, // 587
  regD_rule, // 588
  _LoadD_memory__rule, // 589
  regD_rule, // 590
  regD_rule, // 591
  _MulD_regD_regD_rule, // 592
  regD_rule, // 593
  regD_rule, // 594
  regnotDPR1_rule, // 595
  0, // 596
  regD_rule, // 597
  regXD_rule, // 598
  0, // 599
  0, // 600
  0, // 601
  0, // 602
  0, // 603
  0, // 604
  regD_rule, // 605
  regXD_rule, // 606
  0, // 607
  regDPR1_rule, // 608
  regXD_rule, // 609
  0, // 610
  0, // 611
  0, // 612
  0, // 613
  0, // 614
  0, // 615
  immF0_rule, // 616
  regF_rule, // 617
  regX_rule, // 618
  _LoadF_memory__rule, // 619
  regF_rule, // 620
  regF_rule, // 621
  regF_rule, // 622
  regF_rule, // 623
  regX_rule, // 624
  immXF_rule, // 625
  _LoadF_memory__rule, // 626
  regX_rule, // 627
  regX_rule, // 628
  immXF_rule, // 629
  _LoadF_memory__rule, // 630
  regX_rule, // 631
  immXF_rule, // 632
  _LoadF_memory__rule, // 633
  regX_rule, // 634
  regX_rule, // 635
  immXF_rule, // 636
  _LoadF_memory__rule, // 637
  0, // 638
  0, // 639
  0, // 640
  0, // 641
  _LoadF_memory__rule, // 642
  regF_rule, // 643
  _LoadF_memory__rule, // 644
  regF_rule, // 645
  regF_rule, // 646
  _LoadF_memory__rule, // 647
  memory_rule, // 648
  memory_rule, // 649
  immF_rule, // 650
  immF_rule, // 651
  regF_rule, // 652
  regF_rule, // 653
  _LoadF_memory__rule, // 654
  regF_rule, // 655
  _LoadF_memory__rule, // 656
  regF_rule, // 657
  memory_rule, // 658
  immF_rule, // 659
  immF_rule, // 660
  regF_rule, // 661
  _LoadF_memory__rule, // 662
  regF_rule, // 663
  _MulF__LoadF_memory__regF_rule, // 664
  regF_rule, // 665
  _MulF_regF__LoadF_memory__rule, // 666
  regF_rule, // 667
  _MulF_regF_regF_rule, // 668
  regF_rule, // 669
  regF_rule, // 670
  regF_rule, // 671
  regF_rule, // 672
  regF_rule, // 673
  regX_rule, // 674
  0, // 675
  0, // 676
  0, // 677
  0, // 678
  0, // 679
  0, // 680
  0, // 681
  0, // 682
  0, // 683
  0, // 684
  0, // 685
  0, // 686
  0, // 687
  0, // 688
  0, // 689
  0, // 690
  0, // 691
  0, // 692
  0, // 693
  0, // 694
  0, // 695
  0, // 696
  0, // 697
  0, // 698
  0, // 699
  0, // 700
  0, // 701
  0, // 702
  0, // 703
  0, // 704
  immL_32bits_rule, // 705
  immL_32bits_rule, // 706
  0, // 707
  0, // 708
  0, // 709
  0, // 710
  0, // 711
  0, // 712
  0, // 713
  0, // 714
  0, // 715
  0, // 716
  0, // 717
  0, // 718
  0, // 719
  0, // 720
  0, // 721
  0, // 722
  0, // 723
  0, // 724
  0, // 725
  0, // 726
  0, // 727
  0, // 728
  0, // 729
  0, // 730
  0, // 731
  0, // 732
  0, // 733
  0, // 734
  eDIRegP_rule, // 735
  eSIRegP_rule, // 736
  eSIRegP_rule, // 737
  eRegI_rule, // 738
  eRegI_rule, // 739
  0, // 740
  0, // 741
  eFlagsReg_rule, // 742
  eFlagsReg_rule, // 743
  eFlagsRegU_rule, // 744
  eFlagsRegUCF_rule, // 745
  eFlagsRegU_rule, // 746
  eFlagsRegUCF_rule, // 747
  eFlagsRegUCF_rule, // 748
  eAXRegP_rule, // 749
  0, // 750
  eFlagsReg_rule, // 751
  eFlagsReg_rule, // 752
  eFlagsRegU_rule, // 753
  eFlagsRegUCF_rule, // 754
  eFlagsRegU_rule, // 755
  eFlagsRegUCF_rule, // 756
  eFlagsRegUCF_rule, // 757
  eRegL_rule, // 758
  flagsReg_long_LTGE_rule, // 759
  _Binary_eRegL_eRegL_rule, // 760
  _Binary_eRegL__LoadL_load_long_memory__rule, // 761
  _Binary_eRegI_eRegI_rule, // 762
  _Binary_eRegI__LoadI_memory__rule, // 763
  _Binary_eRegP_eRegP_rule, // 764
  _Binary_regD_regD_rule, // 765
  _Binary_regXD_regXD_rule, // 766
  _Binary_regF_regF_rule, // 767
  _Binary_regX_regX_rule, // 768
  flagsReg_long_EQNE_rule, // 769
  _Binary_eRegL_eRegL_rule, // 770
  _Binary_eRegL__LoadL_load_long_memory__rule, // 771
  _Binary_eRegI_eRegI_rule, // 772
  _Binary_eRegI__LoadI_memory__rule, // 773
  _Binary_eRegP_eRegP_rule, // 774
  _Binary_regD_regD_rule, // 775
  _Binary_regXD_regXD_rule, // 776
  _Binary_regF_regF_rule, // 777
  _Binary_regX_regX_rule, // 778
  flagsReg_long_LEGT_rule, // 779
  _Binary_eRegL_eRegL_rule, // 780
  _Binary_eRegL__LoadL_load_long_memory__rule, // 781
  _Binary_eRegI_eRegI_rule, // 782
  _Binary_eRegI__LoadI_memory__rule, // 783
  _Binary_eRegP_eRegP_rule, // 784
  _Binary_regD_regD_rule, // 785
  _Binary_regXD_regXD_rule, // 786
  _Binary_regF_regF_rule, // 787
  _Binary_regX_regX_rule, // 788
  0, // 789
  0, // 790
  0, // 791
  0, // 792
  0, // 793
  0, // 794
  eBXRegP_rule, // 795
  eAXRegP_rule, // 796
  0, // 797
  0, // 798
  0, // 799
  0, // 800
  0, // 801
  // last instruction
  0 // no trailing comma
};

const char        *ruleName[] = {
  "UNIVERSE", // 0
  "LABEL", // 1
  "SREGI", // 2
  "SREGP", // 3
  "SREGF", // 4
  "SREGD", // 5
  "SREGL", // 6
  "METHOD", // 7
  "IMMI", // 8
  "IMMI0", // 9
  "IMMI1", // 10
  "IMMI_M1", // 11
  "IMMI2", // 12
  "IMMI8", // 13
  "IMMI16", // 14
  "IMMI_32", // 15
  "IMMI_1_31", // 16
  "IMMI_32_63", // 17
  "IMMI_1", // 18
  "IMMI_2", // 19
  "IMMI_3", // 20
  "IMMP", // 21
  "IMMP0", // 22
  "IMML", // 23
  "IMML0", // 24
  "IMML_M1", // 25
  "IMML_127", // 26
  "IMML_32BITS", // 27
  "IMML32", // 28
  "IMMD0", // 29
  "IMMD1", // 30
  "IMMD", // 31
  "IMMXD", // 32
  "IMMXD0", // 33
  "IMMF0", // 34
  "IMMF", // 35
  "IMMXF", // 36
  "IMMXF0", // 37
  "IMMI_16", // 38
  "IMMI_24", // 39
  "IMMI_255", // 40
  "EREGI", // 41
  "XREGI", // 42
  "EAXREGI", // 43
  "EBXREGI", // 44
  "ECXREGI", // 45
  "EDXREGI", // 46
  "EDIREGI", // 47
  "NAXREGI", // 48
  "NADXREGI", // 49
  "NCXREGI", // 50
  "ESIREGI", // 51
  "ANYREGP", // 52
  "EREGP", // 53
  "EREGP_NO_EBP", // 54
  "NAXREGP", // 55
  "NABXREGP", // 56
  "PREGP", // 57
  "EAXREGP", // 58
  "EBXREGP", // 59
  "ECXREGP", // 60
  "ESIREGP", // 61
  "EDIREGP", // 62
  "EBPREGP", // 63
  "EREGL", // 64
  "EADXREGL", // 65
  "EBCXREGL", // 66
  "EADXREGL_LOW_ONLY", // 67
  "EFLAGSREG", // 68
  "EFLAGSREGU", // 69
  "EFLAGSREGUCF", // 70
  "FLAGSREG_LONG_LTGE", // 71
  "FLAGSREG_LONG_EQNE", // 72
  "FLAGSREG_LONG_LEGT", // 73
  "REGD", // 74
  "REGDPR1", // 75
  "REGDPR2", // 76
  "REGNOTDPR1", // 77
  "REGXD", // 78
  "REGXD6", // 79
  "REGXD7", // 80
  "REGF", // 81
  "REGFPR1", // 82
  "REGX", // 83
  "DIRECT", // 84
  "INDIRECT", // 85
  "INDOFFSET8", // 86
  "INDOFFSET32", // 87
  "INDOFFSET32X", // 88
  "INDINDEXOFFSET", // 89
  "INDINDEX", // 90
  "INDINDEXSCALE", // 91
  "INDINDEXSCALEOFFSET", // 92
  "LOAD_LONG_REGP", // 93
  "LOAD_LONG_INDIRECT", // 94
  "LOAD_LONG_INDOFFSET32", // 95
  "STACKSLOTP", // 96
  "STACKSLOTI", // 97
  "STACKSLOTF", // 98
  "STACKSLOTD", // 99
  "STACKSLOTL", // 100
  "INDIRECT_WIN95_SAFE", // 101
  "INDOFFSET8_WIN95_SAFE", // 102
  "INDOFFSET32_WIN95_SAFE", // 103
  "INDINDEXOFFSET_WIN95_SAFE", // 104
  "INDINDEXSCALE_WIN95_SAFE", // 105
  "INDINDEXSCALEOFFSET_WIN95_SAFE", // 106
  "CMPOP", // 107
  "CMPOPU", // 108
  "CMPOPUCF", // 109
  "CMPOPUCF2", // 110
  "CMPOP_FCMOV", // 111
  "CMPOP_COMMUTE", // 112
  // last operand
  "LOAD_LONG_MEMORY", // 113
  "MEMORY", // 114
  "LONG_MEMORY", // 115
  // last operand class
  "_AddP_eRegP_eRegI", // 116
  "_LShiftI_eRegI_immI2", // 117
  "_AddP_eRegP__LShiftI_eRegI_immI2", // 118
  "_AddP_eRegP_no_EBP_eRegI", // 119
  "_AddP_eRegP_no_EBP__LShiftI_eRegI_immI2", // 120
  "_LoadB_memory_", // 121
  "_RoundDouble_regDPR1_", // 122
  "_RoundFloat_regFPR1_", // 123
  "_ConvD2F_regDPR1_", // 124
  "_Binary_cmpOp_eFlagsReg", // 125
  "_Binary_eRegI_eRegI", // 126
  "_Binary_cmpOpU_eFlagsRegU", // 127
  "_Binary_cmpOpUCF_eFlagsRegUCF", // 128
  "_LoadI_memory_", // 129
  "_Binary_eRegI__LoadI_memory_", // 130
  "_Binary_eRegP_eRegP", // 131
  "_Binary_cmpOp_fcmov_eFlagsRegU", // 132
  "_Binary_regDPR1_regD", // 133
  "_Binary_regFPR1_regF", // 134
  "_Binary_regD_regD", // 135
  "_Binary_regF_regF", // 136
  "_Binary_regX_regX", // 137
  "_Binary_regXD_regXD", // 138
  "_Binary_eRegL_eRegL", // 139
  "_AddI__LoadI_memory__eRegI", // 140
  "_AddI_eRegI__LoadI_memory_", // 141
  "_AddI__LoadI_memory__immI", // 142
  "_AddI__LoadI_memory__immI1", // 143
  "_AddI__LoadI_memory__immI_M1", // 144
  "_Binary_eAXRegP_eRegP", // 145
  "_Binary_eAXRegI_eRegI", // 146
  "_Binary_eADXRegL_eBCXRegL", // 147
  "_Binary_eAXRegP_eCXRegP", // 148
  "_Binary_eAXRegI_eCXRegI", // 149
  "_SubI__LoadI_memory__eRegI", // 150
  "_SubI_immI0_eRegI", // 151
  "_ConvI2L_nadxRegI_", // 152
  "_MulL__ConvI2L_nadxRegI__eADXRegL_low_only", // 153
  "_RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32", // 154
  "_MulL_eADXRegL_low_only__ConvI2L_nadxRegI_", // 155
  "_RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32", // 156
  "_RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_63", // 157
  "_RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_63", // 158
  "_ConvI2L_eAXRegI_", // 159
  "_AndL__ConvI2L_eAXRegI__immL_32bits", // 160
  "_AndL__ConvI2L_nadxRegI__immL_32bits", // 161
  "_RShiftI__LoadI_memory__immI1", // 162
  "_RShiftI__LoadI_memory__immI8", // 163
  "_LShiftI_xRegI_immI_24", // 164
  "_LShiftI_xRegI_immI_16", // 165
  "_AndI__LoadI_memory__eRegI", // 166
  "_AndI_eRegI__LoadI_memory_", // 167
  "_AndI__LoadI_memory__immI", // 168
  "_CastP2X_eRegP_", // 169
  "_OrI__LoadI_memory__eRegI", // 170
  "_OrI_eRegI__LoadI_memory_", // 171
  "_OrI__LoadI_memory__immI", // 172
  "_LShiftI_eRegI_immI1", // 173
  "_URShiftI_eRegI_immI_M1", // 174
  "_LShiftI_eRegI_immI8", // 175
  "_URShiftI_eRegI_immI8", // 176
  "_LShiftI_ncxRegI_eCXRegI", // 177
  "_SubI_immI0_eCXRegI", // 178
  "_URShiftI_ncxRegI__SubI_immI0_eCXRegI", // 179
  "_SubI_immI_32_eCXRegI", // 180
  "_URShiftI_ncxRegI__SubI_immI_32_eCXRegI", // 181
  "_URShiftI_eRegI_immI1", // 182
  "_LShiftI_eRegI_immI_M1", // 183
  "_URShiftI_ncxRegI_eCXRegI", // 184
  "_LShiftI_ncxRegI__SubI_immI0_eCXRegI", // 185
  "_LShiftI_ncxRegI__SubI_immI_32_eCXRegI", // 186
  "_XorI__LoadI_memory__eRegI", // 187
  "_XorI_eRegI__LoadI_memory_", // 188
  "_XorI__LoadI_memory__immI", // 189
  "_CmpLTMask_ncxRegI_ncxRegI", // 190
  "_AndI__CmpLTMask_ncxRegI_ncxRegI_ncxRegI", // 191
  "_SubI_ncxRegI_ncxRegI", // 192
  "_AndI_ncxRegI__CmpLTMask_ncxRegI_ncxRegI", // 193
  "_LoadL_load_long_memory_", // 194
  "_LoadD_memory_", // 195
  "_SubD_regD_regD", // 196
  "_AddD_regD_regD", // 197
  "_AddD__LoadD_memory__regD", // 198
  "_RoundDouble__AddD__LoadD_memory__regD_", // 199
  "_AddD_regD__LoadD_memory_", // 200
  "_RoundDouble__AddD_regD__LoadD_memory__", // 201
  "_AddD_regD_immD", // 202
  "_MulD_regD_regD", // 203
  "_DivD_regD_regD", // 204
  "_LoadF_memory_", // 205
  "_ConvF2D_regX_", // 206
  "_SqrtD__ConvF2D_regX__", // 207
  "_ConvF2D__LoadF_memory__", // 208
  "_SqrtD__ConvF2D__LoadF_memory___", // 209
  "_MulF__LoadF_memory__regF", // 210
  "_MulF_regF__LoadF_memory_", // 211
  "_MulF_regF_regF", // 212
  "_SubF_regF_regF", // 213
  "_ConvI2L_eRegI_", // 214
  "_AndI_eRegI_immI", // 215
  "_AndI_eRegI_memory", // 216
  "_LoadP_memory_", // 217
  "_PartialSubtypeCheck_eSIRegP_eAXRegP", // 218
  "_Binary_cmpOp_flagsReg_long_LTGE", // 219
  "_Binary_eRegL__LoadL_load_long_memory_", // 220
  "_Binary_cmpOp_flagsReg_long_EQNE", // 221
  "_Binary_cmpOp_commute_flagsReg_long_LEGT", // 222
  // last internally defined operand
  "storeSSI", // 223
  "storeSSP", // 224
  "storeSSL", // 225
  "leaP8", // 226
  "leaP32", // 227
  "leaPIdxOff", // 228
  "leaPIdxScale", // 229
  "leaPIdxScaleOff", // 230
  "loadConI", // 231
  "loadConI0", // 232
  "loadConP", // 233
  "loadConL", // 234
  "loadConL0", // 235
  "loadConF", // 236
  "loadConX", // 237
  "loadConX0", // 238
  "loadConD", // 239
  "loadConXD", // 240
  "loadConXD0", // 241
  "loadSSI", // 242
  "loadSSL", // 243
  "loadSSP", // 244
  "loadSSF", // 245
  "loadSSD", // 246
  "loadConL_low_only", // 247
  "castX2P", // 248
  "castP2X", // 249
  "checkCastPP", // 250
  "castPP", // 251
  "castII", // 252
  "storePConditional", // 253
  "storeIConditional", // 254
  "storeLConditional", // 255
  "movI_nocopy", // 256
  "movP_nocopy", // 257
  "cmpD_cc_P6", // 258
  "cmpD_cc_P6CF", // 259
  "cmpD_cc", // 260
  "cmpXD_cc", // 261
  "cmpXD_ccCF", // 262
  "cmpXD_ccmem", // 263
  "cmpXD_ccmemCF", // 264
  "absD_reg", // 265
  "absXD_reg", // 266
  "negD_reg", // 267
  "negXD_reg", // 268
  "cmpF_cc_P6", // 269
  "cmpF_cc_P6CF", // 270
  "cmpF_cc", // 271
  "cmpX_cc", // 272
  "cmpX_ccCF", // 273
  "cmpX_ccmem", // 274
  "cmpX_ccmemCF", // 275
  "absF_reg", // 276
  "absX_reg", // 277
  "negF_reg", // 278
  "negX_reg", // 279
  "MoveF2I_stack_reg", // 280
  "MoveF2I_reg_reg_sse", // 281
  "MoveI2F_stack_reg", // 282
  "MoveI2F_stack_reg_sse", // 283
  "MoveI2F_reg_reg_sse", // 284
  "MoveD2L_stack_reg", // 285
  "MoveL2D_stack_reg", // 286
  "MoveL2D_stack_reg_sse", // 287
  "MoveL2D_stack_reg_sse_partial", // 288
  "compI_eReg", // 289
  "compI_eReg_imm", // 290
  "compI_eReg_mem", // 291
  "testI_reg", // 292
  "testI_reg_imm", // 293
  "testI_reg_mem", // 294
  "compU_eReg", // 295
  "compU_eReg_imm", // 296
  "compU_eReg_mem", // 297
  "testU_reg", // 298
  "compP_eReg", // 299
  "compP_eReg_imm", // 300
  "compP_eReg_mem", // 301
  "compP_mem_eReg", // 302
  "testP_reg", // 303
  "testP_Reg_mem", // 304
  "partialSubtypeCheck_vs_Zero", // 305
  "cmpL_zero_flags_LTGE", // 306
  "cmpL_reg_flags_LTGE", // 307
  "cmpL_zero_flags_EQNE", // 308
  "cmpL_reg_flags_EQNE", // 309
  "cmpL_zero_flags_LEGT", // 310
  "cmpL_reg_flags_LEGT", // 311
  "cmpFastLock", // 312
  "cmpFastUnlock", // 313
  "safePoint_poll", // 314
  "bytes_reverse_int", // 315
  "bytes_reverse_long", // 316
  "loadB", // 317
  "loadUB", // 318
  "loadC", // 319
  "loadI", // 320
  "loadL", // 321
  "loadL_volatile", // 322
  "loadLX_volatile", // 323
  "loadLX_reg_volatile", // 324
  "loadRange", // 325
  "loadP", // 326
  "loadKlass", // 327
  "loadS", // 328
  "loadD", // 329
  "loadXD", // 330
  "loadXD_partial", // 331
  "loadX", // 332
  "loadF", // 333
  "loadA8B", // 334
  "loadA4S", // 335
  "loadA4C", // 336
  "load2IU", // 337
  "loadA2F", // 338
  "prefetchr0", // 339
  "prefetchr", // 340
  "prefetchrNTA", // 341
  "prefetchrT0", // 342
  "prefetchrT2", // 343
  "prefetchw0", // 344
  "prefetchw", // 345
  "prefetchwNTA", // 346
  "prefetchwT0", // 347
  "prefetchwT2", // 348
  "storeB", // 349
  "storeC", // 350
  "storeI", // 351
  "storeL", // 352
  "storeL_volatile", // 353
  "storeLX_volatile", // 354
  "storeLX_reg_volatile", // 355
  "storeP", // 356
  "storeImmI", // 357
  "storeImmI16", // 358
  "storeImmP", // 359
  "storeImmB", // 360
  "storeA8B", // 361
  "storeA4C", // 362
  "storeA2I", // 363
  "storeImmCM", // 364
  "storeD", // 365
  "storeD_rounded", // 366
  "storeXD", // 367
  "storeX", // 368
  "storeA2F", // 369
  "storeF", // 370
  "storeF_rounded", // 371
  "storeF_Drounded", // 372
  "storeF_imm", // 373
  "storeX_imm", // 374
  "membar_acquire", // 375
  "membar_acquire_lock", // 376
  "membar_release", // 377
  "membar_release_lock", // 378
  "membar_volatile", // 379
  "unnecessary_membar_volatile", // 380
  "cmovI_reg", // 381
  "cmovI_regU", // 382
  "cmovI_regUCF", // 383
  "cmovI_mem", // 384
  "cmovI_memU", // 385
  "cmovI_memUCF", // 386
  "cmovP_reg", // 387
  "cmovP_reg_nonP6", // 388
  "cmovP_regU", // 389
  "cmovP_regUCF", // 390
  "fcmovD_regU", // 391
  "fcmovF_regU", // 392
  "fcmovD_regS", // 393
  "fcmovF_regS", // 394
  "fcmovX_regS", // 395
  "fcmovXD_regS", // 396
  "fcmovX_regU", // 397
  "fcmovX_regUCF", // 398
  "fcmovXD_regU", // 399
  "fcmovXD_regUCF", // 400
  "cmovL_reg", // 401
  "cmovL_regU", // 402
  "cmovL_regUCF", // 403
  "addI_eReg", // 404
  "addI_eReg_imm", // 405
  "incI_eReg", // 406
  "leaI_eReg_immI", // 407
  "leaP_eReg_immI", // 408
  "decI_eReg", // 409
  "addP_eReg", // 410
  "addP_eReg_imm", // 411
  "addI_eReg_mem", // 412
  "addI_eReg_mem_0", // 413
  "addI_mem_eReg", // 414
  "addI_mem_eReg_0", // 415
  "addI_mem_imm", // 416
  "incI_mem", // 417
  "decI_mem", // 418
  "loadPLocked", // 419
  "loadLLocked", // 420
  "loadLX_Locked", // 421
  "loadLX_reg_Locked", // 422
  "compareAndSwapL", // 423
  "compareAndSwapP", // 424
  "compareAndSwapI", // 425
  "subI_eReg", // 426
  "subI_eReg_imm", // 427
  "subI_eReg_mem", // 428
  "subI_mem_eReg", // 429
  "subP_eReg", // 430
  "negI_eReg", // 431
  "mulI_eReg", // 432
  "mulI_eReg_imm", // 433
  "mulI_imm_high", // 434
  "mulI_imm_high_0", // 435
  "mulI_imm_RShift_high", // 436
  "mulI_imm_RShift_high_0", // 437
  "mulI_mem_imm", // 438
  "mulI", // 439
  "mulI_0", // 440
  "mulI2L", // 441
  "mulI2L_0", // 442
  "mulIS_eReg", // 443
  "mulIS_eReg_0", // 444
  "mulL_eReg", // 445
  "mulL_eReg_con", // 446
  "divI_eReg", // 447
  "divL_eReg", // 448
  "divModI_eReg_divmod", // 449
  "modI_eReg", // 450
  "modL_eReg", // 451
  "shlI_eReg_1", // 452
  "salI_eReg_imm", // 453
  "salI_eReg_CL", // 454
  "sarI_eReg_1", // 455
  "sarI_mem_1", // 456
  "sarI_eReg_imm", // 457
  "sarI_mem_imm", // 458
  "sarI_eReg_CL", // 459
  "shrI_eReg_1", // 460
  "shrI_eReg_imm", // 461
  "i2b", // 462
  "i2s", // 463
  "shrI_eReg_CL", // 464
  "andI_eReg", // 465
  "andI_eReg_imm", // 466
  "andI_eReg_mem", // 467
  "andI_eReg_mem_0", // 468
  "andI_mem_eReg", // 469
  "andI_mem_eReg_0", // 470
  "andI_mem_imm", // 471
  "orI_eReg", // 472
  "orI_eReg_castP2X", // 473
  "orI_eReg_castP2X_0", // 474
  "orI_eReg_imm", // 475
  "orI_eReg_mem", // 476
  "orI_eReg_mem_0", // 477
  "orI_mem_eReg", // 478
  "orI_mem_eReg_0", // 479
  "orI_mem_imm", // 480
  "rolI_eReg_imm1", // 481
  "rolI_eReg_imm8", // 482
  "rolI_eReg_CL", // 483
  "rolI_eReg_i1", // 484
  "rolI_eReg_i1_0", // 485
  "rolI_eReg_i8", // 486
  "rolI_eReg_i8_0", // 487
  "rolI_eReg_Var_C0", // 488
  "rolI_eReg_Var_C0_0", // 489
  "rolI_eReg_Var_C32", // 490
  "rolI_eReg_Var_C32_0", // 491
  "rorI_eReg_imm1", // 492
  "rorI_eReg_imm8", // 493
  "rorI_eReg_CL", // 494
  "rorI_eReg_i1", // 495
  "rorI_eReg_i1_0", // 496
  "rorI_eReg_i8", // 497
  "rorI_eReg_i8_0", // 498
  "rorI_eReg_Var_C0", // 499
  "rorI_eReg_Var_C0_0", // 500
  "rorI_eReg_Var_C32", // 501
  "rorI_eReg_Var_C32_0", // 502
  "xorI_eReg", // 503
  "xorI_eReg_im1", // 504
  "xorI_eReg_imm", // 505
  "xorI_eReg_mem", // 506
  "xorI_eReg_mem_0", // 507
  "xorI_mem_eReg", // 508
  "xorI_mem_eReg_0", // 509
  "xorI_mem_imm", // 510
  "ci2b", // 511
  "convI2B", // 512
  "cp2b", // 513
  "convP2B", // 514
  "cmpLTMask", // 515
  "cmpLTMask0", // 516
  "cadd_cmpLTMask", // 517
  "cadd_cmpLTMask_1", // 518
  "cadd_cmpLTMask_0", // 519
  "cadd_cmpLTMask_2", // 520
  "addL_eReg", // 521
  "addL_eReg_imm", // 522
  "addL_eReg_mem", // 523
  "addL_eReg_mem_0", // 524
  "subL_eReg", // 525
  "subL_eReg_imm", // 526
  "subL_eReg_mem", // 527
  "negL_eReg", // 528
  "andL_eReg", // 529
  "andL_eReg_imm", // 530
  "andL_eReg_mem", // 531
  "andL_eReg_mem_0", // 532
  "orl_eReg", // 533
  "orl_eReg_imm", // 534
  "orl_eReg_mem", // 535
  "orl_eReg_mem_0", // 536
  "xorl_eReg", // 537
  "xorl_eReg_im1", // 538
  "xorl_eReg_imm", // 539
  "xorl_eReg_mem", // 540
  "xorl_eReg_mem_0", // 541
  "shlL_eReg_1", // 542
  "shlL_eReg_2", // 543
  "shlL_eReg_3", // 544
  "shlL_eReg_1_31", // 545
  "shlL_eReg_32_63", // 546
  "salL_eReg_CL", // 547
  "shrL_eReg_1_31", // 548
  "shrL_eReg_32_63", // 549
  "shrL_eReg_CL", // 550
  "sarL_eReg_1_31", // 551
  "sarL_eReg_32_63", // 552
  "sarL_eReg_CL", // 553
  "cmpD_0", // 554
  "cmpD_reg", // 555
  "cmpXD_reg", // 556
  "cmpXD_regmem", // 557
  "subD_reg", // 558
  "subD_reg_round", // 559
  "subD_reg_mem", // 560
  "addD_reg", // 561
  "addD_reg_round", // 562
  "addD_reg_mem", // 563
  "addD_reg_mem_0", // 564
  "addD_mem_reg", // 565
  "addD_mem_reg_0", // 566
  "addD_reg_imm1", // 567
  "addD_reg_imm", // 568
  "addD_reg_imm_round", // 569
  "addXD_reg", // 570
  "addXD_imm", // 571
  "addXD_mem", // 572
  "addXD_mem_0", // 573
  "subXD_reg", // 574
  "subXD_imm", // 575
  "subXD_mem", // 576
  "mulXD_reg", // 577
  "mulXD_imm", // 578
  "mulXD_mem", // 579
  "mulXD_mem_0", // 580
  "divXD_reg", // 581
  "divXD_imm", // 582
  "divXD_mem", // 583
  "mulD_reg", // 584
  "strictfp_mulD_reg", // 585
  "mulD_reg_imm", // 586
  "mulD_reg_mem", // 587
  "mulD_reg_mem_0", // 588
  "mulD_reg_mem_cisc", // 589
  "mulD_reg_mem_cisc_0", // 590
  "addD_mulD_reg", // 591
  "addD_mulD_reg_0", // 592
  "subD_mulD_reg", // 593
  "divD_reg", // 594
  "strictfp_divD_reg", // 595
  "divD_reg_round", // 596
  "modD_reg", // 597
  "modXD_reg", // 598
  "sinD_reg", // 599
  "sinXD_reg", // 600
  "cosD_reg", // 601
  "cosXD_reg", // 602
  "tanD_reg", // 603
  "tanXD_reg", // 604
  "atanD_reg", // 605
  "atanXD_reg", // 606
  "sqrtD_reg", // 607
  "powD_reg", // 608
  "powXD_reg", // 609
  "expD_reg", // 610
  "expXD_reg", // 611
  "log10D_reg", // 612
  "log10XD_reg", // 613
  "logD_reg", // 614
  "logXD_reg", // 615
  "cmpF_0", // 616
  "cmpF_reg", // 617
  "cmpX_reg", // 618
  "cmpX_regmem", // 619
  "subF24_reg", // 620
  "subF_reg", // 621
  "addF24_reg", // 622
  "addF_reg", // 623
  "addX_reg", // 624
  "addX_imm", // 625
  "addX_mem", // 626
  "addX_mem_0", // 627
  "subX_reg", // 628
  "subX_imm", // 629
  "subX_mem", // 630
  "mulX_reg", // 631
  "mulX_imm", // 632
  "mulX_mem", // 633
  "mulX_mem_0", // 634
  "divX_reg", // 635
  "divX_imm", // 636
  "divX_mem", // 637
  "sqrtX_reg", // 638
  "sqrtX_mem", // 639
  "sqrtXD_reg", // 640
  "sqrtXD_mem", // 641
  "addF24_reg_mem", // 642
  "addF24_reg_mem_0", // 643
  "addF_reg_mem", // 644
  "addF_reg_mem_0", // 645
  "addF24_mem_reg", // 646
  "addF24_mem_cisc", // 647
  "addF24_mem_cisc_0", // 648
  "addF24_mem_mem", // 649
  "addF24_reg_imm", // 650
  "addF_reg_imm", // 651
  "mulF24_reg", // 652
  "mulF_reg", // 653
  "mulF24_reg_mem", // 654
  "mulF24_reg_mem_0", // 655
  "mulF_reg_mem", // 656
  "mulF_reg_mem_0", // 657
  "mulF24_mem_mem", // 658
  "mulF24_reg_imm", // 659
  "mulF_reg_imm", // 660
  "mulF_reg_load1", // 661
  "mulF_reg_load1_0", // 662
  "addF_mulF_reg_load1", // 663
  "addF_mulF_reg_load1_1", // 664
  "addF_mulF_reg_load1_0", // 665
  "addF_mulF_reg_load1_2", // 666
  "addF_mulF_reg", // 667
  "addF_mulF_reg_0", // 668
  "subF_divF_reg", // 669
  "divF24_reg", // 670
  "divF_reg", // 671
  "modF24_reg", // 672
  "modF_reg", // 673
  "modX_reg", // 674
  "roundFloat_mem_reg", // 675
  "roundDouble_mem_reg", // 676
  "convD2F_reg", // 677
  "convD2X_reg", // 678
  "convXD2X_reg", // 679
  "convF2D_reg_reg", // 680
  "convF2D_reg", // 681
  "convX2D_reg", // 682
  "convX2XD_reg", // 683
  "convD2I_reg_reg", // 684
  "convXD2I_reg_reg", // 685
  "convD2L_reg_reg", // 686
  "convXD2L_reg_reg", // 687
  "convF2I_reg_reg", // 688
  "convX2I_reg", // 689
  "convF2L_reg_reg", // 690
  "convX2L_reg_reg", // 691
  "convI2D_reg", // 692
  "convI2XD_reg", // 693
  "convI2XD_mem", // 694
  "convXI2XD_reg", // 695
  "convI2D_mem", // 696
  "conv24I2F_reg", // 697
  "convI2F_SSF", // 698
  "convI2F_SSF_mem", // 699
  "convI2F_reg", // 700
  "convI2F_mem", // 701
  "convI2X_reg", // 702
  "convXI2X_reg", // 703
  "convI2L_reg", // 704
  "convI2L_reg_zex", // 705
  "zerox_long", // 706
  "convL2D_reg", // 707
  "convL2XD_reg", // 708
  "convL2X_reg", // 709
  "convL2F_reg", // 710
  "convL2I_reg", // 711
  "MoveF2I_reg_stack", // 712
  "MoveF2I_reg_stack_sse", // 713
  "MoveI2F_reg_stack", // 714
  "MoveD2L_reg_stack", // 715
  "MoveD2L_reg_stack_sse", // 716
  "MoveD2L_reg_reg_sse", // 717
  "MoveL2D_reg_stack", // 718
  "MoveL2D_reg_reg_sse", // 719
  "Repl8B_reg", // 720
  "Repl8B_eRegI", // 721
  "Repl8B_immI0", // 722
  "Repl4S_reg", // 723
  "Repl4S_eRegI", // 724
  "Repl4S_immI0", // 725
  "Repl4C_reg", // 726
  "Repl4C_eRegI", // 727
  "Repl4C_immI0", // 728
  "Repl2I_reg", // 729
  "Repl2I_eRegI", // 730
  "Repl2I_immI0", // 731
  "Repl2F_reg", // 732
  "Repl2F_regX", // 733
  "Repl2F_immXF0", // 734
  "rep_stos", // 735
  "string_compare", // 736
  "array_equals", // 737
  "minI_eReg", // 738
  "maxI_eReg", // 739
  "jumpXtnd", // 740
  "jmpDir", // 741
  "jmpCon", // 742
  "jmpLoopEnd", // 743
  "jmpLoopEndU", // 744
  "jmpLoopEndUCF", // 745
  "jmpConU", // 746
  "jmpConUCF", // 747
  "jmpConUCF2", // 748
  "partialSubtypeCheck", // 749
  "jmpDir_short", // 750
  "jmpCon_short", // 751
  "jmpLoopEnd_short", // 752
  "jmpLoopEndU_short", // 753
  "jmpLoopEndUCF_short", // 754
  "jmpConU_short", // 755
  "jmpConUCF_short", // 756
  "jmpConUCF2_short", // 757
  "cmpL3_reg_reg", // 758
  "cmpL_LTGE", // 759
  "cmovLL_reg_LTGE", // 760
  "cmovLL_mem_LTGE", // 761
  "cmovII_reg_LTGE", // 762
  "cmovII_mem_LTGE", // 763
  "cmovPP_reg_LTGE", // 764
  "cmovDD_reg_LTGE", // 765
  "cmovXDD_reg_LTGE", // 766
  "cmovFF_reg_LTGE", // 767
  "cmovXX_reg_LTGE", // 768
  "cmpL_EQNE", // 769
  "cmovLL_reg_EQNE", // 770
  "cmovLL_mem_EQNE", // 771
  "cmovII_reg_EQNE", // 772
  "cmovII_mem_EQNE", // 773
  "cmovPP_reg_EQNE", // 774
  "cmovDD_reg_EQNE", // 775
  "cmovXDD_reg_EQNE", // 776
  "cmovFF_reg_EQNE", // 777
  "cmovXX_reg_EQNE", // 778
  "cmpL_LEGT", // 779
  "cmovLL_reg_LEGT", // 780
  "cmovLL_mem_LEGT", // 781
  "cmovII_reg_LEGT", // 782
  "cmovII_mem_LEGT", // 783
  "cmovPP_reg_LEGT", // 784
  "cmovDD_reg_LEGT", // 785
  "cmovXDD_reg_LEGT", // 786
  "cmovFF_reg_LEGT", // 787
  "cmovXX_reg_LEGT", // 788
  "CallStaticJavaDirect", // 789
  "CallDynamicJavaDirect", // 790
  "CallRuntimeDirect", // 791
  "CallLeafDirect", // 792
  "CallLeafNoFPDirect", // 793
  "Ret", // 794
  "TailCalljmpInd", // 795
  "tailjmpInd", // 796
  "CreateException", // 797
  "RethrowException", // 798
  "tlsLoadP", // 799
  "TLS", // 800
  "ShouldNotReachHere", // 801
  // last instruction
  "no trailing comma"
};

const        bool  swallowed[] = {
  false, // 0
  false, // 1
  false, // 2
  false, // 3
  false, // 4
  false, // 5
  false, // 6
  false, // 7
  true, // 8
  true, // 9
  true, // 10
  true, // 11
  true, // 12
  true, // 13
  true, // 14
  true, // 15
  true, // 16
  true, // 17
  true, // 18
  true, // 19
  true, // 20
  true, // 21
  true, // 22
  true, // 23
  true, // 24
  true, // 25
  true, // 26
  true, // 27
  true, // 28
  true, // 29
  true, // 30
  true, // 31
  true, // 32
  true, // 33
  true, // 34
  true, // 35
  true, // 36
  true, // 37
  true, // 38
  true, // 39
  true, // 40
  false, // 41
  false, // 42
  false, // 43
  false, // 44
  false, // 45
  false, // 46
  false, // 47
  false, // 48
  false, // 49
  false, // 50
  false, // 51
  false, // 52
  false, // 53
  false, // 54
  false, // 55
  false, // 56
  false, // 57
  false, // 58
  false, // 59
  false, // 60
  false, // 61
  false, // 62
  false, // 63
  false, // 64
  false, // 65
  false, // 66
  false, // 67
  false, // 68
  false, // 69
  false, // 70
  false, // 71
  false, // 72
  false, // 73
  false, // 74
  false, // 75
  false, // 76
  false, // 77
  false, // 78
  false, // 79
  false, // 80
  false, // 81
  false, // 82
  false, // 83
  true, // 84
  false, // 85
  false, // 86
  false, // 87
  false, // 88
  false, // 89
  false, // 90
  false, // 91
  false, // 92
  false, // 93
  false, // 94
  false, // 95
  false, // 96
  false, // 97
  false, // 98
  false, // 99
  false, // 100
  false, // 101
  false, // 102
  false, // 103
  false, // 104
  false, // 105
  false, // 106
  true, // 107
  true, // 108
  true, // 109
  true, // 110
  true, // 111
  true, // 112
  // last operand
  false, // 113
  false, // 114
  false, // 115
  // last operand class
  false, // 116
  false, // 117
  false, // 118
  false, // 119
  false, // 120
  false, // 121
  false, // 122
  false, // 123
  false, // 124
  false, // 125
  false, // 126
  false, // 127
  false, // 128
  false, // 129
  false, // 130
  false, // 131
  false, // 132
  false, // 133
  false, // 134
  false, // 135
  false, // 136
  false, // 137
  false, // 138
  false, // 139
  false, // 140
  false, // 141
  false, // 142
  false, // 143
  false, // 144
  false, // 145
  false, // 146
  false, // 147
  false, // 148
  false, // 149
  false, // 150
  false, // 151
  false, // 152
  false, // 153
  false, // 154
  false, // 155
  false, // 156
  false, // 157
  false, // 158
  false, // 159
  false, // 160
  false, // 161
  false, // 162
  false, // 163
  false, // 164
  false, // 165
  false, // 166
  false, // 167
  false, // 168
  false, // 169
  false, // 170
  false, // 171
  false, // 172
  false, // 173
  false, // 174
  false, // 175
  false, // 176
  false, // 177
  false, // 178
  false, // 179
  false, // 180
  false, // 181
  false, // 182
  false, // 183
  false, // 184
  false, // 185
  false, // 186
  false, // 187
  false, // 188
  false, // 189
  false, // 190
  false, // 191
  false, // 192
  false, // 193
  false, // 194
  false, // 195
  false, // 196
  false, // 197
  false, // 198
  false, // 199
  false, // 200
  false, // 201
  false, // 202
  false, // 203
  false, // 204
  false, // 205
  false, // 206
  false, // 207
  false, // 208
  false, // 209
  false, // 210
  false, // 211
  false, // 212
  false, // 213
  false, // 214
  false, // 215
  false, // 216
  false, // 217
  false, // 218
  false, // 219
  false, // 220
  false, // 221
  false, // 222
  // last internally defined operand
  false, // 223
  false, // 224
  false, // 225
  false, // 226
  false, // 227
  false, // 228
  false, // 229
  false, // 230
  false, // 231
  false, // 232
  false, // 233
  false, // 234
  false, // 235
  false, // 236
  false, // 237
  false, // 238
  false, // 239
  false, // 240
  false, // 241
  false, // 242
  false, // 243
  false, // 244
  false, // 245
  false, // 246
  false, // 247
  false, // 248
  false, // 249
  false, // 250
  false, // 251
  false, // 252
  false, // 253
  false, // 254
  false, // 255
  false, // 256
  false, // 257
  false, // 258
  false, // 259
  false, // 260
  false, // 261
  false, // 262
  false, // 263
  false, // 264
  false, // 265
  false, // 266
  false, // 267
  false, // 268
  false, // 269
  false, // 270
  false, // 271
  false, // 272
  false, // 273
  false, // 274
  false, // 275
  false, // 276
  false, // 277
  false, // 278
  false, // 279
  false, // 280
  false, // 281
  false, // 282
  false, // 283
  false, // 284
  false, // 285
  false, // 286
  false, // 287
  false, // 288
  false, // 289
  false, // 290
  false, // 291
  false, // 292
  false, // 293
  false, // 294
  false, // 295
  false, // 296
  false, // 297
  false, // 298
  false, // 299
  false, // 300
  false, // 301
  false, // 302
  false, // 303
  false, // 304
  false, // 305
  false, // 306
  false, // 307
  false, // 308
  false, // 309
  false, // 310
  false, // 311
  false, // 312
  false, // 313
  false, // 314
  false, // 315
  false, // 316
  false, // 317
  false, // 318
  false, // 319
  false, // 320
  false, // 321
  false, // 322
  false, // 323
  false, // 324
  false, // 325
  false, // 326
  false, // 327
  false, // 328
  false, // 329
  false, // 330
  false, // 331
  false, // 332
  false, // 333
  false, // 334
  false, // 335
  false, // 336
  false, // 337
  false, // 338
  false, // 339
  false, // 340
  false, // 341
  false, // 342
  false, // 343
  false, // 344
  false, // 345
  false, // 346
  false, // 347
  false, // 348
  false, // 349
  false, // 350
  false, // 351
  false, // 352
  false, // 353
  false, // 354
  false, // 355
  false, // 356
  false, // 357
  false, // 358
  false, // 359
  false, // 360
  false, // 361
  false, // 362
  false, // 363
  false, // 364
  false, // 365
  false, // 366
  false, // 367
  false, // 368
  false, // 369
  false, // 370
  false, // 371
  false, // 372
  false, // 373
  false, // 374
  false, // 375
  false, // 376
  false, // 377
  false, // 378
  false, // 379
  false, // 380
  false, // 381
  false, // 382
  false, // 383
  false, // 384
  false, // 385
  false, // 386
  false, // 387
  false, // 388
  false, // 389
  false, // 390
  false, // 391
  false, // 392
  false, // 393
  false, // 394
  false, // 395
  false, // 396
  false, // 397
  false, // 398
  false, // 399
  false, // 400
  false, // 401
  false, // 402
  false, // 403
  false, // 404
  false, // 405
  false, // 406
  false, // 407
  false, // 408
  false, // 409
  false, // 410
  false, // 411
  false, // 412
  false, // 413
  false, // 414
  false, // 415
  false, // 416
  false, // 417
  false, // 418
  false, // 419
  false, // 420
  false, // 421
  false, // 422
  false, // 423
  false, // 424
  false, // 425
  false, // 426
  false, // 427
  false, // 428
  false, // 429
  false, // 430
  false, // 431
  false, // 432
  false, // 433
  false, // 434
  false, // 435
  false, // 436
  false, // 437
  false, // 438
  false, // 439
  false, // 440
  false, // 441
  false, // 442
  false, // 443
  false, // 444
  false, // 445
  false, // 446
  false, // 447
  false, // 448
  false, // 449
  false, // 450
  false, // 451
  false, // 452
  false, // 453
  false, // 454
  false, // 455
  false, // 456
  false, // 457
  false, // 458
  false, // 459
  false, // 460
  false, // 461
  false, // 462
  false, // 463
  false, // 464
  false, // 465
  false, // 466
  false, // 467
  false, // 468
  false, // 469
  false, // 470
  false, // 471
  false, // 472
  false, // 473
  false, // 474
  false, // 475
  false, // 476
  false, // 477
  false, // 478
  false, // 479
  false, // 480
  false, // 481
  false, // 482
  false, // 483
  false, // 484
  false, // 485
  false, // 486
  false, // 487
  false, // 488
  false, // 489
  false, // 490
  false, // 491
  false, // 492
  false, // 493
  false, // 494
  false, // 495
  false, // 496
  false, // 497
  false, // 498
  false, // 499
  false, // 500
  false, // 501
  false, // 502
  false, // 503
  false, // 504
  false, // 505
  false, // 506
  false, // 507
  false, // 508
  false, // 509
  false, // 510
  false, // 511
  false, // 512
  false, // 513
  false, // 514
  false, // 515
  false, // 516
  false, // 517
  false, // 518
  false, // 519
  false, // 520
  false, // 521
  false, // 522
  false, // 523
  false, // 524
  false, // 525
  false, // 526
  false, // 527
  false, // 528
  false, // 529
  false, // 530
  false, // 531
  false, // 532
  false, // 533
  false, // 534
  false, // 535
  false, // 536
  false, // 537
  false, // 538
  false, // 539
  false, // 540
  false, // 541
  false, // 542
  false, // 543
  false, // 544
  false, // 545
  false, // 546
  false, // 547
  false, // 548
  false, // 549
  false, // 550
  false, // 551
  false, // 552
  false, // 553
  false, // 554
  false, // 555
  false, // 556
  false, // 557
  false, // 558
  false, // 559
  false, // 560
  false, // 561
  false, // 562
  false, // 563
  false, // 564
  false, // 565
  false, // 566
  false, // 567
  false, // 568
  false, // 569
  false, // 570
  false, // 571
  false, // 572
  false, // 573
  false, // 574
  false, // 575
  false, // 576
  false, // 577
  false, // 578
  false, // 579
  false, // 580
  false, // 581
  false, // 582
  false, // 583
  false, // 584
  false, // 585
  false, // 586
  false, // 587
  false, // 588
  false, // 589
  false, // 590
  false, // 591
  false, // 592
  false, // 593
  false, // 594
  false, // 595
  false, // 596
  false, // 597
  false, // 598
  false, // 599
  false, // 600
  false, // 601
  false, // 602
  false, // 603
  false, // 604
  false, // 605
  false, // 606
  false, // 607
  false, // 608
  false, // 609
  false, // 610
  false, // 611
  false, // 612
  false, // 613
  false, // 614
  false, // 615
  false, // 616
  false, // 617
  false, // 618
  false, // 619
  false, // 620
  false, // 621
  false, // 622
  false, // 623
  false, // 624
  false, // 625
  false, // 626
  false, // 627
  false, // 628
  false, // 629
  false, // 630
  false, // 631
  false, // 632
  false, // 633
  false, // 634
  false, // 635
  false, // 636
  false, // 637
  false, // 638
  false, // 639
  false, // 640
  false, // 641
  false, // 642
  false, // 643
  false, // 644
  false, // 645
  false, // 646
  false, // 647
  false, // 648
  false, // 649
  false, // 650
  false, // 651
  false, // 652
  false, // 653
  false, // 654
  false, // 655
  false, // 656
  false, // 657
  false, // 658
  false, // 659
  false, // 660
  false, // 661
  false, // 662
  false, // 663
  false, // 664
  false, // 665
  false, // 666
  false, // 667
  false, // 668
  false, // 669
  false, // 670
  false, // 671
  false, // 672
  false, // 673
  false, // 674
  false, // 675
  false, // 676
  false, // 677
  false, // 678
  false, // 679
  false, // 680
  false, // 681
  false, // 682
  false, // 683
  false, // 684
  false, // 685
  false, // 686
  false, // 687
  false, // 688
  false, // 689
  false, // 690
  false, // 691
  false, // 692
  false, // 693
  false, // 694
  false, // 695
  false, // 696
  false, // 697
  false, // 698
  false, // 699
  false, // 700
  false, // 701
  false, // 702
  false, // 703
  false, // 704
  false, // 705
  false, // 706
  false, // 707
  false, // 708
  false, // 709
  false, // 710
  false, // 711
  false, // 712
  false, // 713
  false, // 714
  false, // 715
  false, // 716
  false, // 717
  false, // 718
  false, // 719
  false, // 720
  false, // 721
  false, // 722
  false, // 723
  false, // 724
  false, // 725
  false, // 726
  false, // 727
  false, // 728
  false, // 729
  false, // 730
  false, // 731
  false, // 732
  false, // 733
  false, // 734
  false, // 735
  false, // 736
  false, // 737
  false, // 738
  false, // 739
  false, // 740
  false, // 741
  false, // 742
  false, // 743
  false, // 744
  false, // 745
  false, // 746
  false, // 747
  false, // 748
  false, // 749
  false, // 750
  false, // 751
  false, // 752
  false, // 753
  false, // 754
  false, // 755
  false, // 756
  false, // 757
  false, // 758
  false, // 759
  false, // 760
  false, // 761
  false, // 762
  false, // 763
  false, // 764
  false, // 765
  false, // 766
  false, // 767
  false, // 768
  false, // 769
  false, // 770
  false, // 771
  false, // 772
  false, // 773
  false, // 774
  false, // 775
  false, // 776
  false, // 777
  false, // 778
  false, // 779
  false, // 780
  false, // 781
  false, // 782
  false, // 783
  false, // 784
  false, // 785
  false, // 786
  false, // 787
  false, // 788
  false, // 789
  false, // 790
  false, // 791
  false, // 792
  false, // 793
  false, // 794
  false, // 795
  false, // 796
  false, // 797
  false, // 798
  false, // 799
  false, // 800
  false, // 801
  // last instruction
  false // no trailing comma
};

// Mapping from machine-independent opcode to boolean
const        char must_clone[] = {
  0, // Node: 0
  0, // Set: 1
  0, // RegN: 2
  0, // RegI: 3
  0, // RegP: 4
  0, // RegF: 5
  0, // RegD: 6
  0, // RegL: 7
  0, // RegFlags: 8
  0, // _last_machine_leaf: 9
  0, // AbsD: 10
  0, // AbsF: 11
  0, // AbsI: 12
  0, // AddD: 13
  0, // AddF: 14
  0, // AddI: 15
  0, // AddL: 16
  0, // AddP: 17
  0, // Allocate: 18
  0, // AllocateArray: 19
  0, // AndI: 20
  0, // AndL: 21
  0, // AryEq: 22
  0, // AtanD: 23
  1, // Binary: 24
  1, // Bool: 25
  0, // BoxLock: 26
  0, // ReverseBytesI: 27
  0, // ReverseBytesL: 28
  0, // CProj: 29
  0, // CallDynamicJava: 30
  0, // CallJava: 31
  0, // CallLeaf: 32
  0, // CallLeafNoFP: 33
  0, // CallRuntime: 34
  0, // CallStaticJava: 35
  0, // CastII: 36
  0, // CastX2P: 37
  0, // CastP2X: 38
  0, // CastPP: 39
  0, // Catch: 40
  0, // CatchProj: 41
  0, // CheckCastPP: 42
  0, // ClearArray: 43
  0, // ConstraintCast: 44
  0, // CMoveD: 45
  0, // CMoveF: 46
  0, // CMoveI: 47
  0, // CMoveL: 48
  0, // CMoveP: 49
  0, // CMoveN: 50
  1, // CmpN: 51
  1, // CmpD: 52
  0, // CmpD3: 53
  1, // CmpF: 54
  0, // CmpF3: 55
  1, // CmpI: 56
  1, // CmpL: 57
  0, // CmpL3: 58
  0, // CmpLTMask: 59
  1, // CmpP: 60
  1, // CmpU: 61
  0, // CompareAndSwapI: 62
  0, // CompareAndSwapL: 63
  0, // CompareAndSwapP: 64
  0, // CompareAndSwapN: 65
  0, // Con: 66
  0, // ConN: 67
  0, // ConD: 68
  0, // ConF: 69
  0, // ConI: 70
  0, // ConL: 71
  0, // ConP: 72
  0, // Conv2B: 73
  0, // ConvD2F: 74
  0, // ConvD2I: 75
  0, // ConvD2L: 76
  0, // ConvF2D: 77
  0, // ConvF2I: 78
  0, // ConvF2L: 79
  0, // ConvI2D: 80
  0, // ConvI2F: 81
  0, // ConvI2L: 82
  0, // ConvL2D: 83
  0, // ConvL2F: 84
  0, // ConvL2I: 85
  0, // CosD: 86
  0, // CountedLoop: 87
  0, // CountedLoopEnd: 88
  0, // CreateEx: 89
  0, // DecodeN: 90
  0, // DivD: 91
  0, // DivF: 92
  0, // DivI: 93
  0, // DivL: 94
  0, // DivMod: 95
  0, // DivModI: 96
  0, // DivModL: 97
  0, // EncodeP: 98
  0, // ExpD: 99
  1, // FastLock: 100
  1, // FastUnlock: 101
  0, // Goto: 102
  0, // Halt: 103
  0, // If: 104
  0, // IfFalse: 105
  0, // IfTrue: 106
  0, // Initialize: 107
  0, // JProj: 108
  0, // Jump: 109
  0, // JumpProj: 110
  0, // LShiftI: 111
  0, // LShiftL: 112
  0, // LoadB: 113
  0, // LoadC: 114
  0, // LoadD: 115
  0, // LoadD_unaligned: 116
  0, // LoadF: 117
  0, // LoadI: 118
  0, // LoadKlass: 119
  0, // LoadNKlass: 120
  0, // LoadL: 121
  0, // LoadL_unaligned: 122
  0, // LoadPLocked: 123
  0, // LoadLLocked: 124
  0, // LoadP: 125
  0, // LoadN: 126
  0, // LoadRange: 127
  0, // LoadS: 128
  0, // Lock: 129
  0, // LogD: 130
  0, // Log10D: 131
  0, // Loop: 132
  0, // Mach: 133
  0, // MachProj: 134
  0, // MaxI: 135
  0, // MemBarAcquire: 136
  0, // MemBarCPUOrder: 137
  0, // MemBarRelease: 138
  0, // MemBarVolatile: 139
  0, // MergeMem: 140
  0, // MinI: 141
  0, // ModD: 142
  0, // ModF: 143
  0, // ModI: 144
  0, // ModL: 145
  0, // MoveI2F: 146
  0, // MoveF2I: 147
  0, // MoveL2D: 148
  0, // MoveD2L: 149
  0, // MulD: 150
  0, // MulF: 151
  0, // MulHiL: 152
  0, // MulI: 153
  0, // MulL: 154
  0, // Multi: 155
  0, // NegD: 156
  0, // NegF: 157
  0, // NeverBranch: 158
  0, // Opaque1: 159
  0, // Opaque2: 160
  0, // OrI: 161
  0, // OrL: 162
  0, // PCTable: 163
  0, // Parm: 164
  0, // PartialSubtypeCheck: 165
  0, // Phi: 166
  0, // PowD: 167
  0, // PrefetchRead: 168
  0, // PrefetchWrite: 169
  0, // Proj: 170
  0, // RShiftI: 171
  0, // RShiftL: 172
  0, // Region: 173
  0, // Rethrow: 174
  0, // Return: 175
  0, // Root: 176
  0, // RoundDouble: 177
  0, // RoundFloat: 178
  0, // SafePoint: 179
  0, // SafePointScalarObject: 180
  0, // SCMemProj: 181
  0, // SinD: 182
  0, // SqrtD: 183
  0, // Start: 184
  0, // StartOSR: 185
  0, // StoreB: 186
  0, // StoreC: 187
  0, // StoreCM: 188
  0, // StorePConditional: 189
  0, // StoreIConditional: 190
  0, // StoreLConditional: 191
  0, // StoreD: 192
  0, // StoreF: 193
  0, // StoreI: 194
  0, // StoreL: 195
  0, // StoreP: 196
  0, // StoreN: 197
  0, // StrComp: 198
  0, // SubD: 199
  0, // SubF: 200
  0, // SubI: 201
  0, // SubL: 202
  0, // TailCall: 203
  0, // TailJump: 204
  0, // TanD: 205
  0, // ThreadLocal: 206
  0, // Unlock: 207
  0, // URShiftI: 208
  0, // URShiftL: 209
  0, // XorI: 210
  0, // XorL: 211
  0, // Vector: 212
  0, // AddVB: 213
  0, // AddVC: 214
  0, // AddVS: 215
  0, // AddVI: 216
  0, // AddVL: 217
  0, // AddVF: 218
  0, // AddVD: 219
  0, // SubVB: 220
  0, // SubVC: 221
  0, // SubVS: 222
  0, // SubVI: 223
  0, // SubVL: 224
  0, // SubVF: 225
  0, // SubVD: 226
  0, // MulVF: 227
  0, // MulVD: 228
  0, // DivVF: 229
  0, // DivVD: 230
  0, // LShiftVB: 231
  0, // LShiftVC: 232
  0, // LShiftVS: 233
  0, // LShiftVI: 234
  0, // URShiftVB: 235
  0, // URShiftVC: 236
  0, // URShiftVS: 237
  0, // URShiftVI: 238
  0, // AndV: 239
  0, // OrV: 240
  0, // XorV: 241
  0, // VectorLoad: 242
  0, // Load16B: 243
  0, // Load8B: 244
  0, // Load4B: 245
  0, // Load8C: 246
  0, // Load4C: 247
  0, // Load2C: 248
  0, // Load8S: 249
  0, // Load4S: 250
  0, // Load2S: 251
  0, // Load4I: 252
  0, // Load2I: 253
  0, // Load2L: 254
  0, // Load4F: 255
  0, // Load2F: 256
  0, // Load2D: 257
  0, // VectorStore: 258
  0, // Store16B: 259
  0, // Store8B: 260
  0, // Store4B: 261
  0, // Store8C: 262
  0, // Store4C: 263
  0, // Store2C: 264
  0, // Store4I: 265
  0, // Store2I: 266
  0, // Store2L: 267
  0, // Store4F: 268
  0, // Store2F: 269
  0, // Store2D: 270
  0, // Pack: 271
  0, // PackB: 272
  0, // PackS: 273
  0, // PackC: 274
  0, // PackI: 275
  0, // PackL: 276
  0, // PackF: 277
  0, // PackD: 278
  0, // Pack2x1B: 279
  0, // Pack2x2B: 280
  0, // Replicate16B: 281
  0, // Replicate8B: 282
  0, // Replicate4B: 283
  0, // Replicate8S: 284
  0, // Replicate4S: 285
  0, // Replicate2S: 286
  0, // Replicate8C: 287
  0, // Replicate4C: 288
  0, // Replicate2C: 289
  0, // Replicate4I: 290
  0, // Replicate2I: 291
  0, // Replicate2L: 292
  0, // Replicate4F: 293
  0, // Replicate2F: 294
  0, // Replicate2D: 295
  0, // Extract: 296
  0, // ExtractB: 297
  0, // ExtractS: 298
  0, // ExtractC: 299
  0, // ExtractI: 300
  0, // ExtractL: 301
  0, // ExtractF: 302
  0 // no trailing comma // ExtractD: 303
};
//  The following instructions can cisc-spill
//  cmovI_reg can cisc-spill operand 4 to cmovI_mem
//  cmovI_regU can cisc-spill operand 4 to cmovI_memU
//  cmovI_regUCF can cisc-spill operand 4 to cmovI_memUCF
//  addI_eReg can cisc-spill operand 2 to addI_eReg_mem
//  subI_eReg can cisc-spill operand 2 to subI_eReg_mem
//  mulI_eReg can cisc-spill operand 2 to mulI
//  mulI_eReg_imm can cisc-spill operand 1 to mulI_mem_imm
//  andI_eReg can cisc-spill operand 2 to andI_eReg_mem
//  orI_eReg can cisc-spill operand 2 to orI_eReg_mem
//  xorI_eReg can cisc-spill operand 2 to xorI_eReg_mem
//  addL_eReg can cisc-spill operand 2 to addL_eReg_mem
//  subL_eReg can cisc-spill operand 2 to subL_eReg_mem
//  andL_eReg can cisc-spill operand 2 to andL_eReg_mem
//  orl_eReg can cisc-spill operand 2 to orl_eReg_mem
//  xorl_eReg can cisc-spill operand 2 to xorl_eReg_mem
//  cmpXD_cc can cisc-spill operand 2 to cmpXD_ccmem
//  cmpXD_ccCF can cisc-spill operand 2 to cmpXD_ccmemCF
//  cmpXD_reg can cisc-spill operand 2 to cmpXD_regmem
//  subD_reg can cisc-spill operand 2 to subD_reg_mem
//  addD_reg can cisc-spill operand 2 to addD_reg_mem
//  addXD_reg can cisc-spill operand 2 to addXD_mem
//  subXD_reg can cisc-spill operand 2 to subXD_mem
//  mulXD_reg can cisc-spill operand 2 to mulXD_mem
//  divXD_reg can cisc-spill operand 2 to divXD_mem
//  cmpX_cc can cisc-spill operand 2 to cmpX_ccmem
//  cmpX_ccCF can cisc-spill operand 2 to cmpX_ccmemCF
//  cmpX_reg can cisc-spill operand 2 to cmpX_regmem
//  addF24_reg can cisc-spill operand 2 to addF24_reg_mem
//  addF_reg can cisc-spill operand 2 to addF_reg_mem
//  addX_reg can cisc-spill operand 2 to addX_mem
//  subX_reg can cisc-spill operand 2 to subX_mem
//  mulX_reg can cisc-spill operand 2 to mulX_mem
//  divX_reg can cisc-spill operand 2 to divX_mem
//  sqrtX_reg can cisc-spill operand 1 to sqrtX_mem
//  sqrtXD_reg can cisc-spill operand 1 to sqrtXD_mem
//  addF24_reg_mem can cisc-spill operand 1 to addF24_mem_cisc
//  addF24_reg_mem_0 can cisc-spill operand 2 to addF24_mem_cisc_0
//  addF24_mem_reg can cisc-spill operand 2 to addF24_mem_cisc
//  mulF24_reg can cisc-spill operand 2 to mulF24_reg_mem
//  mulF_reg can cisc-spill operand 2 to mulF_reg_mem
//  addF_mulF_reg can cisc-spill operand 1 to addF_mulF_reg_load1
//  addF_mulF_reg_0 can cisc-spill operand 2 to addF_mulF_reg_load1_1
//  compI_eReg can cisc-spill operand 2 to compI_eReg_mem
//  compU_eReg can cisc-spill operand 2 to compU_eReg_mem
//  compP_eReg can cisc-spill operand 2 to compP_eReg_mem
//  cmovLL_reg_LTGE can cisc-spill operand 4 to cmovLL_mem_LTGE
//  cmovII_reg_LTGE can cisc-spill operand 4 to cmovII_mem_LTGE
//  cmovLL_reg_EQNE can cisc-spill operand 4 to cmovLL_mem_EQNE
//  cmovII_reg_EQNE can cisc-spill operand 4 to cmovII_mem_EQNE
//  cmovLL_reg_LEGT can cisc-spill operand 4 to cmovLL_mem_LEGT
//  cmovII_reg_LEGT can cisc-spill operand 4 to cmovII_mem_LEGT



// An array of character pointers to machine register names.
const char *Matcher::regName[REG_COUNT] = {
  "ECX",
  "EBX",
  "EBP",
  "EDI",
  "EAX",
  "EDX",
  "ESI",
  "ESP",
  "FPR0L",
  "FPR0H",
  "FPR1L",
  "FPR1H",
  "FPR2L",
  "FPR2H",
  "FPR3L",
  "FPR3H",
  "FPR4L",
  "FPR4H",
  "FPR5L",
  "FPR5H",
  "FPR6L",
  "FPR6H",
  "FPR7L",
  "FPR7H",
  "XMM0a",
  "XMM0b",
  "XMM1a",
  "XMM1b",
  "XMM2a",
  "XMM2b",
  "XMM3a",
  "XMM3b",
  "XMM4a",
  "XMM4b",
  "XMM5a",
  "XMM5b",
  "XMM6a",
  "XMM6b",
  "XMM7a",
  "XMM7b",
  "EFLAGS" // no trailing comma
};

// An array of character pointers to machine register names.
const VMReg OptoReg::opto2vm[REG_COUNT] = {
	rcx->as_VMReg(),
	rbx->as_VMReg(),
	rbp->as_VMReg(),
	rdi->as_VMReg(),
	rax->as_VMReg(),
	rdx->as_VMReg(),
	rsi->as_VMReg(),
	rsp->as_VMReg(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	as_FloatRegister(0)->as_VMReg(),
	as_FloatRegister(0)->as_VMReg()->next(),
	as_FloatRegister(1)->as_VMReg(),
	as_FloatRegister(1)->as_VMReg()->next(),
	as_FloatRegister(2)->as_VMReg(),
	as_FloatRegister(2)->as_VMReg()->next(),
	as_FloatRegister(3)->as_VMReg(),
	as_FloatRegister(3)->as_VMReg()->next(),
	as_FloatRegister(4)->as_VMReg(),
	as_FloatRegister(4)->as_VMReg()->next(),
	as_FloatRegister(5)->as_VMReg(),
	as_FloatRegister(5)->as_VMReg()->next(),
	as_FloatRegister(6)->as_VMReg(),
	as_FloatRegister(6)->as_VMReg()->next(),
	xmm0->as_VMReg(),
	xmm0->as_VMReg()->next(),
	xmm1->as_VMReg(),
	xmm1->as_VMReg()->next(),
	xmm2->as_VMReg(),
	xmm2->as_VMReg()->next(),
	xmm3->as_VMReg(),
	xmm3->as_VMReg()->next(),
	xmm4->as_VMReg(),
	xmm4->as_VMReg()->next(),
	xmm5->as_VMReg(),
	xmm5->as_VMReg()->next(),
	xmm6->as_VMReg(),
	xmm6->as_VMReg()->next(),
	xmm7->as_VMReg(),
	xmm7->as_VMReg()->next(),
	VMRegImpl::Bad() // no trailing comma
	};

 OptoReg::Name OptoReg::vm2opto[ConcreteRegisterImpl::number_of_registers];

// An array of the machine register encode values
const unsigned char Matcher::_regEncode[REG_COUNT] = {
  (unsigned char)'\x1',  // ECX
  (unsigned char)'\x3',  // EBX
  (unsigned char)'\x5',  // EBP
  (unsigned char)'\x7',  // EDI
  (unsigned char)'\x0',  // EAX
  (unsigned char)'\x2',  // EDX
  (unsigned char)'\x6',  // ESI
  (unsigned char)'\x4',  // ESP
  (unsigned char)'\x0',  // FPR0L
  (unsigned char)'\x0',  // FPR0H
  (unsigned char)'\x1',  // FPR1L
  (unsigned char)'\x1',  // FPR1H
  (unsigned char)'\x2',  // FPR2L
  (unsigned char)'\x2',  // FPR2H
  (unsigned char)'\x3',  // FPR3L
  (unsigned char)'\x3',  // FPR3H
  (unsigned char)'\x4',  // FPR4L
  (unsigned char)'\x4',  // FPR4H
  (unsigned char)'\x5',  // FPR5L
  (unsigned char)'\x5',  // FPR5H
  (unsigned char)'\x6',  // FPR6L
  (unsigned char)'\x6',  // FPR6H
  (unsigned char)'\x7',  // FPR7L
  (unsigned char)'\x7',  // FPR7H
  (unsigned char)'\x0',  // XMM0a
  (unsigned char)'\x0',  // XMM0b
  (unsigned char)'\x1',  // XMM1a
  (unsigned char)'\x1',  // XMM1b
  (unsigned char)'\x2',  // XMM2a
  (unsigned char)'\x2',  // XMM2b
  (unsigned char)'\x3',  // XMM3a
  (unsigned char)'\x3',  // XMM3b
  (unsigned char)'\x4',  // XMM4a
  (unsigned char)'\x4',  // XMM4b
  (unsigned char)'\x5',  // XMM5a
  (unsigned char)'\x5',  // XMM5b
  (unsigned char)'\x6',  // XMM6a
  (unsigned char)'\x6',  // XMM6b
  (unsigned char)'\x7',  // XMM7a
  (unsigned char)'\x7',  // XMM7b
  (unsigned char)'\x8' // no trailing comma  // EFLAGS
};


//------------------Define classes derived from MachOper---------------------
MachOper  *labelOper::clone(Compile* C) const {
  return  new (C) labelOper(_label, _block_num);
}
uint labelOper::opcode() const { return LABEL; }

const RegMask *sRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

MachOper  *methodOper::clone(Compile* C) const {
  return  new (C) methodOper(_method);
}
uint methodOper::opcode() const { return METHOD; }

const RegMask *eRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &E_REG_mask;
}

const RegMask *xRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &X_REG_mask;
}

const RegMask *eAXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EAX_REG_mask;
}

const RegMask *eBXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBX_REG_mask;
}

const RegMask *eCXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ECX_REG_mask;
}

const RegMask *eDXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDX_REG_mask;
}

const RegMask *eDIRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDI_REG_mask;
}

const RegMask *naxRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NAX_REG_mask;
}

const RegMask *nadxRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NADX_REG_mask;
}

const RegMask *ncxRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NCX_REG_mask;
}

const RegMask *eSIRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask;
}

const RegMask *anyRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask;
}

const RegMask *eRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &E_REG_mask;
}

const RegMask *eRegP_no_EBPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &E_REG_NO_RBP_mask;
}

const RegMask *naxRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NAX_REG_mask;
}

const RegMask *nabxRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NABX_REG_mask;
}

const RegMask *pRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &P_REG_mask;
}

const RegMask *eAXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EAX_REG_mask;
}

const RegMask *eBXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBX_REG_mask;
}

const RegMask *eCXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ECX_REG_mask;
}

const RegMask *eSIRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask;
}

const RegMask *eDIRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDI_REG_mask;
}

const RegMask *eBPRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBP_REG_mask;
}

const RegMask *eRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_REG_mask;
}

const RegMask *eADXRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EADX_REG_mask;
}

const RegMask *eBCXRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBCX_REG_mask;
}

const RegMask *eADXRegL_low_onlyOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EADX_REG_mask;
}

const RegMask *eFlagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *eFlagsRegUOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *eFlagsRegUCFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *flagsReg_long_LTGEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *flagsReg_long_EQNEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *flagsReg_long_LEGTOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *regDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DBL_REG_mask;
}

const RegMask *regDPR1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DBL_REG0_mask;
}

const RegMask *regDPR2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DBL_REG1_mask;
}

const RegMask *regnotDPR1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DBL_NOTREG0_mask;
}

const RegMask *regXDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &XDB_REG_mask;
}

const RegMask *regXD6Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &XDB_REG6_mask;
}

const RegMask *regXD7Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &XDB_REG7_mask;
}

const RegMask *regFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLT_REG_mask;
}

const RegMask *regFPR1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLT_REG0_mask;
}

const RegMask *regXOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &XMM_REG_mask;
}

const RegMask *indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &E_REG_mask;
}

const RegMask *indOffset8Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &E_REG_mask;
}

const RegMask *indOffset32Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &E_REG_mask;
}

const RegMask *indOffset32XOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &E_REG_mask;
}

const RegMask *indIndexOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &E_REG_mask;
}

const RegMask *indIndexOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &E_REG_mask;
}

const RegMask *indIndexScaleOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &E_REG_mask;
}

const RegMask *indIndexScaleOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &E_REG_mask;
}

const RegMask *load_long_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask;
}

const RegMask *load_long_indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask;
}

const RegMask *load_long_indOffset32Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask;
}

const RegMask *stackSlotPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *indirect_win95_safeOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &E_REG_mask;
}

const RegMask *indOffset8_win95_safeOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &E_REG_NO_RBP_mask;
}

const RegMask *indOffset32_win95_safeOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &E_REG_NO_RBP_mask;
}

const RegMask *indIndexOffset_win95_safeOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &E_REG_NO_RBP_mask;
  case 1: return &E_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScale_win95_safeOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &E_REG_NO_RBP_mask;
  case 1: return &E_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaleOffset_win95_safeOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &E_REG_NO_RBP_mask;
  case 1: return &E_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

//------------------Define members for classes derived from MachNode----------

void cmovI_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovI_regNode::cisc_version( int offset, Compile* C ) {
  cmovI_memNode *node = new (C) cmovI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovI_regUNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovI_regUNode::cisc_version( int offset, Compile* C ) {
  cmovI_memUNode *node = new (C) cmovI_memUNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovI_regUCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovI_regUCFNode::cisc_version( int offset, Compile* C ) {
  cmovI_memUCFNode *node = new (C) cmovI_memUCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *addI_eRegNode::cisc_version( int offset, Compile* C ) {
  addI_eReg_memNode *node = new (C) addI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *subI_eRegNode::cisc_version( int offset, Compile* C ) {
  subI_eReg_memNode *node = new (C) subI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulI_eRegNode::cisc_version( int offset, Compile* C ) {
  mulINode *node = new (C) mulINode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulI_eReg_immNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulI_eReg_immNode::cisc_version( int offset, Compile* C ) {
  mulI_mem_immNode *node = new (C) mulI_mem_immNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void andI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *andI_eRegNode::cisc_version( int offset, Compile* C ) {
  andI_eReg_memNode *node = new (C) andI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void orI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *orI_eRegNode::cisc_version( int offset, Compile* C ) {
  orI_eReg_memNode *node = new (C) orI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void xorI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *xorI_eRegNode::cisc_version( int offset, Compile* C ) {
  xorI_eReg_memNode *node = new (C) xorI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addL_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *addL_eRegNode::cisc_version( int offset, Compile* C ) {
  addL_eReg_memNode *node = new (C) addL_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subL_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *subL_eRegNode::cisc_version( int offset, Compile* C ) {
  subL_eReg_memNode *node = new (C) subL_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void andL_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *andL_eRegNode::cisc_version( int offset, Compile* C ) {
  andL_eReg_memNode *node = new (C) andL_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void orl_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *orl_eRegNode::cisc_version( int offset, Compile* C ) {
  orl_eReg_memNode *node = new (C) orl_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void xorl_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *xorl_eRegNode::cisc_version( int offset, Compile* C ) {
  xorl_eReg_memNode *node = new (C) xorl_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpXD_ccNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XDB_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpXD_ccNode::cisc_version( int offset, Compile* C ) {
  cmpXD_ccmemNode *node = new (C) cmpXD_ccmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpXD_ccCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XDB_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpXD_ccCFNode::cisc_version( int offset, Compile* C ) {
  cmpXD_ccmemCFNode *node = new (C) cmpXD_ccmemCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpXD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XDB_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpXD_regNode::cisc_version( int offset, Compile* C ) {
  cmpXD_regmemNode *node = new (C) cmpXD_regmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DBL_REG_mask;
}

// Build CISC version of this instruction
MachNode *subD_regNode::cisc_version( int offset, Compile* C ) {
  subD_reg_memNode *node = new (C) subD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DBL_REG_mask;
}

// Build CISC version of this instruction
MachNode *addD_regNode::cisc_version( int offset, Compile* C ) {
  addD_reg_memNode *node = new (C) addD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addXD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XDB_REG_mask;
}

// Build CISC version of this instruction
MachNode *addXD_regNode::cisc_version( int offset, Compile* C ) {
  addXD_memNode *node = new (C) addXD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subXD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XDB_REG_mask;
}

// Build CISC version of this instruction
MachNode *subXD_regNode::cisc_version( int offset, Compile* C ) {
  subXD_memNode *node = new (C) subXD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulXD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XDB_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulXD_regNode::cisc_version( int offset, Compile* C ) {
  mulXD_memNode *node = new (C) mulXD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void divXD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XDB_REG_mask;
}

// Build CISC version of this instruction
MachNode *divXD_regNode::cisc_version( int offset, Compile* C ) {
  divXD_memNode *node = new (C) divXD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpX_ccNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XMM_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpX_ccNode::cisc_version( int offset, Compile* C ) {
  cmpX_ccmemNode *node = new (C) cmpX_ccmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpX_ccCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XMM_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpX_ccCFNode::cisc_version( int offset, Compile* C ) {
  cmpX_ccmemCFNode *node = new (C) cmpX_ccmemCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpX_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XMM_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpX_regNode::cisc_version( int offset, Compile* C ) {
  cmpX_regmemNode *node = new (C) cmpX_regmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addF24_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLT_REG_mask;
}

// Build CISC version of this instruction
MachNode *addF24_regNode::cisc_version( int offset, Compile* C ) {
  addF24_reg_memNode *node = new (C) addF24_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLT_REG_mask;
}

// Build CISC version of this instruction
MachNode *addF_regNode::cisc_version( int offset, Compile* C ) {
  addF_reg_memNode *node = new (C) addF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addX_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XMM_REG_mask;
}

// Build CISC version of this instruction
MachNode *addX_regNode::cisc_version( int offset, Compile* C ) {
  addX_memNode *node = new (C) addX_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subX_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XMM_REG_mask;
}

// Build CISC version of this instruction
MachNode *subX_regNode::cisc_version( int offset, Compile* C ) {
  subX_memNode *node = new (C) subX_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulX_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XMM_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulX_regNode::cisc_version( int offset, Compile* C ) {
  mulX_memNode *node = new (C) mulX_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void divX_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XMM_REG_mask;
}

// Build CISC version of this instruction
MachNode *divX_regNode::cisc_version( int offset, Compile* C ) {
  divX_memNode *node = new (C) divX_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void sqrtX_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XMM_REG_mask;
}

// Build CISC version of this instruction
MachNode *sqrtX_regNode::cisc_version( int offset, Compile* C ) {
  sqrtX_memNode *node = new (C) sqrtX_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void sqrtXD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_XDB_REG_mask;
}

// Build CISC version of this instruction
MachNode *sqrtXD_regNode::cisc_version( int offset, Compile* C ) {
  sqrtXD_memNode *node = new (C) sqrtXD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addF24_reg_memNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLT_REG_mask;
}

// Build CISC version of this instruction
MachNode *addF24_reg_memNode::cisc_version( int offset, Compile* C ) {
  addF24_mem_ciscNode *node = new (C) addF24_mem_ciscNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addF24_reg_mem_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLT_REG_mask;
}

// Build CISC version of this instruction
MachNode *addF24_reg_mem_0Node::cisc_version( int offset, Compile* C ) {
  addF24_mem_cisc_0Node *node = new (C) addF24_mem_cisc_0Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addF24_mem_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLT_REG_mask;
}

// Build CISC version of this instruction
MachNode *addF24_mem_regNode::cisc_version( int offset, Compile* C ) {
  addF24_mem_ciscNode *node = new (C) addF24_mem_ciscNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulF24_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLT_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulF24_regNode::cisc_version( int offset, Compile* C ) {
  mulF24_reg_memNode *node = new (C) mulF24_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLT_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulF_regNode::cisc_version( int offset, Compile* C ) {
  mulF_reg_memNode *node = new (C) mulF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addF_mulF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLT_REG_mask;
}

// Build CISC version of this instruction
MachNode *addF_mulF_regNode::cisc_version( int offset, Compile* C ) {
  addF_mulF_reg_load1Node *node = new (C) addF_mulF_reg_load1Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addF_mulF_reg_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLT_REG_mask;
}

// Build CISC version of this instruction
MachNode *addF_mulF_reg_0Node::cisc_version( int offset, Compile* C ) {
  addF_mulF_reg_load1_1Node *node = new (C) addF_mulF_reg_load1_1Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *compI_eRegNode::cisc_version( int offset, Compile* C ) {
  compI_eReg_memNode *node = new (C) compI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compU_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *compU_eRegNode::cisc_version( int offset, Compile* C ) {
  compU_eReg_memNode *node = new (C) compU_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compP_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *compP_eRegNode::cisc_version( int offset, Compile* C ) {
  compP_eReg_memNode *node = new (C) compP_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}

// Build short branch version of this instruction
MachNode *jmpDirNode::short_branch_version(Compile* C) {
  jmpDir_shortNode *node = new (C) jmpDir_shortNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConNode::short_branch_version(Compile* C) {
  jmpCon_shortNode *node = new (C) jmpCon_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndNode::short_branch_version(Compile* C) {
  jmpLoopEnd_shortNode *node = new (C) jmpLoopEnd_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndUNode::short_branch_version(Compile* C) {
  jmpLoopEndU_shortNode *node = new (C) jmpLoopEndU_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndUCFNode::short_branch_version(Compile* C) {
  jmpLoopEndUCF_shortNode *node = new (C) jmpLoopEndUCF_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUNode::short_branch_version(Compile* C) {
  jmpConU_shortNode *node = new (C) jmpConU_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUCFNode::short_branch_version(Compile* C) {
  jmpConUCF_shortNode *node = new (C) jmpConUCF_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUCF2Node::short_branch_version(Compile* C) {
  jmpConUCF2_shortNode *node = new (C) jmpConUCF2_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}


void cmovLL_reg_LTGENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_LTGENode::cisc_version( int offset, Compile* C ) {
  cmovLL_mem_LTGENode *node = new (C) cmovLL_mem_LTGENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovII_reg_LTGENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovII_reg_LTGENode::cisc_version( int offset, Compile* C ) {
  cmovII_mem_LTGENode *node = new (C) cmovII_mem_LTGENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovLL_reg_EQNENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_EQNENode::cisc_version( int offset, Compile* C ) {
  cmovLL_mem_EQNENode *node = new (C) cmovLL_mem_EQNENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovII_reg_EQNENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovII_reg_EQNENode::cisc_version( int offset, Compile* C ) {
  cmovII_mem_EQNENode *node = new (C) cmovII_mem_EQNENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovLL_reg_LEGTNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_LEGTNode::cisc_version( int offset, Compile* C ) {
  cmovLL_mem_LEGTNode *node = new (C) cmovLL_mem_LEGTNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovII_reg_LEGTNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_E_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovII_reg_LEGTNode::cisc_version( int offset, Compile* C ) {
  cmovII_mem_LEGTNode *node = new (C) cmovII_mem_LEGTNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


// Copy _idx, inputs and operands to new node
void MachNode::fill_new_machnode( MachNode* node, Compile* C) const {
  // New node must use same node index
  node->set_idx( _idx );
  // Copy machine-independent inputs
  for( uint j = 0; j < req(); j++ ) {
    node->add_req(in(j));
  }
  // Copy my operands, except for cisc position
  int nopnds = num_opnds();
  assert( node->num_opnds() == (uint)nopnds, "Must have same number of operands");
  MachOper **to = node->_opnds;
  for( int i = 0; i < nopnds; i++ ) {
    if( i != cisc_operand() ) 
      to[i] = _opnds[i]->clone(C);
  }
}

void jmpDirNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEndNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEndUNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEndUCFNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConUNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConUCFNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConUCF2Node::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpDir_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpCon_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEnd_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEndU_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEndUCF_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConU_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConUCF_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConUCF2_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void cmpL_LTGENode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void cmpL_EQNENode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void cmpL_LEGTNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void CallStaticJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallRuntimeDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafNoFPDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

int  loadBNode::reloc()   const {
  return  1;
}

int  loadUBNode::reloc()   const {
  return  1;
}

int  loadCNode::reloc()   const {
  return  1;
}

int  loadINode::reloc()   const {
  return  1;
}

int  loadLNode::reloc()   const {
  return  1;
}

int  loadL_volatileNode::reloc()   const {
  return  1;
}

int  loadLX_volatileNode::reloc()   const {
  return  1;
}

int  loadLX_reg_volatileNode::reloc()   const {
  return  1;
}

int  loadRangeNode::reloc()   const {
  return  1;
}

int  loadPNode::reloc()   const {
  return  1;
}

int  loadKlassNode::reloc()   const {
  return  1;
}

int  loadSNode::reloc()   const {
  return  1;
}

int  loadDNode::reloc()   const {
  return  1;
}

int  loadXDNode::reloc()   const {
  return  1;
}

int  loadXD_partialNode::reloc()   const {
  return  1;
}

int  loadXNode::reloc()   const {
  return  1;
}

int  loadFNode::reloc()   const {
  return  1;
}

int  loadA8BNode::reloc()   const {
  return  1;
}

int  loadA4SNode::reloc()   const {
  return  1;
}

int  loadA4CNode::reloc()   const {
  return  1;
}

int  load2IUNode::reloc()   const {
  return  1;
}

int  loadA2FNode::reloc()   const {
  return  1;
}

int  loadConPNode::reloc()   const {
  return  1;
}

int  loadConFNode::reloc()   const {
  return  2;
}

int  loadConXNode::reloc()   const {
  return  2;
}

int  loadConX0Node::reloc()   const {
  return  2;
}

int  loadConDNode::reloc()   const {
  return  2;
}

int  loadConXDNode::reloc()   const {
  return  2;
}

int  loadConXD0Node::reloc()   const {
  return  2;
}

int  prefetchr0Node::reloc()   const {
  return  1;
}

int  prefetchrNode::reloc()   const {
  return  1;
}

int  prefetchrNTANode::reloc()   const {
  return  1;
}

int  prefetchrT0Node::reloc()   const {
  return  1;
}

int  prefetchrT2Node::reloc()   const {
  return  1;
}

int  prefetchw0Node::reloc()   const {
  return  1;
}

int  prefetchwNode::reloc()   const {
  return  1;
}

int  prefetchwNTANode::reloc()   const {
  return  1;
}

int  prefetchwT0Node::reloc()   const {
  return  1;
}

int  prefetchwT2Node::reloc()   const {
  return  1;
}

int  storeBNode::reloc()   const {
  return  2;
}

int  storeCNode::reloc()   const {
  return  2;
}

int  storeINode::reloc()   const {
  return  2;
}

int  storeLNode::reloc()   const {
  return  2;
}

int  storeL_volatileNode::reloc()   const {
  return  2;
}

int  storeLX_volatileNode::reloc()   const {
  return  2;
}

int  storeLX_reg_volatileNode::reloc()   const {
  return  2;
}

int  storePNode::reloc()   const {
  return  2;
}

int  storeImmINode::reloc()   const {
  return  2;
}

int  storeImmI16Node::reloc()   const {
  return  2;
}

int  storeImmPNode::reloc()   const {
  return  3;
}

int  storeImmBNode::reloc()   const {
  return  2;
}

int  storeA8BNode::reloc()   const {
  return  2;
}

int  storeA4CNode::reloc()   const {
  return  2;
}

int  storeA2INode::reloc()   const {
  return  2;
}

int  storeImmCMNode::reloc()   const {
  return  2;
}

int  storeDNode::reloc()   const {
  return  2;
}

int  storeD_roundedNode::reloc()   const {
  return  2;
}

int  storeXDNode::reloc()   const {
  return  2;
}

int  storeXNode::reloc()   const {
  return  2;
}

int  storeA2FNode::reloc()   const {
  return  2;
}

int  storeFNode::reloc()   const {
  return  2;
}

int  storeF_roundedNode::reloc()   const {
  return  2;
}

int  storeF_DroundedNode::reloc()   const {
  return  2;
}

int  storeF_immNode::reloc()   const {
  return  3;
}

int  storeX_immNode::reloc()   const {
  return  3;
}

int  cmovI_memNode::reloc()   const {
  return  1;
}

int  cmovI_memUNode::reloc()   const {
  return  1;
}

int  cmovI_memUCFNode::reloc()   const {
  return  1;
}

int  addI_eReg_memNode::reloc()   const {
  return  1;
}

int  addI_eReg_mem_0Node::reloc()   const {
  return  1;
}

int  addI_mem_eRegNode::reloc()   const {
  return  3;
}

int  addI_mem_eReg_0Node::reloc()   const {
  return  3;
}

int  addI_mem_immNode::reloc()   const {
  return  3;
}

int  incI_memNode::reloc()   const {
  return  3;
}

int  decI_memNode::reloc()   const {
  return  3;
}

int  loadPLockedNode::reloc()   const {
  return  1;
}

int  loadLLockedNode::reloc()   const {
  return  1;
}

int  loadLX_LockedNode::reloc()   const {
  return  1;
}

int  loadLX_reg_LockedNode::reloc()   const {
  return  1;
}

int  storePConditionalNode::reloc()   const {
  return  1;
}

int  storeIConditionalNode::reloc()   const {
  return  1;
}

int  storeLConditionalNode::reloc()   const {
  return  1;
}

int  subI_eReg_memNode::reloc()   const {
  return  1;
}

int  subI_mem_eRegNode::reloc()   const {
  return  3;
}

int  mulI_mem_immNode::reloc()   const {
  return  1;
}

int  mulINode::reloc()   const {
  return  1;
}

int  mulI_0Node::reloc()   const {
  return  1;
}

int  sarI_mem_1Node::reloc()   const {
  return  3;
}

int  sarI_mem_immNode::reloc()   const {
  return  3;
}

int  andI_eReg_memNode::reloc()   const {
  return  1;
}

int  andI_eReg_mem_0Node::reloc()   const {
  return  1;
}

int  andI_mem_eRegNode::reloc()   const {
  return  3;
}

int  andI_mem_eReg_0Node::reloc()   const {
  return  3;
}

int  andI_mem_immNode::reloc()   const {
  return  3;
}

int  orI_eReg_memNode::reloc()   const {
  return  1;
}

int  orI_eReg_mem_0Node::reloc()   const {
  return  1;
}

int  orI_mem_eRegNode::reloc()   const {
  return  3;
}

int  orI_mem_eReg_0Node::reloc()   const {
  return  3;
}

int  orI_mem_immNode::reloc()   const {
  return  3;
}

int  xorI_eReg_memNode::reloc()   const {
  return  1;
}

int  xorI_eReg_mem_0Node::reloc()   const {
  return  1;
}

int  xorI_mem_eRegNode::reloc()   const {
  return  3;
}

int  xorI_mem_eReg_0Node::reloc()   const {
  return  3;
}

int  xorI_mem_immNode::reloc()   const {
  return  3;
}

int  addL_eReg_memNode::reloc()   const {
  return  1;
}

int  addL_eReg_mem_0Node::reloc()   const {
  return  1;
}

int  subL_eReg_memNode::reloc()   const {
  return  1;
}

int  andL_eReg_memNode::reloc()   const {
  return  1;
}

int  andL_eReg_mem_0Node::reloc()   const {
  return  1;
}

int  orl_eReg_memNode::reloc()   const {
  return  1;
}

int  orl_eReg_mem_0Node::reloc()   const {
  return  1;
}

int  xorl_eReg_memNode::reloc()   const {
  return  1;
}

int  xorl_eReg_mem_0Node::reloc()   const {
  return  1;
}

int  cmpD_0Node::reloc()   const {
  return  1;
}

int  cmpXD_ccmemNode::reloc()   const {
  return  1;
}

int  cmpXD_ccmemCFNode::reloc()   const {
  return  1;
}

int  cmpXD_regmemNode::reloc()   const {
  return  1;
}

int  subD_reg_memNode::reloc()   const {
  return  1;
}

int  addD_reg_memNode::reloc()   const {
  return  1;
}

int  addD_reg_mem_0Node::reloc()   const {
  return  1;
}

int  addD_mem_regNode::reloc()   const {
  return  3;
}

int  addD_mem_reg_0Node::reloc()   const {
  return  3;
}

int  addD_reg_imm1Node::reloc()   const {
  return  1;
}

int  addD_reg_immNode::reloc()   const {
  return  1;
}

int  addD_reg_imm_roundNode::reloc()   const {
  return  1;
}

int  addXD_immNode::reloc()   const {
  return  1;
}

int  addXD_memNode::reloc()   const {
  return  1;
}

int  addXD_mem_0Node::reloc()   const {
  return  1;
}

int  subXD_immNode::reloc()   const {
  return  1;
}

int  subXD_memNode::reloc()   const {
  return  1;
}

int  mulXD_immNode::reloc()   const {
  return  1;
}

int  mulXD_memNode::reloc()   const {
  return  1;
}

int  mulXD_mem_0Node::reloc()   const {
  return  1;
}

int  divXD_immNode::reloc()   const {
  return  1;
}

int  divXD_memNode::reloc()   const {
  return  1;
}

int  mulD_reg_immNode::reloc()   const {
  return  1;
}

int  mulD_reg_memNode::reloc()   const {
  return  1;
}

int  mulD_reg_mem_0Node::reloc()   const {
  return  1;
}

int  mulD_reg_mem_ciscNode::reloc()   const {
  return  1;
}

int  mulD_reg_mem_cisc_0Node::reloc()   const {
  return  1;
}

int  cmpF_0Node::reloc()   const {
  return  1;
}

int  cmpX_ccmemNode::reloc()   const {
  return  1;
}

int  cmpX_ccmemCFNode::reloc()   const {
  return  1;
}

int  cmpX_regmemNode::reloc()   const {
  return  1;
}

int  addX_immNode::reloc()   const {
  return  1;
}

int  addX_memNode::reloc()   const {
  return  1;
}

int  addX_mem_0Node::reloc()   const {
  return  1;
}

int  subX_immNode::reloc()   const {
  return  1;
}

int  subX_memNode::reloc()   const {
  return  1;
}

int  mulX_immNode::reloc()   const {
  return  1;
}

int  mulX_memNode::reloc()   const {
  return  1;
}

int  mulX_mem_0Node::reloc()   const {
  return  1;
}

int  divX_immNode::reloc()   const {
  return  1;
}

int  divX_memNode::reloc()   const {
  return  1;
}

int  sqrtX_memNode::reloc()   const {
  return  1;
}

int  sqrtXD_memNode::reloc()   const {
  return  1;
}

int  addF24_reg_memNode::reloc()   const {
  return  1;
}

int  addF24_reg_mem_0Node::reloc()   const {
  return  1;
}

int  addF_reg_memNode::reloc()   const {
  return  1;
}

int  addF_reg_mem_0Node::reloc()   const {
  return  1;
}

int  addF24_mem_regNode::reloc()   const {
  return  1;
}

int  addF24_mem_ciscNode::reloc()   const {
  return  2;
}

int  addF24_mem_cisc_0Node::reloc()   const {
  return  2;
}

int  addF24_mem_memNode::reloc()   const {
  return  2;
}

int  addF24_reg_immNode::reloc()   const {
  return  1;
}

int  addF_reg_immNode::reloc()   const {
  return  1;
}

int  mulF24_reg_memNode::reloc()   const {
  return  1;
}

int  mulF24_reg_mem_0Node::reloc()   const {
  return  1;
}

int  mulF_reg_memNode::reloc()   const {
  return  1;
}

int  mulF_reg_mem_0Node::reloc()   const {
  return  1;
}

int  mulF24_mem_memNode::reloc()   const {
  return  2;
}

int  mulF24_reg_immNode::reloc()   const {
  return  1;
}

int  mulF_reg_immNode::reloc()   const {
  return  1;
}

int  mulF_reg_load1Node::reloc()   const {
  return  1;
}

int  mulF_reg_load1_0Node::reloc()   const {
  return  1;
}

int  addF_mulF_reg_load1Node::reloc()   const {
  return  1;
}

int  addF_mulF_reg_load1_1Node::reloc()   const {
  return  1;
}

int  addF_mulF_reg_load1_0Node::reloc()   const {
  return  1;
}

int  addF_mulF_reg_load1_2Node::reloc()   const {
  return  1;
}

int  convI2XD_memNode::reloc()   const {
  return  1;
}

int  convI2D_memNode::reloc()   const {
  return  1;
}

int  convI2F_SSF_memNode::reloc()   const {
  return  1;
}

int  convI2F_memNode::reloc()   const {
  return  1;
}

int  Repl2F_immXF0Node::reloc()   const {
  return  1;
}

int  compI_eReg_memNode::reloc()   const {
  return  1;
}

int  testI_reg_memNode::reloc()   const {
  return  1;
}

int  compU_eReg_memNode::reloc()   const {
  return  1;
}

int  compP_eReg_immNode::reloc()   const {
  return  1;
}

int  compP_eReg_memNode::reloc()   const {
  return  1;
}

int  compP_mem_eRegNode::reloc()   const {
  return  1;
}

int  testP_regNode::reloc()   const {
  return  1;
}

int  testP_Reg_memNode::reloc()   const {
  return  1;
}

int  partialSubtypeCheck_vs_ZeroNode::reloc()   const {
  return  1;
}

int  cmovLL_mem_LTGENode::reloc()   const {
  return  1;
}

int  cmovII_mem_LTGENode::reloc()   const {
  return  1;
}

int  cmovLL_mem_EQNENode::reloc()   const {
  return  1;
}

int  cmovII_mem_EQNENode::reloc()   const {
  return  1;
}

int  cmovLL_mem_LEGTNode::reloc()   const {
  return  1;
}

int  cmovII_mem_LEGTNode::reloc()   const {
  return  1;
}

int  CallStaticJavaDirectNode::reloc()   const {
  return  1;
}

int  CallDynamicJavaDirectNode::reloc()   const {
  return  1;
}

int  CallRuntimeDirectNode::reloc()   const {
  return  1;
}

int  CallLeafDirectNode::reloc()   const {
  return  1;
}

int  CallLeafNoFPDirectNode::reloc()   const {
  return  1;
}

int  RetNode::reloc()   const {
  return  1;
}

int  TailCalljmpIndNode::reloc()   const {
  return  1;
}

int  tailjmpIndNode::reloc()   const {
  return  1;
}

int  RethrowExceptionNode::reloc()   const {
  return  1;
}

int  safePoint_pollNode::reloc()   const {
  return  1;
}


void  bytes_reverse_intNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_cc(cbuf, (0xC8 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  bytes_reverse_longNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int destlo = opnd_array(1)->reg(ra_,this,idx1);
    int desthi = HIGH_FROM_LOW(destlo);
    // bswap lo
    emit_opcode(cbuf, 0x0F);
    emit_cc(cbuf, 0xC8, destlo);
    // bswap hi
    emit_opcode(cbuf, 0x0F);
    emit_cc(cbuf, 0xC8, desthi);
    // xchg lo and hi
    emit_opcode(cbuf, 0x87);
    emit_rm(cbuf, 0x3, destlo, desthi);
  }
}
void  loadBNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xBE /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadUBNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xB6 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadCNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xB7 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x8B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x8B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x8B /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this));  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(1)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  loadL_volatileNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,0xDF);
    int rm_byte_opcode = 0x05;
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
    store_to_stackslot( cbuf, 0x0DF, 0x07, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  loadLX_volatileNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    { // Atomic long load
      // UseXmmLoadAndClearUpper ? movsd $tmp,$mem : movlpd $tmp,$mem
      emit_opcode(cbuf,UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,UseXmmLoadAndClearUpper ? 0x10 : 0x12);
      int base     = opnd_array(1)->base(ra_,this,idx1);
      int index    = opnd_array(1)->index(ra_,this,idx1);
      int scale    = opnd_array(1)->scale();
      int displace = opnd_array(1)->disp(ra_,this,idx1);
      bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
      encode_RegMem(cbuf, opnd_array(2)->reg(ra_,this,idx2), base, index, scale, displace, disp_is_oop);
    }
    { // MOVSD $dst,$tmp ! atomic long store
      emit_opcode(cbuf,0xF2);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x11);
      int base     = opnd_array(0)->base(ra_,this,idx0);
      int index    = opnd_array(0)->index(ra_,this,idx0);
      int scale    = opnd_array(0)->scale();
      int displace = opnd_array(0)->disp(ra_,this,0);
      bool disp_is_oop = opnd_array(0)->disp_is_oop(); // disp-as-oop when working with static globals
      encode_RegMem(cbuf, opnd_array(2)->reg(ra_,this,idx2), base, index, scale, displace, disp_is_oop);
    }
  }
}
void  loadLX_reg_volatileNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    { // Atomic long load
      // UseXmmLoadAndClearUpper ? movsd $tmp,$mem : movlpd $tmp,$mem
      emit_opcode(cbuf,UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,UseXmmLoadAndClearUpper ? 0x10 : 0x12);
      int base     = opnd_array(1)->base(ra_,this,idx1);
      int index    = opnd_array(1)->index(ra_,this,idx1);
      int scale    = opnd_array(1)->scale();
      int displace = opnd_array(1)->disp(ra_,this,idx1);
      bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
      encode_RegMem(cbuf, opnd_array(2)->reg(ra_,this,idx2), base, index, scale, displace, disp_is_oop);
    }
    { // MOVD $dst.lo,$tmp
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x7E);
      emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2), opnd_array(0)->reg(ra_,this));
    }
    { // PSRLQ $tmp,32
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x73);
      emit_rm(cbuf, 0x3, 0x02, opnd_array(2)->reg(ra_,this,idx2));
      emit_d8(cbuf, 0x20);
    }
    { // MOVD $dst.hi,$tmp
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x7E);
      emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2), HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)));
    }
  }
}
void  loadRangeNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x8B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x8B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadKlassNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x8B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadSNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xBF /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0xDD /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  loadXDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x10));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadXD_partialNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x66));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x12));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadXNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x10));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0xD9 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  loadA8BNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(as_XMMRegister(opnd_array(0)->reg(ra_,this)), madr);
  }
}
void  loadA4SNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(as_XMMRegister(opnd_array(0)->reg(ra_,this)), madr);
  }
}
void  loadA4CNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(as_XMMRegister(opnd_array(0)->reg(ra_,this)), madr);
  }
}
void  load2IUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(as_XMMRegister(opnd_array(0)->reg(ra_,this)), madr);
  }
}
void  loadA2FNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(as_XMMRegister(opnd_array(0)->reg(ra_,this)), madr);
  }
}
void  leaP8Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x8D /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  leaP32Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x8D /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  leaPIdxOffNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x8D /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  leaPIdxScaleNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x8D /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  leaPIdxScaleOffNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x8D /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadConINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    emit_opcode(cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32(cbuf, opnd_array(1)->constant());
  }
}
void  loadConI0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0x33 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(0)->reg(ra_,this));
  }
}
void  loadConPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    emit_opcode(cbuf, (0xB8 /*primary()*/) + opnd_array(0)->reg(ra_,this));
    
if ( opnd_array(1)->constant_is_oop() ) {
  emit_d32_reloc(cbuf, opnd_array(1)->constant(), relocInfo::oop_type, 0);
} else {
  emit_d32(cbuf, opnd_array(1)->constant());
};
  }
}
void  loadConLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    int dst_enc = opnd_array(0)->reg(ra_,this);
    int src_con = opnd_array(1)->constantL() & 0x0FFFFFFFFL;
    if (src_con == 0) {
      // xor dst, dst
      emit_opcode(cbuf, 0x33);
      emit_rm(cbuf, 0x3, dst_enc, dst_enc);
    } else {
      emit_opcode(cbuf, (0xB8 /*primary()*/) + dst_enc);
      emit_d32(cbuf, src_con);
    }
  }
  {
    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    int dst_enc = opnd_array(0)->reg(ra_,this) + 2;
    int src_con = ((julong)(opnd_array(1)->constantL())) >> 32;
    if (src_con == 0) {
      // xor dst, dst
      emit_opcode(cbuf, 0x33);
      emit_rm(cbuf, 0x3, dst_enc, dst_enc);
    } else {
      emit_opcode(cbuf, (0xB8 /*primary()*/) + dst_enc);
      emit_d32(cbuf, src_con);
    }
  }
}
void  loadConL0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_d8(cbuf, (0x33 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(0)->reg(ra_,this));
  }
  {
    emit_d8(cbuf, (0x33 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)), HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)));
  }
}
void  loadConFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if( is_positive_zero_float(opnd_array(1)->constantF())) {
      emit_opcode(cbuf,0xD9);
      emit_opcode(cbuf,0xEE);
    } else if( is_positive_one_float(opnd_array(1)->constantF())) {
      emit_opcode(cbuf,0xD9);
      emit_opcode(cbuf,0xE8);
    } else {
      emit_d8(cbuf, (0xD9 /*primary()*/));
      // Load immediate does not have a zero or sign extended version
      // for 8-bit immediates
      // First load to TOS, then move to dst
      emit_rm(cbuf, 0x0, 0x0, 0x5);
      emit_float_constant(cbuf, opnd_array(1)->constantF());
    }
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  loadConXNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x10));
  }
  {
    emit_rm(cbuf, 0x0, opnd_array(0)->reg(ra_,this), 0x5);
    emit_float_constant(cbuf, opnd_array(1)->constantF());
  }
}
void  loadConX0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x57));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(0)->reg(ra_,this));
  }
}
void  loadConDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    if( is_positive_zero_double(opnd_array(1)->constantD())) {
      // FLDZ
      emit_opcode(cbuf,0xD9);
      emit_opcode(cbuf,0xEE);
    } else if( is_positive_one_double(opnd_array(1)->constantD())) {
      // FLD1
      emit_opcode(cbuf,0xD9);
      emit_opcode(cbuf,0xE8);
    } else {
      emit_opcode(cbuf,0xDD);
      emit_rm(cbuf, 0x0, 0x0, 0x5);
      emit_double_constant(cbuf, opnd_array(1)->constantD());
    }
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  loadConXDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // UseXmmLoadAndClearUpper ? movsd(dst, con) : movlpd(dst, con)
    emit_opcode(cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
    emit_rm(cbuf, 0x0, opnd_array(0)->reg(ra_,this), 0x5);
    emit_double_constant(cbuf, opnd_array(1)->constantD());
  }
}
void  loadConXD0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0x66));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x57));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(0)->reg(ra_,this));
  }
}
void  loadSSINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0x8B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadSSLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0x8B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x8B /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this));  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(1)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  loadSSPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0x8B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadSSFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xD9 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( !opnd_array(1)->disp_is_oop(), "No oops here because no relo info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, false);
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  loadSSDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xDD /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( !opnd_array(1)->disp_is_oop(), "No oops here because no relo info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, false);
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  prefetchr0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  // User did not define which encode class to use.
}
uint  prefetchr0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  prefetchrNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x0d /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  prefetchrNTANode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x18 /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  prefetchrT0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x18 /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x01);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  prefetchrT2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x18 /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x03);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  prefetchw0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  // User did not define which encode class to use.
}
uint  prefetchw0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  prefetchwNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x0D /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x01);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  prefetchwNTANode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x18 /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  prefetchwT0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x18 /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x01);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  prefetchwT2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x18 /*secondary()*/));
  }
  {
    int rm_byte_opcode = (0x03);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  storeBNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x88 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  storeCNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x66 /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0x89 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  storeINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x89 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  storeLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x89 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x89 /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2));  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(1)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  storeL_volatileNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x3B /*primary()*/));
  }
  {
    int reg_encoding = (EAX_enc);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    store_to_stackslot( cbuf, 0x0DF, 0x05, opnd_array(2)->disp(ra_,this,idx2) );
    cbuf.set_inst_mark();            // Mark start of FIST in case $mem has an oop
    emit_opcode(cbuf,0xDF);
    int rm_byte_opcode = 0x07;
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  storeLX_volatileNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode(cbuf,(0x3B /*primary()*/));
  }
  {
    int reg_encoding = (EAX_enc);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    { // Atomic long load
      // UseXmmLoadAndClearUpper ? movsd $tmp,[$src] : movlpd $tmp,[$src]
      emit_opcode(cbuf,UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,UseXmmLoadAndClearUpper ? 0x10 : 0x12);
      int base     = opnd_array(2)->base(ra_,this,idx2);
      int index    = opnd_array(2)->index(ra_,this,idx2);
      int scale    = opnd_array(2)->scale();
      int displace = opnd_array(2)->disp(ra_,this,idx2);
      bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
      encode_RegMem(cbuf, opnd_array(3)->reg(ra_,this,idx3), base, index, scale, displace, disp_is_oop);
    }
    cbuf.set_inst_mark();            // Mark start of MOVSD in case $mem has an oop
    { // MOVSD $mem,$tmp ! atomic long store
      emit_opcode(cbuf,0xF2);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x11);
      int base     = opnd_array(1)->base(ra_,this,idx1);
      int index    = opnd_array(1)->index(ra_,this,idx1);
      int scale    = opnd_array(1)->scale();
      int displace = opnd_array(1)->disp(ra_,this,idx1);
      bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
      encode_RegMem(cbuf, opnd_array(3)->reg(ra_,this,idx3), base, index, scale, displace, disp_is_oop);
    }
  }
}
void  storeLX_reg_volatileNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_opcode(cbuf,(0x3B /*primary()*/));
  }
  {
    int reg_encoding = (EAX_enc);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    { // MOVD $tmp,$src.lo
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x6E);
      emit_rm(cbuf, 0x3, opnd_array(4)->reg(ra_,this,idx4), opnd_array(2)->reg(ra_,this,idx2));
    }
    { // MOVD $tmp2,$src.hi
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x6E);
      emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)));
    }
    { // PUNPCKLDQ $tmp,$tmp2
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x62);
      emit_rm(cbuf, 0x3, opnd_array(4)->reg(ra_,this,idx4), opnd_array(3)->reg(ra_,this,idx3));
    }
    cbuf.set_inst_mark();            // Mark start of MOVSD in case $mem has an oop
    { // MOVSD $mem,$tmp ! atomic long store
      emit_opcode(cbuf,0xF2);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x11);
      int base     = opnd_array(1)->base(ra_,this,idx1);
      int index    = opnd_array(1)->index(ra_,this,idx1);
      int scale    = opnd_array(1)->scale();
      int displace = opnd_array(1)->disp(ra_,this,idx1);
      bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
      encode_RegMem(cbuf, opnd_array(4)->reg(ra_,this,idx4), base, index, scale, displace, disp_is_oop);
    }
  }
}
void  storePNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x89 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  storeImmINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xC7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  }
}
void  storeImmI16Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,0x66);
  }
  {
    emit_opcode(cbuf,(0xC7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Output immediate
    emit_d16(cbuf, opnd_array(2)->constant());
  }
}
void  storeImmPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xC7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Output immediate
    
if ( opnd_array(2)->constant_is_oop() ) {
  emit_d32_reloc(cbuf, opnd_array(2)->constant(), relocInfo::oop_type, 0);
} else {
  emit_d32(cbuf, opnd_array(2)->constant());
};
  }
}
void  storeImmBNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xC6 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  storeA8BNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(madr, as_XMMRegister(opnd_array(2)->reg(ra_,this,idx2)));
  }
}
void  storeA4CNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(madr, as_XMMRegister(opnd_array(2)->reg(ra_,this,idx2)));
  }
}
void  storeA2INode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(madr, as_XMMRegister(opnd_array(2)->reg(ra_,this,idx2)));
  }
}
void  storeImmCMNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xC6 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  storeDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2) != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );
    }
    cbuf.set_inst_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xDD /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  storeD_roundedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2) != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );
    }
    cbuf.set_inst_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xDD /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  storeXDNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x11));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  storeXNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x11));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  storeA2FNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);
    Address madr = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ movq(madr, as_XMMRegister(opnd_array(2)->reg(ra_,this,idx2)));
  }
}
void  storeFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2) != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );
    }
    cbuf.set_inst_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xD9 /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  storeF_roundedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2) != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );
    }
    cbuf.set_inst_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xD9 /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  storeF_DroundedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2) != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );
    }
    cbuf.set_inst_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xD9 /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  storeF_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xC7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantF();
    int    jf_as_bits = jint_cast( jf );
    emit_d32(cbuf, jf_as_bits);
  }
}
void  storeX_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xC7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantF();
    int    jf_as_bits = jint_cast( jf );
    emit_d32(cbuf, jf_as_bits);
  }
}
void  storeSSINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    store_to_stackslot( cbuf, (0x89 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(0)->disp(ra_,this,0) );
  }
}
void  storeSSPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    store_to_stackslot( cbuf, (0x89 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(0)->disp(ra_,this,0) );
  }
}
void  storeSSLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0x89 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x89 /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1));  // Hi register of pair, computed from lo
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(0)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  membar_acquireNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Doug Lea believes this is not needed with current Sparcs and TSO.
    // MacroAssembler masm(&cbuf);
    // masm.membar();
  }
}
uint  membar_acquireNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  membar_acquire_lockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  membar_acquire_lockNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  membar_releaseNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Doug Lea believes this is not needed with current Sparcs and TSO.
    // MacroAssembler masm(&cbuf);
    // masm.membar();
  }
}
uint  membar_releaseNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  membar_release_lockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  membar_release_lockNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  membar_volatileNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler masm(&cbuf);
    masm.membar(Assembler::Membar_mask_bits(Assembler::StoreLoad |
                                            Assembler::StoreStore));
  }
}
void  unnecessary_membar_volatileNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  unnecessary_membar_volatileNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  castX2PNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
void  castP2XNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  cmovI_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
}
void  cmovI_regUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
}
void  cmovI_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3);
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  cmovI_memUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3);
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  cmovP_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
}
void  cmovP_reg_nonP6Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // Invert sense of branch from sense of CMOV
    emit_cc( cbuf, 0x70, (opnd_array(1)->ccode()^1) );
    emit_d8( cbuf, (0x2) );
  }
  {
    emit_opcode(cbuf,(0x8b /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
}
void  cmovP_regUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
}
void  fcmovD_regUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = 0xDA00 + opnd_array(1)->ccode() + (opnd_array(4)->reg(ra_,this,idx4)-1);
    emit_d8(cbuf, op >> 8 );
    emit_d8(cbuf, op & 255);
  }
}
void  fcmovF_regUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    int op = 0xDA00 + opnd_array(1)->ccode() + (opnd_array(4)->reg(ra_,this,idx4)-1);
    emit_d8(cbuf, op >> 8 );
    emit_d8(cbuf, op & 255);
  }
}
void  fcmovD_regSNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // Invert sense of branch from sense of CMOV
    emit_cc( cbuf, 0x70, (opnd_array(1)->ccode()^1) );
    emit_d8( cbuf, (0x4) );
  }
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(4)->reg(ra_,this,idx4) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xdd /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(3)->reg(ra_,this,idx3) );
  }
}
void  fcmovF_regSNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // Invert sense of branch from sense of CMOV
    emit_cc( cbuf, 0x70, (opnd_array(1)->ccode()^1) );
    emit_d8( cbuf, (0x4) );
  }
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(4)->reg(ra_,this,idx4) );
  }
  {
    emit_opcode(cbuf,(0xdd /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(3)->reg(ra_,this,idx3) );
  }
}
void  fcmovX_regSNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3), opnd_array(4)->as_XMMRegister(ra_,this,idx4));
    __ bind(skip);
  }
}
void  fcmovXD_regSNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3), opnd_array(4)->as_XMMRegister(ra_,this,idx4));
    __ bind(skip);
  }
}
void  fcmovX_regUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3), opnd_array(4)->as_XMMRegister(ra_,this,idx4));
    __ bind(skip);
  }
}
void  fcmovXD_regUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3), opnd_array(4)->as_XMMRegister(ra_,this,idx4));
    __ bind(skip);
  }
}
void  cmovL_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)));
  }
}
void  cmovL_regUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)));
  }
}
void  addI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x03 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  addI_eRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  addI_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  incI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0x40 /*primary()*/) + opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  incI_eRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 1, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 1);
}
void  leaI_eReg_immINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x8D /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base         = opnd_array(1)->reg(ra_,this,idx1);      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constant(); // 0x00 indicates no displacement
    bool disp_is_oop = false;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  leaP_eReg_immINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x8D /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base         = opnd_array(1)->reg(ra_,this,idx1);      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constant(); // 0x00 indicates no displacement
    bool disp_is_oop = false;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  decI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, (0x48 /*primary()*/) + opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  decI_eRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 1, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 1);
}
void  addP_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x03 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  addP_eRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  addP_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  addI_eReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x03 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  addI_eReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x03 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  addI_mem_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x01 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  addI_mem_eReg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x01 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  addI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  incI_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xFF /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  decI_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xFF /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x01);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  checkCastPPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  checkCastPPNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  castPPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
void  castIINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
void  loadPLockedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0x8B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  loadLLockedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,0xDF);
    int rm_byte_opcode = 0x05;
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
    store_to_stackslot( cbuf, 0x0DF, 0x07, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  loadLX_LockedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    { // Atomic long load
      // UseXmmLoadAndClearUpper ? movsd $tmp,$mem : movlpd $tmp,$mem
      emit_opcode(cbuf,UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,UseXmmLoadAndClearUpper ? 0x10 : 0x12);
      int base     = opnd_array(1)->base(ra_,this,idx1);
      int index    = opnd_array(1)->index(ra_,this,idx1);
      int scale    = opnd_array(1)->scale();
      int displace = opnd_array(1)->disp(ra_,this,idx1);
      bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
      encode_RegMem(cbuf, opnd_array(2)->reg(ra_,this,idx2), base, index, scale, displace, disp_is_oop);
    }
    { // MOVSD $dst,$tmp ! atomic long store
      emit_opcode(cbuf,0xF2);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x11);
      int base     = opnd_array(0)->base(ra_,this,idx0);
      int index    = opnd_array(0)->index(ra_,this,idx0);
      int scale    = opnd_array(0)->scale();
      int displace = opnd_array(0)->disp(ra_,this,0);
      bool disp_is_oop = opnd_array(0)->disp_is_oop(); // disp-as-oop when working with static globals
      encode_RegMem(cbuf, opnd_array(2)->reg(ra_,this,idx2), base, index, scale, displace, disp_is_oop);
    }
  }
}
void  loadLX_reg_LockedNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    { // Atomic long load
      // UseXmmLoadAndClearUpper ? movsd $tmp,$mem : movlpd $tmp,$mem
      emit_opcode(cbuf,UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,UseXmmLoadAndClearUpper ? 0x10 : 0x12);
      int base     = opnd_array(1)->base(ra_,this,idx1);
      int index    = opnd_array(1)->index(ra_,this,idx1);
      int scale    = opnd_array(1)->scale();
      int displace = opnd_array(1)->disp(ra_,this,idx1);
      bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
      encode_RegMem(cbuf, opnd_array(2)->reg(ra_,this,idx2), base, index, scale, displace, disp_is_oop);
    }
    { // MOVD $dst.lo,$tmp
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x7E);
      emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2), opnd_array(0)->reg(ra_,this));
    }
    { // PSRLQ $tmp,32
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x73);
      emit_rm(cbuf, 0x3, 0x02, opnd_array(2)->reg(ra_,this,idx2));
      emit_d8(cbuf, 0x20);
    }
    { // MOVD $dst.hi,$tmp
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x7E);
      emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2), HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)));
    }
  }
}
void  storePConditionalNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if( os::is_MP() )
      emit_opcode(cbuf,0xF0);         // [Lock]
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0xB1));
  }
  {
    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  storeIConditionalNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    if( os::is_MP() )
      emit_opcode(cbuf,0xF0);         // [Lock]
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0xB1));
  }
  {
    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  storeLConditionalNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    // Note: we need to swap rbx, and rcx before and after the
    //       cmpxchg8 instruction because the instruction uses
    //       rcx as the high order word of the new value to store but
    //       our register encoding uses rbx.
    __ xchgl(as_Register(EBX_enc), as_Register(ECX_enc));
    if( os::is_MP() )
      __ lock();
    __ cmpxchg8(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)));
    __ xchgl(as_Register(EBX_enc), as_Register(ECX_enc));
  }
}
void  compareAndSwapLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
    // [Lock]
    if( os::is_MP() )
      emit_opcode(cbuf,0xF0);
    // CMPXCHG8 [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xC7);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1) );
    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
  }
  {
    int res_encoding = opnd_array(0)->reg(ra_,this);

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  }
}
void  compareAndSwapPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // [Lock]
    if( os::is_MP() )
      emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    int res_encoding = opnd_array(0)->reg(ra_,this);

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  }
}
void  compareAndSwapINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // [Lock]
    if( os::is_MP() )
      emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    int res_encoding = opnd_array(0)->reg(ra_,this);

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  }
}
void  subI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x2B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  subI_eRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  subI_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  subI_eReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x2B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  subI_mem_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x29 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  subP_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode(cbuf,(0x2B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(3)->reg(ra_,this,idx3));
  }
}
uint  subP_eRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  negI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF7 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x03 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  negI_eRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  mulI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xAF /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  mulI_eRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 3);
}
void  mulI_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  loadConL_low_onlyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    int dst_enc = opnd_array(0)->reg(ra_,this);
    int src_con = opnd_array(1)->constantL() & 0x0FFFFFFFFL;
    if (src_con == 0) {
      // xor dst, dst
      emit_opcode(cbuf, 0x33);
      emit_rm(cbuf, 0x3, dst_enc, dst_enc);
    } else {
      emit_opcode(cbuf, (0xB8 /*primary()*/) + dst_enc);
      emit_d32(cbuf, src_con);
    }
  }
}
void  mulI_imm_highNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(1)->reg(ra_,this,idx1) );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this) );
      emit_d8(cbuf, shift_count);
    }
  }
}
void  mulI_imm_high_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(2)->reg(ra_,this,idx2) );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this) );
      emit_d8(cbuf, shift_count);
    }
  }
}
void  mulI_imm_RShift_highNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(1)->reg(ra_,this,idx1) );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this) );
      emit_d8(cbuf, shift_count);
    }
  }
}
void  mulI_imm_RShift_high_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(2)->reg(ra_,this,idx2) );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this) );
      emit_d8(cbuf, shift_count);
    }
  }
}
void  mulI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  mulINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xAF /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  mulI_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xAF /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  mulI2LNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Basic idea: long = (long)int * (long)int
    // IMUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  mulI2L_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Basic idea: long = (long)int * (long)int
    // IMUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  mulIS_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // Basic Idea:  long = (int & 0xffffffffL) * (int & 0xffffffffL)
    // MUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, opnd_array(3)->reg(ra_,this,idx3));
  }
}
void  mulIS_eReg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // Basic Idea:  long = (int & 0xffffffffL) * (int & 0xffffffffL)
    // MUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  mulL_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // Basic idea: lo(result) = lo(x_lo * y_lo)
    //             hi(result) = hi(x_lo * y_lo) + lo(x_hi * y_lo) + lo(x_lo * y_hi)
    // MOV    $tmp,$src.lo
    encode_Copy( cbuf, opnd_array(3)->reg(ra_,this,idx3), opnd_array(2)->reg(ra_,this,idx2) );
    // IMUL   $tmp,EDX
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0xAF );
    emit_rm( cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)) );
    // MOV    EDX,$src.hi
    encode_Copy( cbuf, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)) );
    // IMUL   EDX,EAX
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0xAF );
    emit_rm( cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), opnd_array(1)->reg(ra_,this,idx1) );
    // ADD    $tmp,EDX
    emit_opcode( cbuf, 0x03 );
    emit_rm( cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)) );
    // MUL   EDX:EAX,$src.lo
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, opnd_array(2)->reg(ra_,this,idx2) );
    // ADD    EDX,ESI
    emit_opcode( cbuf, 0x03 );
    emit_rm( cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), opnd_array(3)->reg(ra_,this,idx3) );
  }
}
void  mulL_eReg_conNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // Basic idea: lo(result) = lo(src * y_lo)
    //             hi(result) = hi(src * y_lo) + lo(src * y_hi)
    // IMUL   $tmp,EDX,$src
    emit_opcode( cbuf, 0x6B );
    emit_rm( cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)) );
    emit_d8( cbuf, (int)opnd_array(2)->constantL() );
    // MOV    EDX,$src
    emit_opcode(cbuf, 0xB8 + EDX_enc);
    emit_d32( cbuf, (int)opnd_array(2)->constantL() );
    // MUL   EDX:EAX,EDX
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, EDX_enc );
    // ADD    EDX,ESI
    emit_opcode( cbuf, 0x03 );
    emit_rm( cbuf, 0x3, EDX_enc, opnd_array(3)->reg(ra_,this,idx3) );
  }
}
uint  mulL_eReg_conNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 12);
}
void  divI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax,: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax,: quotient  (= rax, idiv reg)       min_int
    //         rdx: remainder (= rax, irem reg)       0
    //
    //  Code sequnce:
    //
    //  81 F8 00 00 00 80    cmp         rax,80000000h
    //  0F 85 0B 00 00 00    jne         normal_case
    //  33 D2                xor         rdx,edx
    //  83 F9 FF             cmp         rcx,0FFh
    //  0F 84 03 00 00 00    je          done
    //                  normal_case:
    //  99                   cdq
    //  F7 F9                idiv        rax,ecx
    //                  done:
    //
    emit_opcode(cbuf,0x81); emit_d8(cbuf,0xF8);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x80);                     // cmp rax,80000000h
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x85);
    emit_opcode(cbuf,0x0B); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // jne normal_case
    emit_opcode(cbuf,0x33); emit_d8(cbuf,0xD2);                     // xor rdx,edx
    emit_opcode(cbuf,0x83); emit_d8(cbuf,0xF9); emit_d8(cbuf,0xFF); // cmp rcx,0FFh
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x84);
    emit_opcode(cbuf,0x03); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // je done
    // normal_case:
    emit_opcode(cbuf,0x99);                                         // cdq
    // idiv (note: must be emitted by the user of this rule)
    // normal:
  }
  {
    emit_opcode(cbuf,(0xF7 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  divI_eRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 26, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 26);
}
void  divL_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // PUSH src1.hi
    emit_opcode(cbuf, HIGH_FROM_LOW(0x50+opnd_array(1)->reg(ra_,this,idx1)) );
    // PUSH src1.lo
    emit_opcode(cbuf,               0x50+opnd_array(1)->reg(ra_,this,idx1)  );
    // PUSH src2.hi
    emit_opcode(cbuf, HIGH_FROM_LOW(0x50+opnd_array(2)->reg(ra_,this,idx2)) );
    // PUSH src2.lo
    emit_opcode(cbuf,               0x50+opnd_array(2)->reg(ra_,this,idx2)  );
    // CALL directly to the runtime
    cbuf.set_inst_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (CAST_FROM_FN_PTR(address, SharedRuntime::ldiv) - cbuf.code_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Restore stack
    emit_opcode(cbuf, 0x83); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 4*4);
  }
}
void  divModI_eReg_divmodNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax,: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax,: quotient  (= rax, idiv reg)       min_int
    //         rdx: remainder (= rax, irem reg)       0
    //
    //  Code sequnce:
    //
    //  81 F8 00 00 00 80    cmp         rax,80000000h
    //  0F 85 0B 00 00 00    jne         normal_case
    //  33 D2                xor         rdx,edx
    //  83 F9 FF             cmp         rcx,0FFh
    //  0F 84 03 00 00 00    je          done
    //                  normal_case:
    //  99                   cdq
    //  F7 F9                idiv        rax,ecx
    //                  done:
    //
    emit_opcode(cbuf,0x81); emit_d8(cbuf,0xF8);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x80);                     // cmp rax,80000000h
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x85);
    emit_opcode(cbuf,0x0B); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // jne normal_case
    emit_opcode(cbuf,0x33); emit_d8(cbuf,0xD2);                     // xor rdx,edx
    emit_opcode(cbuf,0x83); emit_d8(cbuf,0xF9); emit_d8(cbuf,0xFF); // cmp rcx,0FFh
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x84);
    emit_opcode(cbuf,0x03); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // je done
    // normal_case:
    emit_opcode(cbuf,0x99);                                         // cdq
    // idiv (note: must be emitted by the user of this rule)
    // normal:
  }
  {
    emit_opcode(cbuf,(0xF7 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  divModI_eReg_divmodNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 26, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 26);
}
void  modI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax,: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax,: quotient  (= rax, idiv reg)       min_int
    //         rdx: remainder (= rax, irem reg)       0
    //
    //  Code sequnce:
    //
    //  81 F8 00 00 00 80    cmp         rax,80000000h
    //  0F 85 0B 00 00 00    jne         normal_case
    //  33 D2                xor         rdx,edx
    //  83 F9 FF             cmp         rcx,0FFh
    //  0F 84 03 00 00 00    je          done
    //                  normal_case:
    //  99                   cdq
    //  F7 F9                idiv        rax,ecx
    //                  done:
    //
    emit_opcode(cbuf,0x81); emit_d8(cbuf,0xF8);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x80);                     // cmp rax,80000000h
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x85);
    emit_opcode(cbuf,0x0B); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // jne normal_case
    emit_opcode(cbuf,0x33); emit_d8(cbuf,0xD2);                     // xor rdx,edx
    emit_opcode(cbuf,0x83); emit_d8(cbuf,0xF9); emit_d8(cbuf,0xFF); // cmp rcx,0FFh
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x84);
    emit_opcode(cbuf,0x03); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // je done
    // normal_case:
    emit_opcode(cbuf,0x99);                                         // cdq
    // idiv (note: must be emitted by the user of this rule)
    // normal:
  }
  {
    emit_opcode(cbuf,(0xF7 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
uint  modI_eRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 26, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 26);
}
void  modL_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // PUSH src1.hi
    emit_opcode(cbuf, HIGH_FROM_LOW(0x50+opnd_array(1)->reg(ra_,this,idx1)) );
    // PUSH src1.lo
    emit_opcode(cbuf,               0x50+opnd_array(1)->reg(ra_,this,idx1)  );
    // PUSH src2.hi
    emit_opcode(cbuf, HIGH_FROM_LOW(0x50+opnd_array(2)->reg(ra_,this,idx2)) );
    // PUSH src2.lo
    emit_opcode(cbuf,               0x50+opnd_array(2)->reg(ra_,this,idx2)  );
    // CALL directly to the runtime
    cbuf.set_inst_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (CAST_FROM_FN_PTR(address, SharedRuntime::lrem ) - cbuf.code_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Restore stack
    emit_opcode(cbuf, 0x83); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 4*4);
  }
}
void  shlI_eReg_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD1 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
uint  shlI_eReg_1Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  salI_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
uint  salI_eReg_immNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 3);
}
void  salI_eReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD3 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
uint  salI_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  sarI_eReg_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD1 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
uint  sarI_eReg_1Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  sarI_mem_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x7 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  sarI_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
uint  sarI_eReg_immNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 3);
}
void  sarI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xC1 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x7 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  sarI_eReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD3 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
uint  sarI_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  shrI_eReg_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD1 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
uint  shrI_eReg_1Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  shrI_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
uint  shrI_eReg_immNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 3);
}
void  i2bNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xBE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  i2bNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 3);
}
void  i2sNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xBF /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  i2sNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 3);
}
void  shrI_eReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD3 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
uint  shrI_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  andI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x23 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  andI_eRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  andI_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  andI_eReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x23 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  andI_eReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x23 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  andI_mem_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x21 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  andI_mem_eReg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x21 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  andI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x4 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  orI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  orI_eRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  orI_eReg_castP2XNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  orI_eReg_castP2XNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  orI_eReg_castP2X_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2), opnd_array(1)->reg(ra_,this,idx1));
  }
}
uint  orI_eReg_castP2X_0Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  orI_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  orI_eReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  orI_eReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  orI_mem_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x09 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  orI_mem_eReg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x09 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  orI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x1 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  rolI_eReg_imm1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD1 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  rolI_eReg_imm8Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
void  rolI_eReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD3 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  rorI_eReg_imm1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD1 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  rorI_eReg_imm8Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
    emit_d8(cbuf, opnd_array(2)->constant());
  }
}
void  rorI_eReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD3 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  xorI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x33 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  xorI_eRegNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  xorI_eReg_im1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

     __ notl(opnd_array(1)->as_Register(ra_,this,idx1));
  }
}
uint  xorI_eReg_im1Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  xorI_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  xorI_eReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x33 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  xorI_eReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x33 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  xorI_mem_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x31 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  xorI_mem_eReg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x31 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  xorI_mem_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  }
  {
    int rm_byte_opcode = (0x6 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  movI_nocopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  ci2bNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // NEG $dst
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0x03, opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_opcode(cbuf,(0x13));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
uint  ci2bNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  movP_nocopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  cp2bNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // NEG $dst
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0x03, opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_opcode(cbuf,(0x13));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  cmpLTMaskNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x33));
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(0)->reg(ra_,this));
  }
  {
    emit_opcode(cbuf,(0x3B));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    // SETLT $dst
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0x9C);
    emit_rm( cbuf, 0x3, 0x4, opnd_array(0)->reg(ra_,this) );
  }
  {
    // NEG $dst
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0x03, opnd_array(0)->reg(ra_,this) );
  }
}
void  cmpLTMask0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
    emit_d8(cbuf, (0x1F));
  }
}
void  cadd_cmpLTMaskNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    int tmpReg = ECX_enc;

    // SUB $p,$q
    emit_opcode(cbuf,0x2B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
    // SBB $tmp,$tmp
    emit_opcode(cbuf,0x1B);
    emit_rm(cbuf, 0x3, tmpReg, tmpReg);
    // AND $tmp,$y
    emit_opcode(cbuf,0x23);
    emit_rm(cbuf, 0x3, tmpReg, opnd_array(3)->reg(ra_,this,idx3));
    // ADD $p,$tmp
    emit_opcode(cbuf,0x03);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), tmpReg);
  }
}
void  cadd_cmpLTMask_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    int tmpReg = ECX_enc;

    // SUB $p,$q
    emit_opcode(cbuf,0x2B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
    // SBB $tmp,$tmp
    emit_opcode(cbuf,0x1B);
    emit_rm(cbuf, 0x3, tmpReg, tmpReg);
    // AND $tmp,$y
    emit_opcode(cbuf,0x23);
    emit_rm(cbuf, 0x3, tmpReg, opnd_array(3)->reg(ra_,this,idx3));
    // ADD $p,$tmp
    emit_opcode(cbuf,0x03);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), tmpReg);
  }
}
void  cadd_cmpLTMask_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    int tmpReg = ECX_enc;

    // SUB $p,$q
    emit_opcode(cbuf,0x2B);
    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2), opnd_array(3)->reg(ra_,this,idx3));
    // SBB $tmp,$tmp
    emit_opcode(cbuf,0x1B);
    emit_rm(cbuf, 0x3, tmpReg, tmpReg);
    // AND $tmp,$y
    emit_opcode(cbuf,0x23);
    emit_rm(cbuf, 0x3, tmpReg, opnd_array(1)->reg(ra_,this,idx1));
    // ADD $p,$tmp
    emit_opcode(cbuf,0x03);
    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2), tmpReg);
  }
}
void  cadd_cmpLTMask_2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    int tmpReg = ECX_enc;

    // SUB $p,$q
    emit_opcode(cbuf,0x2B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
    // SBB $tmp,$tmp
    emit_opcode(cbuf,0x1B);
    emit_rm(cbuf, 0x3, tmpReg, tmpReg);
    // AND $tmp,$y
    emit_opcode(cbuf,0x23);
    emit_rm(cbuf, 0x3, tmpReg, opnd_array(3)->reg(ra_,this,idx3));
    // ADD $p,$tmp
    emit_opcode(cbuf,0x03);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), tmpReg);
  }
}
void  addL_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_d8(cbuf, (0x03 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    emit_d8(cbuf, (0x13 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)));
  }
}
void  addL_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x02 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  }
}
void  addL_eReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x03 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x13 /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1));  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(2)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  addL_eReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x03 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x13 /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2));  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(1)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  subL_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_d8(cbuf, (0x2B /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    emit_d8(cbuf, (0x1B /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)));
  }
}
void  subL_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x03 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  }
}
void  subL_eReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x2B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x1B /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1));  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(2)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  negL_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,0xF7);    // NEG hi
    emit_rm    (cbuf,0x3, 0x3, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)));
    emit_opcode(cbuf,0xF7);    // NEG lo
    emit_rm    (cbuf,0x3, 0x3,               opnd_array(2)->reg(ra_,this,idx2) );
    emit_opcode(cbuf,0x83);    // SBB hi,0
    emit_rm    (cbuf,0x3, 0x3, HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)));
    emit_d8    (cbuf,0 );
  }
}
void  andL_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_d8(cbuf, (0x23 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    emit_d8(cbuf, (0x23 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)));
  }
}
void  andL_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  }
}
void  andL_eReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x23 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x23 /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1));  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(2)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  andL_eReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x23 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x23 /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2));  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(1)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  orl_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_d8(cbuf, (0x0B /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    emit_d8(cbuf, (0x0B /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)));
  }
}
void  orl_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  }
}
void  orl_eReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x0B /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1));  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(2)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  orl_eReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x0B /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2));  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(1)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  xorl_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_d8(cbuf, (0x33 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    emit_d8(cbuf, (0x33 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)));
  }
}
void  xorl_eReg_im1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

     __ notl(opnd_array(1)->as_Register(ra_,this,idx1));
     __ notl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)));
  }
}
void  xorl_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  }
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*tertiary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  }
}
void  xorl_eReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x33 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x33 /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1));  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(2)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  xorl_eReg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x33 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x33 /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2));  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(1)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  shlL_eReg_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1),opnd_array(1)->as_Register(ra_,this,idx1));
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)));
  }
}
void  shlL_eReg_2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1),opnd_array(1)->as_Register(ra_,this,idx1));
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)));
    __ addl(opnd_array(1)->as_Register(ra_,this,idx1),opnd_array(1)->as_Register(ra_,this,idx1));
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)));
  }
}
void  shlL_eReg_3Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1),opnd_array(1)->as_Register(ra_,this,idx1));
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)));
    __ addl(opnd_array(1)->as_Register(ra_,this,idx1),opnd_array(1)->as_Register(ra_,this,idx1));
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)));
    __ addl(opnd_array(1)->as_Register(ra_,this,idx1),opnd_array(1)->as_Register(ra_,this,idx1));
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)));
  }
}
void  shlL_eReg_1_31Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    int r1, r2;
    if( (0xA4 /*tertiary()*/) == 0xA4 ) { r1 = opnd_array(1)->reg(ra_,this,idx1);  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1);  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)); }
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,(0xA4 /*tertiary()*/));
    emit_rm(cbuf, 0x3, r1, r2);
    emit_d8(cbuf,opnd_array(2)->constant());
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), r1);
    emit_d8(cbuf,opnd_array(2)->constant());
  }
}
void  shlL_eReg_32_63Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    int r1, r2;
    if( (0x4 /*secondary()*/) == 0x5 ) { r1 = opnd_array(1)->reg(ra_,this,idx1);  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1);  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)); }

    emit_opcode( cbuf, 0x8B ); // Move r1,r2
    emit_rm(cbuf, 0x3, r1, r2);
    if( opnd_array(2)->constant() > 32 ) { // Shift, if not by zero
      emit_opcode(cbuf,(0xC1 /*primary()*/));
      emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), r1);
      emit_d8(cbuf,opnd_array(2)->constant()-32);
    }
    emit_opcode(cbuf,0x33);  // XOR r2,r2
    emit_rm(cbuf, 0x3, r2, r2);
  }
}
void  salL_eReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // TEST shift,32
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0, ECX_enc);
    emit_d32(cbuf,0x20);
    // JEQ,s small
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x04);
    // MOV    $dst.hi,$dst.lo
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), opnd_array(1)->reg(ra_,this,idx1) );
    // CLR    $dst.lo
    emit_opcode(cbuf, 0x33);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(1)->reg(ra_,this,idx1));
// small:
    // SHLD   $dst.hi,$dst.lo,$shift
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xA5);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
    // SHL    $dst.lo,$shift"
    emit_opcode(cbuf,0xD3);
    emit_rm(cbuf, 0x3, 0x4, opnd_array(1)->reg(ra_,this,idx1) );
  }
}
uint  salL_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 17, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 17);
}
void  shrL_eReg_1_31Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    int r1, r2;
    if( (0xAC /*tertiary()*/) == 0xA4 ) { r1 = opnd_array(1)->reg(ra_,this,idx1);  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1);  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)); }
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,(0xAC /*tertiary()*/));
    emit_rm(cbuf, 0x3, r1, r2);
    emit_d8(cbuf,opnd_array(2)->constant());
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), r1);
    emit_d8(cbuf,opnd_array(2)->constant());
  }
}
void  shrL_eReg_32_63Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    int r1, r2;
    if( (0x5 /*secondary()*/) == 0x5 ) { r1 = opnd_array(1)->reg(ra_,this,idx1);  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1);  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)); }

    emit_opcode( cbuf, 0x8B ); // Move r1,r2
    emit_rm(cbuf, 0x3, r1, r2);
    if( opnd_array(2)->constant() > 32 ) { // Shift, if not by zero
      emit_opcode(cbuf,(0xC1 /*primary()*/));
      emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), r1);
      emit_d8(cbuf,opnd_array(2)->constant()-32);
    }
    emit_opcode(cbuf,0x33);  // XOR r2,r2
    emit_rm(cbuf, 0x3, r2, r2);
  }
}
void  shrL_eReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // TEST shift,32
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0, ECX_enc);
    emit_d32(cbuf,0x20);
    // JEQ,s small
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x04);
    // MOV    $dst.lo,$dst.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)) );
    // CLR    $dst.hi
    emit_opcode(cbuf, 0x33);
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
// small:
    // SHRD   $dst.lo,$dst.hi,$shift
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xAD);
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), opnd_array(1)->reg(ra_,this,idx1));
    // SHR    $dst.hi,$shift"
    emit_opcode(cbuf,0xD3);
    emit_rm(cbuf, 0x3, 0x5, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)) );
  }
}
uint  shrL_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 17, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 17);
}
void  sarL_eReg_1_31Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    int r1, r2;
    if( (0xAC /*tertiary()*/) == 0xA4 ) { r1 = opnd_array(1)->reg(ra_,this,idx1);  r2 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1);  r1 = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)); }
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,(0xAC /*tertiary()*/));
    emit_rm(cbuf, 0x3, r1, r2);
    emit_d8(cbuf,opnd_array(2)->constant());
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), r1);
    emit_d8(cbuf,opnd_array(2)->constant());
  }
}
void  sarL_eReg_32_63Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0x8B ); // Move
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
    emit_d8(cbuf,opnd_array(2)->constant()-32);
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
    emit_d8(cbuf,31);
  }
}
void  sarL_eReg_CLNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // TEST shift,32
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0, ECX_enc);
    emit_d32(cbuf,0x20);
    // JEQ,s small
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x05);
    // MOV    $dst.lo,$dst.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)) );
    // SAR    $dst.hi,31
    emit_opcode(cbuf, 0xC1);
    emit_rm(cbuf, 0x3, 7, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)) );
    emit_d8(cbuf, 0x1F );
// small:
    // SHRD   $dst.lo,$dst.hi,$shift
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xAD);
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), opnd_array(1)->reg(ra_,this,idx1));
    // SAR    $dst.hi,$shift"
    emit_opcode(cbuf,0xD3);
    emit_rm(cbuf, 0x3, 0x7, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)) );
  }
}
uint  sarL_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 18, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 18);
}
void  cmpD_cc_P6Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xDF /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    // Fixup the integer flags in case comparison involved a NaN
    //
    // JNP exit (no unordered comparison, P-flag is set by NaN)
    emit_opcode( cbuf, 0x7B );
    emit_d8    ( cbuf, 0x03 );
    // MOV AH,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // SAHF
    emit_opcode( cbuf, 0x9E);
    // NOP     // target for branch to avoid branch to branch
    emit_opcode( cbuf, 0x90);
  }
}
void  cmpD_cc_P6CFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xDF /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
void  cmpD_ccNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xD8 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    // fnstsw_ax
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // test ax,0x0400
    emit_opcode( cbuf, 0x66 );   // operand-size prefix for 16-bit immediate
    emit_opcode( cbuf, 0xA9 );
    emit_d16   ( cbuf, 0x0400 );
    // // // This sequence works, but stalls for 12-16 cycles on PPro
    // // test rax,0x0400
    // emit_opcode( cbuf, 0xA9 );
    // emit_d32   ( cbuf, 0x00000400 );
    //
    // jz exit (no unordered comparison)
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x02 );
    // mov ah,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // sahf
    emit_opcode( cbuf, 0x9E);
  }
}
void  cmpD_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xD9 /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xE4 /*primary()*/));
  }
  {
    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  }
  {
    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, 1 );
  }
}
void  cmpD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xD8 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, 1 );
  }
}
void  cmpXD_ccNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x66 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0x2F /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    // Fixup the integer flags in case comparison involved a NaN
    //
    // JNP exit (no unordered comparison, P-flag is set by NaN)
    emit_opcode( cbuf, 0x7B );
    emit_d8    ( cbuf, 0x03 );
    // MOV AH,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // SAHF
    emit_opcode( cbuf, 0x9E);
    // NOP     // target for branch to avoid branch to branch
    emit_opcode( cbuf, 0x90);
  }
}
void  cmpXD_ccCFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x66 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0x2F /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  cmpXD_ccmemNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x66 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0x2F /*tertiary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    // Fixup the integer flags in case comparison involved a NaN
    //
    // JNP exit (no unordered comparison, P-flag is set by NaN)
    emit_opcode( cbuf, 0x7B );
    emit_d8    ( cbuf, 0x03 );
    // MOV AH,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // SAHF
    emit_opcode( cbuf, 0x9E);
    // NOP     // target for branch to avoid branch to branch
    emit_opcode( cbuf, 0x90);
  }
}
void  cmpXD_ccmemCFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x66 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0x2F /*tertiary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  cmpXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, 0x33);
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(0)->reg(ra_,this));
  }
  {
    emit_opcode(cbuf,(0x66 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0x2F /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    MacroAssembler _masm(&cbuf);
    Label nan, inc, done;

    __ jccb(Assembler::parity, nan);
    __ jccb(Assembler::equal,  done);
    __ jccb(Assembler::above,  inc);
    __ bind(nan);
    __ decrement(as_Register(opnd_array(0)->reg(ra_,this))); // NO L qqq
    __ jmpb(done);
    __ bind(inc);
    __ increment(as_Register(opnd_array(0)->reg(ra_,this))); // NO L qqq
    __ bind(done);
  }
}
void  cmpXD_regmemNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x66 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0x2F /*tertiary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    emit_opcode(cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32(cbuf, (0x0));
  }
  {
    MacroAssembler _masm(&cbuf);
    Label nan, inc, done;

    __ jccb(Assembler::parity, nan);
    __ jccb(Assembler::equal,  done);
    __ jccb(Assembler::above,  inc);
    __ bind(nan);
    __ decrement(as_Register(opnd_array(0)->reg(ra_,this))); // NO L qqq
    __ jmpb(done);
    __ bind(inc);
    __ increment(as_Register(opnd_array(0)->reg(ra_,this))); // NO L qqq
    __ bind(done);
  }
}
void  subD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  subD_reg_roundNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xD8 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  subD_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xDD /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  absD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xD9 /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xE1 /*primary()*/));
  }
}
void  absXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    address signmask_address=(address)double_signmask_pool;
    // andpd:\tANDPD  $dst,[signconst]
    emit_opcode(cbuf, 0x66);
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x54);
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1), 0x5);
    emit_d32(cbuf, (int)signmask_address);
  }
}
void  negD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xD9 /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xE0 /*primary()*/));
  }
}
void  negXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

     __ xorpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1),
              ExternalAddress((address)double_signflip_pool));
  }
}
void  addD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
uint  addD_regNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  addD_reg_roundNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xD8 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  addD_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xDD /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  addD_reg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xDD /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
void  addD_mem_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xDD));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0xD8));
  }
  {
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    cbuf.set_inst_mark();            // Mark start of opcode for reloc info in mem operand
  }
  {
    emit_opcode(cbuf,(0xDD));
  }
  {
    int rm_byte_opcode = (0x03);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  addD_mem_reg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xDD));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0xD8));
  }
  {
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    cbuf.set_inst_mark();            // Mark start of opcode for reloc info in mem operand
  }
  {
    emit_opcode(cbuf,(0xDD));
  }
  {
    int rm_byte_opcode = (0x03);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
}
void  addD_reg_imm1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if( is_positive_zero_double(opnd_array(2)->constantD())) {
      // FLDZ
      emit_opcode(cbuf,0xD9);
      emit_opcode(cbuf,0xEE);
    } else if( is_positive_one_double(opnd_array(2)->constantD())) {
      // FLD1
      emit_opcode(cbuf,0xD9);
      emit_opcode(cbuf,0xE8);
    } else {
      emit_opcode(cbuf,0xDD);
      emit_rm(cbuf, 0x0, 0x0, 0x5);
      emit_double_constant(cbuf, opnd_array(2)->constantD());
    }
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  addD_reg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if( is_positive_zero_double(opnd_array(2)->constantD())) {
      // FLDZ
      emit_opcode(cbuf,0xD9);
      emit_opcode(cbuf,0xEE);
    } else if( is_positive_one_double(opnd_array(2)->constantD())) {
      // FLD1
      emit_opcode(cbuf,0xD9);
      emit_opcode(cbuf,0xE8);
    } else {
      emit_opcode(cbuf,0xDD);
      emit_rm(cbuf, 0x0, 0x0, 0x5);
      emit_double_constant(cbuf, opnd_array(2)->constantD());
    }
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  addD_reg_imm_roundNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if( is_positive_zero_double(opnd_array(2)->constantD())) {
      // FLDZ
      emit_opcode(cbuf,0xD9);
      emit_opcode(cbuf,0xEE);
    } else if( is_positive_one_double(opnd_array(2)->constantD())) {
      // FLD1
      emit_opcode(cbuf,0xD9);
      emit_opcode(cbuf,0xE8);
    } else {
      emit_opcode(cbuf,0xDD);
      emit_rm(cbuf, 0x0, 0x0, 0x5);
      emit_double_constant(cbuf, opnd_array(2)->constantD());
    }
  }
  {
    emit_opcode(cbuf,(0xD8 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  addXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x58));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  addXD_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x58));
  }
  {
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1), 0x5);
    emit_double_constant(cbuf, opnd_array(2)->constantD());
  }
}
void  addXD_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x58));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  addXD_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x58));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  subXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x5C));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  subXD_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x5C));
  }
  {
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1), 0x5);
    emit_double_constant(cbuf, opnd_array(2)->constantD());
  }
}
void  subXD_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x5C));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  mulXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x59));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  mulXD_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x59));
  }
  {
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1), 0x5);
    emit_double_constant(cbuf, opnd_array(2)->constantD());
  }
}
void  mulXD_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x59));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  mulXD_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x59));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  divXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x5E));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  divXD_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x5E));
  }
  {
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1), 0x5);
    emit_double_constant(cbuf, opnd_array(2)->constantD());
  }
}
void  divXD_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x5E));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  mulD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  strictfp_mulD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::addr_fpu_subnormal_bias1() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::addr_fpu_subnormal_bias2() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  mulD_reg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    if( is_positive_zero_double(opnd_array(2)->constantD())) {
      // FLDZ
      emit_opcode(cbuf,0xD9);
      emit_opcode(cbuf,0xEE);
    } else if( is_positive_one_double(opnd_array(2)->constantD())) {
      // FLD1
      emit_opcode(cbuf,0xD9);
      emit_opcode(cbuf,0xE8);
    } else {
      emit_opcode(cbuf,0xDD);
      emit_rm(cbuf, 0x0, 0x0, 0x5);
      emit_double_constant(cbuf, opnd_array(2)->constantD());
    }
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  mulD_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xDD /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  mulD_reg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xDD /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
void  mulD_reg_mem_ciscNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  mulD_reg_mem_cisc_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  addD_mulD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3));
  }
}
void  addD_mulD_reg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(3)->reg(ra_,this,idx3));
  }
  {
    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  subD_mulD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    emit_opcode(cbuf,(0xDE));
  }
  {
    emit_opcode(cbuf, (0xE0) + opnd_array(3)->reg(ra_,this,idx3));
  }
}
void  divD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  strictfp_divD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::addr_fpu_subnormal_bias1() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::addr_fpu_subnormal_bias2() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  divD_reg_roundNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xD8 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x6 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  modD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // load dst in FPR0
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
    if (opnd_array(2)->reg(ra_,this,idx2) != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // swap src with FPR1:
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2) );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
  }
  {
    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2) );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
    // // following asm replaced with Pop_Reg_F or Pop_Mem_F
    // // FSTP   FPR$dst$$reg
    // emit_opcode( cbuf, 0xDD );
    // emit_d8( cbuf, 0xD8+$dst$$reg );
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  modXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);

    emit_opcode  (cbuf, 0xF2 );     // MOVSD [ESP], src1
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(2)->reg(ra_,this,idx2), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xDD );      // FLD_D [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);

    emit_opcode  (cbuf, 0xF2 );     // MOVSD [ESP], src0
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xDD );      // FLD_D [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);

  }
  {
    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [ESP]

    // UseXmmLoadAndClearUpper ? movsd dst,[esp] : movlpd dst,[esp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
    encode_RegMem(cbuf, opnd_array(0)->reg(ra_,this), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,8
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x08);
  }
  {
    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  }
}
void  sinD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xD9 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0xFE /*secondary()*/));
  }
}
void  sinXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);

    emit_opcode  (cbuf, 0xF2 );     // MOVSD [ESP], src
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xDD );      // FLD_D [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf,(0xD9 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0xFE /*secondary()*/));
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [ESP]

    // UseXmmLoadAndClearUpper ? movsd dst,[esp] : movlpd dst,[esp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,8
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x08);
  }
}
void  cosD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xD9 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0xFF /*secondary()*/));
  }
}
void  cosXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);

    emit_opcode  (cbuf, 0xF2 );     // MOVSD [ESP], src
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xDD );      // FLD_D [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf,(0xD9 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0xFF /*secondary()*/));
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [ESP]

    // UseXmmLoadAndClearUpper ? movsd dst,[esp] : movlpd dst,[esp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,8
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x08);
  }
}
void  tanD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0xDD));
  }
  {
    emit_opcode(cbuf,(0xD8));
  }
}
void  tanXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);

    emit_opcode  (cbuf, 0xF2 );     // MOVSD [ESP], src
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xDD );      // FLD_D [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0xDD));
  }
  {
    emit_opcode(cbuf,(0xD8));
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [ESP]

    // UseXmmLoadAndClearUpper ? movsd dst,[esp] : movlpd dst,[esp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,8
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x08);
  }
}
void  atanD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xD9 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0xF3 /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0xF3 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  atanXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);

    emit_opcode  (cbuf, 0xF2 );     // MOVSD [ESP], src
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(2)->reg(ra_,this,idx2), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xDD );      // FLD_D [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf,(0xD9 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0xF3 /*secondary()*/));
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [ESP]

    // UseXmmLoadAndClearUpper ? movsd dst,[esp] : movlpd dst,[esp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,8
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x08);
  }
}
void  sqrtD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xD9 /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xFA /*primary()*/));
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  powD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,0x83);     // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8    (cbuf,0x08);
  }
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xF1));
  }
  {
    // FPR1 holds Y*ln2(X).  Compute FPR1 = 2^(Y*ln2(X))
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xC0);  // fdup = fld st(0)          Q       Q
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xFC);  // frndint               int(Q)      Q
    emit_opcode(cbuf,0xDC); emit_opcode(cbuf,0xE9);  // fsub st(1) -= st(0);  int(Q) frac(Q)
    emit_opcode(cbuf,0xDB);                          // FISTP [ESP]           frac(Q)
    emit_opcode(cbuf,0x1C);
    emit_d8(cbuf,0x24);
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xF0);  // f2xm1                 2^frac(Q)-1
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xE8);  // fld1                  1 2^frac(Q)-1
    emit_opcode(cbuf,0xDE); emit_opcode(cbuf,0xC1);  // faddp                 2^frac(Q)
    emit_opcode(cbuf,0x8B);                          // mov rax,[esp+0]=int(Q)
    encode_RegMem(cbuf, EAX_enc, ESP_enc, 0x4, 0, 0, false);
    emit_opcode(cbuf,0xC7);                          // mov rcx,0xFFFFF800 - overflow mask
    emit_rm(cbuf, 0x3, 0x0, ECX_enc);
    emit_d32(cbuf,0xFFFFF800);
    emit_opcode(cbuf,0x81);                          // add rax,1023 - the double exponent bias
    emit_rm(cbuf, 0x3, 0x0, EAX_enc);
    emit_d32(cbuf,1023);
    emit_opcode(cbuf,0x8B);                          // mov rbx,eax
    emit_rm(cbuf, 0x3, EBX_enc, EAX_enc);
    emit_opcode(cbuf,0xC1);                          // shl rax,20 - Slide to exponent position
    emit_rm(cbuf,0x3,0x4,EAX_enc);
    emit_d8(cbuf,20);
    emit_opcode(cbuf,0x85);                          // test rbx,ecx - check for overflow
    emit_rm(cbuf, 0x3, EBX_enc, ECX_enc);
    emit_opcode(cbuf,0x0F); emit_opcode(cbuf,0x45);  // CMOVne rax,ecx - overflow; stuff NAN into EAX
    emit_rm(cbuf, 0x3, EAX_enc, ECX_enc);
    emit_opcode(cbuf,0x89);                          // mov [esp+4],eax - Store as part of double word
    encode_RegMem(cbuf, EAX_enc, ESP_enc, 0x4, 0, 4, false);
    emit_opcode(cbuf,0xC7);                          // mov [esp+0],0   - [ESP] = (double)(1<<int(Q)) = 2^int(Q)
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);
    emit_d32(cbuf,0);
    emit_opcode(cbuf,0xDC);                          // fmul dword st(0),[esp+0]; FPR1 = 2^int(Q)*2^frac(Q) = 2^Q
    encode_RegMem(cbuf, 0x1, ESP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf,0x83);     // ADD ESP,8
    emit_opcode(cbuf,0xC4);
    emit_d8    (cbuf,0x08);
  }
}
void  powXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,0x83);     // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8    (cbuf,0x08);
  }
  {
    emit_opcode  (cbuf, 0xF2 );     // MOVSD [ESP], xmm_src
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(2)->reg(ra_,this,idx2), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xDD );      // FLD_D [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode  (cbuf, 0xF2 );     // MOVSD [ESP], xmm_src
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xDD );      // FLD_D [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xF1));
  }
  {
    // FPR1 holds Y*ln2(X).  Compute FPR1 = 2^(Y*ln2(X))
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xC0);  // fdup = fld st(0)          Q       Q
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xFC);  // frndint               int(Q)      Q
    emit_opcode(cbuf,0xDC); emit_opcode(cbuf,0xE9);  // fsub st(1) -= st(0);  int(Q) frac(Q)
    emit_opcode(cbuf,0xDB);                          // FISTP [ESP]           frac(Q)
    emit_opcode(cbuf,0x1C);
    emit_d8(cbuf,0x24);
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xF0);  // f2xm1                 2^frac(Q)-1
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xE8);  // fld1                  1 2^frac(Q)-1
    emit_opcode(cbuf,0xDE); emit_opcode(cbuf,0xC1);  // faddp                 2^frac(Q)
    emit_opcode(cbuf,0x8B);                          // mov rax,[esp+0]=int(Q)
    encode_RegMem(cbuf, EAX_enc, ESP_enc, 0x4, 0, 0, false);
    emit_opcode(cbuf,0xC7);                          // mov rcx,0xFFFFF800 - overflow mask
    emit_rm(cbuf, 0x3, 0x0, ECX_enc);
    emit_d32(cbuf,0xFFFFF800);
    emit_opcode(cbuf,0x81);                          // add rax,1023 - the double exponent bias
    emit_rm(cbuf, 0x3, 0x0, EAX_enc);
    emit_d32(cbuf,1023);
    emit_opcode(cbuf,0x8B);                          // mov rbx,eax
    emit_rm(cbuf, 0x3, EBX_enc, EAX_enc);
    emit_opcode(cbuf,0xC1);                          // shl rax,20 - Slide to exponent position
    emit_rm(cbuf,0x3,0x4,EAX_enc);
    emit_d8(cbuf,20);
    emit_opcode(cbuf,0x85);                          // test rbx,ecx - check for overflow
    emit_rm(cbuf, 0x3, EBX_enc, ECX_enc);
    emit_opcode(cbuf,0x0F); emit_opcode(cbuf,0x45);  // CMOVne rax,ecx - overflow; stuff NAN into EAX
    emit_rm(cbuf, 0x3, EAX_enc, ECX_enc);
    emit_opcode(cbuf,0x89);                          // mov [esp+4],eax - Store as part of double word
    encode_RegMem(cbuf, EAX_enc, ESP_enc, 0x4, 0, 4, false);
    emit_opcode(cbuf,0xC7);                          // mov [esp+0],0   - [ESP] = (double)(1<<int(Q)) = 2^int(Q)
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);
    emit_d32(cbuf,0);
    emit_opcode(cbuf,0xDC);                          // fmul dword st(0),[esp+0]; FPR1 = 2^int(Q)*2^frac(Q) = 2^Q
    encode_RegMem(cbuf, 0x1, ESP_enc, 0x4, 0, 0, false);
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [ESP]

    // UseXmmLoadAndClearUpper ? movsd dst,[esp] : movlpd dst,[esp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
    encode_RegMem(cbuf, opnd_array(0)->reg(ra_,this), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,8
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x08);
  }
}
void  expD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,0x83);     // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8    (cbuf,0x08);
  }
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xEA));
  }
  {
    emit_opcode(cbuf,(0xDE));
  }
  {
    emit_opcode(cbuf,(0xC9));
  }
  {
    // FPR1 holds Y*ln2(X).  Compute FPR1 = 2^(Y*ln2(X))
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xC0);  // fdup = fld st(0)          Q       Q
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xFC);  // frndint               int(Q)      Q
    emit_opcode(cbuf,0xDC); emit_opcode(cbuf,0xE9);  // fsub st(1) -= st(0);  int(Q) frac(Q)
    emit_opcode(cbuf,0xDB);                          // FISTP [ESP]           frac(Q)
    emit_opcode(cbuf,0x1C);
    emit_d8(cbuf,0x24);
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xF0);  // f2xm1                 2^frac(Q)-1
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xE8);  // fld1                  1 2^frac(Q)-1
    emit_opcode(cbuf,0xDE); emit_opcode(cbuf,0xC1);  // faddp                 2^frac(Q)
    emit_opcode(cbuf,0x8B);                          // mov rax,[esp+0]=int(Q)
    encode_RegMem(cbuf, EAX_enc, ESP_enc, 0x4, 0, 0, false);
    emit_opcode(cbuf,0xC7);                          // mov rcx,0xFFFFF800 - overflow mask
    emit_rm(cbuf, 0x3, 0x0, ECX_enc);
    emit_d32(cbuf,0xFFFFF800);
    emit_opcode(cbuf,0x81);                          // add rax,1023 - the double exponent bias
    emit_rm(cbuf, 0x3, 0x0, EAX_enc);
    emit_d32(cbuf,1023);
    emit_opcode(cbuf,0x8B);                          // mov rbx,eax
    emit_rm(cbuf, 0x3, EBX_enc, EAX_enc);
    emit_opcode(cbuf,0xC1);                          // shl rax,20 - Slide to exponent position
    emit_rm(cbuf,0x3,0x4,EAX_enc);
    emit_d8(cbuf,20);
    emit_opcode(cbuf,0x85);                          // test rbx,ecx - check for overflow
    emit_rm(cbuf, 0x3, EBX_enc, ECX_enc);
    emit_opcode(cbuf,0x0F); emit_opcode(cbuf,0x45);  // CMOVne rax,ecx - overflow; stuff NAN into EAX
    emit_rm(cbuf, 0x3, EAX_enc, ECX_enc);
    emit_opcode(cbuf,0x89);                          // mov [esp+4],eax - Store as part of double word
    encode_RegMem(cbuf, EAX_enc, ESP_enc, 0x4, 0, 4, false);
    emit_opcode(cbuf,0xC7);                          // mov [esp+0],0   - [ESP] = (double)(1<<int(Q)) = 2^int(Q)
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);
    emit_d32(cbuf,0);
    emit_opcode(cbuf,0xDC);                          // fmul dword st(0),[esp+0]; FPR1 = 2^int(Q)*2^frac(Q) = 2^Q
    encode_RegMem(cbuf, 0x1, ESP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf,0x83);     // ADD ESP,8
    emit_opcode(cbuf,0xC4);
    emit_d8    (cbuf,0x08);
  }
}
void  expXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);

    emit_opcode  (cbuf, 0xF2 );     // MOVSD [ESP], src
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xDD );      // FLD_D [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xEA));
  }
  {
    emit_opcode(cbuf,(0xDE));
  }
  {
    emit_opcode(cbuf,(0xC9));
  }
  {
    // FPR1 holds Y*ln2(X).  Compute FPR1 = 2^(Y*ln2(X))
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xC0);  // fdup = fld st(0)          Q       Q
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xFC);  // frndint               int(Q)      Q
    emit_opcode(cbuf,0xDC); emit_opcode(cbuf,0xE9);  // fsub st(1) -= st(0);  int(Q) frac(Q)
    emit_opcode(cbuf,0xDB);                          // FISTP [ESP]           frac(Q)
    emit_opcode(cbuf,0x1C);
    emit_d8(cbuf,0x24);
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xF0);  // f2xm1                 2^frac(Q)-1
    emit_opcode(cbuf,0xD9); emit_opcode(cbuf,0xE8);  // fld1                  1 2^frac(Q)-1
    emit_opcode(cbuf,0xDE); emit_opcode(cbuf,0xC1);  // faddp                 2^frac(Q)
    emit_opcode(cbuf,0x8B);                          // mov rax,[esp+0]=int(Q)
    encode_RegMem(cbuf, EAX_enc, ESP_enc, 0x4, 0, 0, false);
    emit_opcode(cbuf,0xC7);                          // mov rcx,0xFFFFF800 - overflow mask
    emit_rm(cbuf, 0x3, 0x0, ECX_enc);
    emit_d32(cbuf,0xFFFFF800);
    emit_opcode(cbuf,0x81);                          // add rax,1023 - the double exponent bias
    emit_rm(cbuf, 0x3, 0x0, EAX_enc);
    emit_d32(cbuf,1023);
    emit_opcode(cbuf,0x8B);                          // mov rbx,eax
    emit_rm(cbuf, 0x3, EBX_enc, EAX_enc);
    emit_opcode(cbuf,0xC1);                          // shl rax,20 - Slide to exponent position
    emit_rm(cbuf,0x3,0x4,EAX_enc);
    emit_d8(cbuf,20);
    emit_opcode(cbuf,0x85);                          // test rbx,ecx - check for overflow
    emit_rm(cbuf, 0x3, EBX_enc, ECX_enc);
    emit_opcode(cbuf,0x0F); emit_opcode(cbuf,0x45);  // CMOVne rax,ecx - overflow; stuff NAN into EAX
    emit_rm(cbuf, 0x3, EAX_enc, ECX_enc);
    emit_opcode(cbuf,0x89);                          // mov [esp+4],eax - Store as part of double word
    encode_RegMem(cbuf, EAX_enc, ESP_enc, 0x4, 0, 4, false);
    emit_opcode(cbuf,0xC7);                          // mov [esp+0],0   - [ESP] = (double)(1<<int(Q)) = 2^int(Q)
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);
    emit_d32(cbuf,0);
    emit_opcode(cbuf,0xDC);                          // fmul dword st(0),[esp+0]; FPR1 = 2^int(Q)*2^frac(Q) = 2^Q
    encode_RegMem(cbuf, 0x1, ESP_enc, 0x4, 0, 0, false);
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [ESP]

    // UseXmmLoadAndClearUpper ? movsd dst,[esp] : movlpd dst,[esp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
    encode_RegMem(cbuf, opnd_array(0)->reg(ra_,this), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,8
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x08);
  }
}
void  log10D_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xEC));
  }
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xC9));
  }
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xF1));
  }
}
void  log10XD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xEC));
  }
  {
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);

    emit_opcode  (cbuf, 0xF2 );     // MOVSD [ESP], src
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xDD );      // FLD_D [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xF1));
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [ESP]

    // UseXmmLoadAndClearUpper ? movsd dst,[esp] : movlpd dst,[esp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
    encode_RegMem(cbuf, opnd_array(0)->reg(ra_,this), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,8
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x08);
  }
}
void  logD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xED));
  }
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xC9));
  }
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xF1));
  }
}
void  logXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xED));
  }
  {
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);

    emit_opcode  (cbuf, 0xF2 );     // MOVSD [ESP], src
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xDD );      // FLD_D [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);
  }
  {
    emit_opcode(cbuf,(0xD9));
  }
  {
    emit_opcode(cbuf,(0xF1));
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [ESP]

    // UseXmmLoadAndClearUpper ? movsd dst,[esp] : movlpd dst,[esp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
    encode_RegMem(cbuf, opnd_array(0)->reg(ra_,this), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,8
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x08);
  }
}
void  cmpF_cc_P6Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xDF /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    // Fixup the integer flags in case comparison involved a NaN
    //
    // JNP exit (no unordered comparison, P-flag is set by NaN)
    emit_opcode( cbuf, 0x7B );
    emit_d8    ( cbuf, 0x03 );
    // MOV AH,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // SAHF
    emit_opcode( cbuf, 0x9E);
    // NOP     // target for branch to avoid branch to branch
    emit_opcode( cbuf, 0x90);
  }
}
void  cmpF_cc_P6CFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xDF /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
void  cmpF_ccNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xD8 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    // fnstsw_ax
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // test ax,0x0400
    emit_opcode( cbuf, 0x66 );   // operand-size prefix for 16-bit immediate
    emit_opcode( cbuf, 0xA9 );
    emit_d16   ( cbuf, 0x0400 );
    // // // This sequence works, but stalls for 12-16 cycles on PPro
    // // test rax,0x0400
    // emit_opcode( cbuf, 0xA9 );
    // emit_d32   ( cbuf, 0x00000400 );
    //
    // jz exit (no unordered comparison)
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x02 );
    // mov ah,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // sahf
    emit_opcode( cbuf, 0x9E);
  }
}
void  cmpF_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xD9 /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xE4 /*primary()*/));
  }
  {
    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  }
  {
    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, 1 );
  }
}
void  cmpF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,(0xD8 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32( cbuf, 1 );
  }
}
void  cmpX_ccNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x2F /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    // Fixup the integer flags in case comparison involved a NaN
    //
    // JNP exit (no unordered comparison, P-flag is set by NaN)
    emit_opcode( cbuf, 0x7B );
    emit_d8    ( cbuf, 0x03 );
    // MOV AH,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // SAHF
    emit_opcode( cbuf, 0x9E);
    // NOP     // target for branch to avoid branch to branch
    emit_opcode( cbuf, 0x90);
  }
}
void  cmpX_ccCFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x2F /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  cmpX_ccmemNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x2F /*secondary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    // Fixup the integer flags in case comparison involved a NaN
    //
    // JNP exit (no unordered comparison, P-flag is set by NaN)
    emit_opcode( cbuf, 0x7B );
    emit_d8    ( cbuf, 0x03 );
    // MOV AH,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // SAHF
    emit_opcode( cbuf, 0x9E);
    // NOP     // target for branch to avoid branch to branch
    emit_opcode( cbuf, 0x90);
  }
}
void  cmpX_ccmemCFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x2F /*secondary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  cmpX_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf, 0x33);
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(0)->reg(ra_,this));
  }
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x2F /*secondary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    MacroAssembler _masm(&cbuf);
    Label nan, inc, done;

    __ jccb(Assembler::parity, nan);
    __ jccb(Assembler::equal,  done);
    __ jccb(Assembler::above,  inc);
    __ bind(nan);
    __ decrement(as_Register(opnd_array(0)->reg(ra_,this))); // NO L qqq
    __ jmpb(done);
    __ bind(inc);
    __ increment(as_Register(opnd_array(0)->reg(ra_,this))); // NO L qqq
    __ bind(done);
  }
}
void  cmpX_regmemNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x0F /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x2F /*secondary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    emit_opcode(cbuf, 0xB8 + opnd_array(0)->reg(ra_,this));
    emit_d32(cbuf, (0x0));
  }
  {
    MacroAssembler _masm(&cbuf);
    Label nan, inc, done;

    __ jccb(Assembler::parity, nan);
    __ jccb(Assembler::equal,  done);
    __ jccb(Assembler::above,  inc);
    __ bind(nan);
    __ decrement(as_Register(opnd_array(0)->reg(ra_,this))); // NO L qqq
    __ jmpb(done);
    __ bind(inc);
    __ increment(as_Register(opnd_array(0)->reg(ra_,this))); // NO L qqq
    __ bind(done);
  }
}
void  subF24_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  subF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  addF24_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  addF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  addX_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x58));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  addX_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x58));
  }
  {
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1), 0x5);
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
}
void  addX_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x58));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  addX_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x58));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  subX_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x5C));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  subX_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x5C));
  }
  {
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1), 0x5);
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
}
void  subX_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x5C));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  mulX_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x59));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  mulX_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x59));
  }
  {
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1), 0x5);
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
}
void  mulX_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x59));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  mulX_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x59));
  }
  {
    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  divX_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x5E));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  divX_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x5E));
  }
  {
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1), 0x5);
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
}
void  divX_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x5E));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  sqrtX_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x51));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  sqrtX_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x51));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  sqrtXD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x51));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  sqrtXD_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x51));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  absF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xD9 /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xE1 /*primary()*/));
  }
}
void  absX_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    address signmask_address=(address)float_signmask_pool;
    // andpd:\tANDPS  $dst,[signconst]
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x54);
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1), 0x5);
    emit_d32(cbuf, (int)signmask_address);
  }
}
void  negF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xD9 /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0xE0 /*primary()*/));
  }
}
void  negX_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    address signmask_address=(address)float_signflip_pool;
    // andpd:\tXORPS  $dst,[signconst]
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x57);
    emit_rm(cbuf, 0x0, opnd_array(1)->reg(ra_,this,idx1), 0x5);
    emit_d32(cbuf, (int)signmask_address);
  }
}
void  addF24_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  addF24_reg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  addF_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  addF_reg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
}
void  addF24_mem_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  addF24_mem_ciscNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    cbuf.set_inst_mark();            // Mark start of opcode for reloc info in mem operand
  }
  {
    emit_opcode(cbuf,(0xD8 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x0 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  addF24_mem_cisc_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    cbuf.set_inst_mark();            // Mark start of opcode for reloc info in mem operand
  }
  {
    emit_opcode(cbuf,(0xD8 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x0 /*secondary()*/);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  addF24_mem_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    cbuf.set_inst_mark();            // Mark start of opcode for reloc info in mem operand
  }
  {
    emit_opcode(cbuf,(0xD8 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x0 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  addF24_reg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    cbuf.set_inst_mark();
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x0, (0x00 /*secondary()*/), 0x5);
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  addF_reg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    cbuf.set_inst_mark();
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x0, (0x00 /*secondary()*/), 0x5);
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  mulF24_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  mulF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  mulF24_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  mulF24_reg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  mulF_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  mulF_reg_mem_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  mulF24_mem_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    cbuf.set_inst_mark();            // Mark start of opcode for reloc info in mem operand
  }
  {
    emit_opcode(cbuf,(0xD8 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x1 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  mulF24_reg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    cbuf.set_inst_mark();
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x0, (0x1 /*secondary()*/), 0x5);
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  mulF_reg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    cbuf.set_inst_mark();
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x0, (0x1 /*secondary()*/), 0x5);
    emit_float_constant(cbuf, opnd_array(2)->constantF());
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  mulF_reg_load1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  mulF_reg_load1_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*tertiary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  addF_mulF_reg_load1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3));
    //could use FADDP  src2,fpST  /* DE C0+i */
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  addF_mulF_reg_load1_1Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(3)->reg(ra_,this,idx3));
  }
  {
    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1));
    //could use FADDP  src2,fpST  /* DE C0+i */
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  addF_mulF_reg_load1_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(1)->reg(ra_,this,idx1));
  }
  {
    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3));
    //could use FADDP  src2,fpST  /* DE C0+i */
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  addF_mulF_reg_load1_2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode(cbuf,(0xD9 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(3)->base(ra_,this,idx3);
    int index    = opnd_array(3)->index(ra_,this,idx3);
    int scale    = opnd_array(3)->scale();
    int displace = opnd_array(3)->disp(ra_,this,idx3);
    bool disp_is_oop = opnd_array(3)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1));
    //could use FADDP  src2,fpST  /* DE C0+i */
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  addF_mulF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2));
  }
  {
    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3));
  }
}
void  addF_mulF_reg_0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(3)->reg(ra_,this,idx3));
  }
  {
    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  subF_divF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    // Operand has been loaded into fp ST (stack top)
      // FSUB   ST,$src1
      emit_opcode(cbuf, 0xD8);
      emit_opcode(cbuf, 0xE0 + opnd_array(2)->reg(ra_,this,idx2));

      // FDIV
      emit_opcode(cbuf, 0xD8);
      emit_opcode(cbuf, 0xF0 + opnd_array(3)->reg(ra_,this,idx3));
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  divF24_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x6 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  divF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2) );
  }
  {
    emit_opcode(cbuf,(0xDE /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  modF24_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // load dst in FPR0
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
    if (opnd_array(2)->reg(ra_,this,idx2) != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // swap src with FPR1:
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2) );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
  }
  {
    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2) );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
    // // following asm replaced with Pop_Reg_F or Pop_Mem_F
    // // FSTP   FPR$dst$$reg
    // emit_opcode( cbuf, 0xDD );
    // emit_d8( cbuf, 0xD8+$dst$$reg );
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  modF_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // load dst in FPR0
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
    if (opnd_array(2)->reg(ra_,this,idx2) != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // swap src with FPR1:
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2) );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
  }
  {
    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  }
  {
    if (opnd_array(2)->reg(ra_,this,idx2) != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2) );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
    // // following asm replaced with Pop_Reg_F or Pop_Mem_F
    // // FSTP   FPR$dst$$reg
    // emit_opcode( cbuf, 0xDD );
    // emit_d8( cbuf, 0xD8+$dst$$reg );
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  modX_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x04);

    emit_opcode  (cbuf, 0xF3 );     // MOVSS [ESP], src1
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(2)->reg(ra_,this,idx2), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xD9 );      // FLD [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);

    emit_opcode  (cbuf, 0xF3 );     // MOVSS [ESP], src0
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xD9 );      // FLD [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);

  }
  {
    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, 0 ); //FSTP_S [ESP]

    emit_opcode  (cbuf, 0xF3 );     // MOVSS dst(xmm), [ESP]
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x10 );
    encode_RegMem(cbuf, opnd_array(0)->reg(ra_,this), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,d8 (4 or 8)
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,(0x4));
  }
  {
    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  }
}
void  roundFloat_mem_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1) != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xD9, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_S  [ESP+dst]
  }
}
void  roundDouble_mem_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1) != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xDD, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_D  [ESP+dst]
  }
}
void  convD2X_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x04);
    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1) != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );       // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xD9, pop, 0 ); // FST<P>_S  [ESP]

    emit_opcode  (cbuf, 0xF3 );        // MOVSS dst(xmm), [ESP]
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x10 );
    encode_RegMem(cbuf, opnd_array(0)->reg(ra_,this), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);            // ADD ESP,4
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x04);
    // Carry on here...
  }
}
void  convXD2X_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xF2 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0x5A /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  convF2D_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int pop = 0xD0 - 1; // -1 since we skip FLD
    if (opnd_array(1)->reg(ra_,this,idx1) != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(src-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
      pop = 0xD8;
    }
    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, pop+opnd_array(0)->reg(ra_,this) );      // FST<P> ST(i)
  }
}
void  convX2D_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Allocate a word
    emit_opcode(cbuf,0x83);     // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x04);

    emit_opcode  (cbuf, 0xF3 ); // MOVSS [ESP], xmm
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xD9 );    // FLD_S [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);     // ADD ESP,4
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x04);

    // Carry on here...
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  convX2XD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xF3 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0x5A /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  convD2I_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    // Flip to round-to-zero mode.  We attempted to allow invalid-op
    // exceptions here, so that a NAN or other corner-case value will
    // thrown an exception (but normal values get converted at full speed).
    // However, I2C adapters and other float-stack manglers leave pending
    // invalid-op exceptions hanging.  We would have to clear them before
    // enabling them and that is more expensive than just testing for the
    // invalid value Intel stores down in the corner cases.
    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x04);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as an int, popping the FPU stack
    emit_opcode(cbuf,0xDB);            // FISTP [ESP]
    emit_opcode(cbuf,0x1C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x3D);       // CMP EAX,imm
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
    // CALL directly to the runtime
    cbuf.set_inst_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2i_wrapper() - cbuf.code_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  }
}
void  convXD2I_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x2C));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));

    // Compare the result to see if we need to go to the slow path
    emit_opcode(cbuf,0x81);       // CMP dst,imm
    emit_rm    (cbuf,0x3,0x7,opnd_array(0)->reg(ra_,this));
    emit_d32   (cbuf,0x80000000); //         0x80000000

    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x13);       // Size of slow_call
    // Store xmm to a temp memory
    // location and push it onto stack.

    emit_opcode(cbuf,0x83);  // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf, (0x1 /*primary()*/) ? 0x8 : 0x4);

    emit_opcode  (cbuf, (0x1 /*primary()*/) ? 0xF2 : 0xF3 );   // MOVSS [ESP], xmm
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf, (0x1 /*primary()*/) ? 0xDD : 0xD9 );      // FLD [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,4
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf, (0x1 /*primary()*/) ? 0x8 : 0x4);

    // CALL directly to the runtime
    cbuf.set_inst_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2i_wrapper() - cbuf.code_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );

    // Carry on here...
  }
}
void  convD2L_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );   // FLD ST(i-1)
  }
  {
    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as a long, popping the FPU stack
    emit_opcode(cbuf,0xDF);            // FISTP [ESP]
    emit_opcode(cbuf,0x3C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x5A);       // POP EDX
    emit_opcode(cbuf,0x81);       // CMP EDX,imm
    emit_d8    (cbuf,0xFA);       // rdx
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07+4);     // Size of slow_call
    emit_opcode(cbuf,0x85);       // TEST EAX,EAX
    emit_opcode(cbuf,0xC0);       // 2/rax,/rax,
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
    // CALL directly to the runtime
    cbuf.set_inst_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2l_wrapper() - cbuf.code_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  }
}
void  convXD2L_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Allocate a word
    emit_opcode(cbuf,0x83);      // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);

    emit_opcode  (cbuf, 0xF2 );  // MOVSD [ESP], src
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xDD );     // FLD_D [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xD9);      // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());

    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as a long, popping the FPU stack
    emit_opcode(cbuf,0xDF);      // FISTP [ESP]
    emit_opcode(cbuf,0x3C);
    emit_d8(cbuf,0x24);

    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);      // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
      ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
      : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);      // POP EAX

    emit_opcode(cbuf,0x5A);      // POP EDX

    emit_opcode(cbuf,0x81);      // CMP EDX,imm
    emit_d8    (cbuf,0xFA);      // rdx
    emit_d32   (cbuf,0x80000000); //         0x80000000

    emit_opcode(cbuf,0x75);      // JNE around_slow_call
    emit_d8    (cbuf,0x13+4);    // Size of slow_call

    emit_opcode(cbuf,0x85);      // TEST EAX,EAX
    emit_opcode(cbuf,0xC0);      // 2/rax,/rax,

    emit_opcode(cbuf,0x75);      // JNE around_slow_call
    emit_d8    (cbuf,0x13);      // Size of slow_call

    // Push src onto stack slow-path
    // Allocate a word
    emit_opcode(cbuf,0x83);      // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);

    emit_opcode  (cbuf, 0xF2 );  // MOVSD [ESP], src
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xDD );     // FLD_D [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);      // ADD ESP,8
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x08);

    // CALL directly to the runtime
    cbuf.set_inst_mark();
    emit_opcode(cbuf,0xE8);      // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2l_wrapper() - cbuf.code_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  }
}
void  convF2I_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    // Flip to round-to-zero mode.  We attempted to allow invalid-op
    // exceptions here, so that a NAN or other corner-case value will
    // thrown an exception (but normal values get converted at full speed).
    // However, I2C adapters and other float-stack manglers leave pending
    // invalid-op exceptions hanging.  We would have to clear them before
    // enabling them and that is more expensive than just testing for the
    // invalid value Intel stores down in the corner cases.
    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x04);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as an int, popping the FPU stack
    emit_opcode(cbuf,0xDB);            // FISTP [ESP]
    emit_opcode(cbuf,0x1C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x3D);       // CMP EAX,imm
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
    // CALL directly to the runtime
    cbuf.set_inst_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2i_wrapper() - cbuf.code_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  }
}
void  convX2I_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x2C));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));

    // Compare the result to see if we need to go to the slow path
    emit_opcode(cbuf,0x81);       // CMP dst,imm
    emit_rm    (cbuf,0x3,0x7,opnd_array(0)->reg(ra_,this));
    emit_d32   (cbuf,0x80000000); //         0x80000000

    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x13);       // Size of slow_call
    // Store xmm to a temp memory
    // location and push it onto stack.

    emit_opcode(cbuf,0x83);  // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf, (0x0 /*primary()*/) ? 0x8 : 0x4);

    emit_opcode  (cbuf, (0x0 /*primary()*/) ? 0xF2 : 0xF3 );   // MOVSS [ESP], xmm
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf, (0x0 /*primary()*/) ? 0xDD : 0xD9 );      // FLD [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,4
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf, (0x0 /*primary()*/) ? 0x8 : 0x4);

    // CALL directly to the runtime
    cbuf.set_inst_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2i_wrapper() - cbuf.code_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );

    // Carry on here...
  }
}
void  convF2L_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as a long, popping the FPU stack
    emit_opcode(cbuf,0xDF);            // FISTP [ESP]
    emit_opcode(cbuf,0x3C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x5A);       // POP EDX
    emit_opcode(cbuf,0x81);       // CMP EDX,imm
    emit_d8    (cbuf,0xFA);       // rdx
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07+4);     // Size of slow_call
    emit_opcode(cbuf,0x85);       // TEST EAX,EAX
    emit_opcode(cbuf,0xC0);       // 2/rax,/rax,
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
    // CALL directly to the runtime
    cbuf.set_inst_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2l_wrapper() - cbuf.code_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  }
}
void  convX2L_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Allocate a word
    emit_opcode(cbuf,0x83);      // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);

    emit_opcode  (cbuf, 0xF3 );  // MOVSS [ESP], src
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xD9 );     // FLD_S [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xD9);      // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::addr_fpu_cntrl_wrd_trunc());

    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as a long, popping the FPU stack
    emit_opcode(cbuf,0xDF);      // FISTP [ESP]
    emit_opcode(cbuf,0x3C);
    emit_d8(cbuf,0x24);

    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);      // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
      ? (int)StubRoutines::addr_fpu_cntrl_wrd_24()
      : (int)StubRoutines::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);      // POP EAX

    emit_opcode(cbuf,0x5A);      // POP EDX

    emit_opcode(cbuf,0x81);      // CMP EDX,imm
    emit_d8    (cbuf,0xFA);      // rdx
    emit_d32   (cbuf,0x80000000);//         0x80000000

    emit_opcode(cbuf,0x75);      // JNE around_slow_call
    emit_d8    (cbuf,0x13+4);    // Size of slow_call

    emit_opcode(cbuf,0x85);      // TEST EAX,EAX
    emit_opcode(cbuf,0xC0);      // 2/rax,/rax,

    emit_opcode(cbuf,0x75);      // JNE around_slow_call
    emit_d8    (cbuf,0x13);      // Size of slow_call

    // Allocate a word
    emit_opcode(cbuf,0x83);      // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x04);

    emit_opcode  (cbuf, 0xF3 );  // MOVSS [ESP], src
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x11 );
    encode_RegMem(cbuf, opnd_array(1)->reg(ra_,this,idx1), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0xD9 );     // FLD_S [ESP]
    encode_RegMem(cbuf, 0x0, ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);      // ADD ESP,4
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x04);

    // CALL directly to the runtime
    cbuf.set_inst_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::d2l_wrapper() - cbuf.code_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    // Carry on here...
  }
}
void  convI2D_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  convI2XD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xF2 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0x2A /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  convI2XD_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0xF2 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0x2A /*tertiary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  convXI2XD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1));
    __ cvtdq2pd(opnd_array(0)->as_XMMRegister(ra_,this), opnd_array(0)->as_XMMRegister(ra_,this));
  }
}
void  convI2D_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0xDB /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  conv24I2F_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  convI2F_SSFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  convI2F_SSF_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0xDB /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  convI2F_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  convI2F_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    emit_opcode(cbuf,(0xDB /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  convI2X_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xF3 /*primary()*/));
  }
  {
    emit_opcode(cbuf,(0x0F /*secondary()*/));
  }
  {
    emit_opcode(cbuf,(0x2A /*tertiary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  convXI2X_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1));
    __ cvtdq2ps(opnd_array(0)->as_XMMRegister(ra_,this), opnd_array(0)->as_XMMRegister(ra_,this));
  }
}
void  convI2L_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // mov $dst.lo,$src
    int dst_encoding = opnd_array(0)->reg(ra_,this);
    int src_encoding = opnd_array(1)->reg(ra_,this,idx1);
    encode_Copy( cbuf, dst_encoding  , src_encoding );
    // mov $dst.hi,$src
    encode_Copy( cbuf, HIGH_FROM_LOW(dst_encoding), src_encoding );
    // sar $dst.hi,31
    emit_opcode( cbuf, 0xC1 );
    emit_rm(cbuf, 0x3, 7, HIGH_FROM_LOW(dst_encoding) );
    emit_d8(cbuf, 0x1F );
  }
}
void  convI2L_reg_zexNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_opcode(cbuf,(0x33 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)), HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)));
  }
}
void  zerox_longNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    emit_opcode(cbuf,(0x33 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)), HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)));
  }
}
void  convL2D_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
    // pop stack
    emit_opcode(cbuf, 0x83); // add  SP, #8
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 0x8);
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  convL2XD_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
  }
  {
    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [ESP]

    // UseXmmLoadAndClearUpper ? movsd dst,[esp] : movlpd dst,[esp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
    encode_RegMem(cbuf, opnd_array(0)->reg(ra_,this), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,8
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,0x08);
  }
}
void  convL2X_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, 0 ); //FSTP_S [ESP]

    emit_opcode  (cbuf, 0xF3 );     // MOVSS dst(xmm), [ESP]
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, 0x10 );
    encode_RegMem(cbuf, opnd_array(0)->reg(ra_,this), ESP_enc, 0x4, 0, 0, false);

    emit_opcode(cbuf,0x83);    // ADD ESP,d8 (4 or 8)
    emit_opcode(cbuf,0xC4);
    emit_d8(cbuf,(0x8));
  }
}
void  convL2F_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
    // pop stack
    emit_opcode(cbuf, 0x83); // add  SP, #8
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 0x8);
  }
  {
    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  }
}
void  convL2I_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  MoveF2I_stack_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0x8B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  MoveF2I_reg_stackNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1) != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xD9, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_S  [ESP+dst]
  }
}
void  MoveF2I_reg_stack_sseNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x11));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  MoveF2I_reg_reg_sseNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, 0x66 );     // MOVD dst,src
    emit_opcode(cbuf, 0x0F );
    emit_opcode(cbuf, 0x7E );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
  }
}
void  MoveI2F_reg_stackNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    store_to_stackslot( cbuf, (0x89 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1), opnd_array(0)->disp(ra_,this,0) );
  }
}
void  MoveI2F_stack_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xD9 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( !opnd_array(1)->disp_is_oop(), "No oops here because no relo info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, false);
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  MoveI2F_stack_reg_sseNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x10));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  MoveI2F_reg_reg_sseNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf, 0x66 );     // MOVD dst,src
    emit_opcode(cbuf, 0x0F );
    emit_opcode(cbuf, 0x6E );
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  MoveD2L_stack_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0x8B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x8B /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this));  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(1)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  MoveD2L_reg_stackNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1) != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1) );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xDD, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_D  [ESP+dst]
  }
}
void  MoveD2L_reg_stack_sseNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x11));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  MoveD2L_reg_reg_sseNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    { // MOVD $dst.lo,$src
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x7E);
      emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(0)->reg(ra_,this));
    }
    { // PSHUFLW $tmp,$src,0x4E  (01001110b)
      emit_opcode(cbuf,0xF2);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x70);
      emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2), opnd_array(1)->reg(ra_,this,idx1));
      emit_d8(cbuf, 0x4E);
    }
    { // MOVD $dst.hi,$tmp
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x7E);
      emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2), HIGH_FROM_LOW(opnd_array(0)->reg(ra_,this)));
    }
  }
}
void  MoveL2D_reg_stackNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0x89 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_opcode(cbuf,(0x89 /*secondary()*/));
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1));  // Hi register of pair, computed from lo
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(0)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  MoveL2D_stack_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xDD /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( !opnd_array(1)->disp_is_oop(), "No oops here because no relo info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, false);
  }
  {
    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this) );
  }
}
void  MoveL2D_stack_reg_sseNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xF2));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x10));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  MoveL2D_stack_reg_sse_partialNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0x66));
  }
  {
    emit_opcode(cbuf,(0x0F));
  }
  {
    emit_opcode(cbuf,(0x12));
  }
  {
    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  MoveL2D_reg_reg_sseNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    { // MOVD $dst,$src.lo
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x6E);
      emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2), opnd_array(1)->reg(ra_,this,idx1));
    }
    { // MOVD $tmp,$src.hi
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x6E);
      emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
    }
    { // PUNPCKLDQ $dst,$tmp
      emit_opcode(cbuf,0x66);
      emit_opcode(cbuf,0x0F);
      emit_opcode(cbuf,0x62);
      emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2), opnd_array(3)->reg(ra_,this,idx3));
     }
  }
}
void  Repl8B_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    encode_CopyXD(cbuf, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
    __ punpcklbw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), 0x00);
  }
}
void  Repl8B_eRegINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ movdl(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_Register(opnd_array(1)->reg(ra_,this,idx1)));
  }
  {
    MacroAssembler _masm(&cbuf);

    encode_CopyXD(cbuf, opnd_array(0)->reg(ra_,this), opnd_array(0)->reg(ra_,this));
    __ punpcklbw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), 0x00);
  }
}
void  Repl8B_immI0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  }
}
void  Repl4S_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), 0x00);
  }
}
void  Repl4S_eRegINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ movdl(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_Register(opnd_array(1)->reg(ra_,this,idx1)));
  }
  {
    MacroAssembler _masm(&cbuf);

    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), 0x00);
  }
}
void  Repl4S_immI0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  }
}
void  Repl4C_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), 0x00);
  }
}
void  Repl4C_eRegINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ movdl(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_Register(opnd_array(1)->reg(ra_,this,idx1)));
  }
  {
    MacroAssembler _masm(&cbuf);

    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), 0x00);
  }
}
void  Repl4C_immI0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  }
}
void  Repl2I_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pshufd(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), (0x00));
  }
}
void  Repl2I_eRegINode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ movdl(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_Register(opnd_array(1)->reg(ra_,this,idx1)));
  }
  {
    MacroAssembler _masm(&cbuf);

    __ pshufd(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), (0x00));
  }
}
void  Repl2I_immI0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  }
}
void  Repl2F_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pshufd(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), (0xe0));
  }
}
void  Repl2F_regXNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pshufd(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), (0xe0));
  }
}
void  Repl2F_immXF0Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  }
}
void  rep_stosNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xD1));
  }
  {
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), (ECX_enc) );
  }
  {
    emit_opcode(cbuf,(0x33));
    emit_rm(cbuf, 0x3, (EAX_enc), (EAX_enc));
  }
  {
    emit_opcode(cbuf,(0xF3));
  }
  {
    emit_opcode(cbuf,(0xAB));
  }
}
void  string_compareNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    Label ECX_GOOD_LABEL, LENGTH_DIFF_LABEL,
          POP_LABEL, DONE_LABEL, CONT_LABEL,
          WHILE_HEAD_LABEL;
    MacroAssembler masm(&cbuf);

    // Get the first character position in both strings
    //         [8] char array, [12] offset, [16] count
    int value_offset  = java_lang_String::value_offset_in_bytes();
    int offset_offset = java_lang_String::offset_offset_in_bytes();
    int count_offset  = java_lang_String::count_offset_in_bytes();
    int base_offset   = arrayOopDesc::base_offset_in_bytes(T_CHAR);

    masm.movptr(rax, Address(rsi, value_offset));
    masm.movl(rcx, Address(rsi, offset_offset));
    masm.lea(rax, Address(rax, rcx, Address::times_2, base_offset));
    masm.movptr(rbx, Address(rdi, value_offset));
    masm.movl(rcx, Address(rdi, offset_offset));
    masm.lea(rbx, Address(rbx, rcx, Address::times_2, base_offset));

    // Compute the minimum of the string lengths(rsi) and the
    // difference of the string lengths (stack)


    if (VM_Version::supports_cmov()) {
      masm.movl(rdi, Address(rdi, count_offset));
      masm.movl(rsi, Address(rsi, count_offset));
      masm.movl(rcx, rdi);
      masm.subl(rdi, rsi);
      masm.push(rdi);
      masm.cmovl(Assembler::lessEqual, rsi, rcx);
    } else {
      masm.movl(rdi, Address(rdi, count_offset));
      masm.movl(rcx, Address(rsi, count_offset));
      masm.movl(rsi, rdi);
      masm.subl(rdi, rcx);
      masm.push(rdi);
      masm.jcc(Assembler::lessEqual, ECX_GOOD_LABEL);
      masm.movl(rsi, rcx);
      // rsi holds min, rcx is unused
    }

    // Is the minimum length zero?
    masm.bind(ECX_GOOD_LABEL);
    masm.testl(rsi, rsi);
    masm.jcc(Assembler::zero, LENGTH_DIFF_LABEL);

    // Load first characters
    masm.load_unsigned_word(rcx, Address(rbx, 0));
    masm.load_unsigned_word(rdi, Address(rax, 0));

    // Compare first characters
    masm.subl(rcx, rdi);
    masm.jcc(Assembler::notZero,  POP_LABEL);
    masm.decrementl(rsi);
    masm.jcc(Assembler::zero, LENGTH_DIFF_LABEL);

    {
      // Check after comparing first character to see if strings are equivalent
      Label LSkip2;
      // Check if the strings start at same location
      masm.cmpptr(rbx,rax);
      masm.jcc(Assembler::notEqual, LSkip2);

      // Check if the length difference is zero (from stack)
      masm.cmpl(Address(rsp, 0), 0x0);
      masm.jcc(Assembler::equal,  LENGTH_DIFF_LABEL);

      // Strings might not be equivalent
      masm.bind(LSkip2);
    }

    // Shift rax, and rbx, to the end of the arrays, negate min
    masm.lea(rax, Address(rax, rsi, Address::times_2, 2));
    masm.lea(rbx, Address(rbx, rsi, Address::times_2, 2));
    masm.negl(rsi);

    // Compare the rest of the characters
    masm.bind(WHILE_HEAD_LABEL);
    masm.load_unsigned_word(rcx, Address(rbx, rsi, Address::times_2, 0));
    masm.load_unsigned_word(rdi, Address(rax, rsi, Address::times_2, 0));
    masm.subl(rcx, rdi);
    masm.jcc(Assembler::notZero, POP_LABEL);
    masm.incrementl(rsi);
    masm.jcc(Assembler::notZero, WHILE_HEAD_LABEL);

    // Strings are equal up to min length.  Return the length difference.
    masm.bind(LENGTH_DIFF_LABEL);
    masm.pop(rcx);
    masm.jmp(DONE_LABEL);

    // Discard the stored length difference
    masm.bind(POP_LABEL);
    masm.addptr(rsp, 4);

    // That's it
    masm.bind(DONE_LABEL);
  }
}
void  array_equalsNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    Label TRUE_LABEL, FALSE_LABEL, DONE_LABEL, COMPARE_LOOP_HDR, COMPARE_LOOP;
    MacroAssembler masm(&cbuf);

    Register ary1Reg   = as_Register(opnd_array(1)->reg(ra_,this,idx1));
    Register ary2Reg   = as_Register(opnd_array(2)->reg(ra_,this,idx2));
    Register tmp1Reg   = as_Register(EAX_enc);
    Register tmp2Reg   = as_Register(EBX_enc);
    Register resultReg = as_Register(opnd_array(0)->reg(ra_,this));

    int length_offset  = arrayOopDesc::length_offset_in_bytes();
    int base_offset    = arrayOopDesc::base_offset_in_bytes(T_CHAR);

    // Check the input args
    masm.cmpl(ary1Reg, ary2Reg);
    masm.jcc(Assembler::equal, TRUE_LABEL);
    masm.testl(ary1Reg, ary1Reg);
    masm.jcc(Assembler::zero, FALSE_LABEL);
    masm.testl(ary2Reg, ary2Reg);
    masm.jcc(Assembler::zero, FALSE_LABEL);

    // Check the lengths
    masm.movl(tmp2Reg, Address(ary1Reg, length_offset));
    masm.movl(resultReg, Address(ary2Reg, length_offset));
    masm.cmpl(tmp2Reg, resultReg);
    masm.jcc(Assembler::notEqual, FALSE_LABEL);
    masm.testl(resultReg, resultReg);
    masm.jcc(Assembler::zero, TRUE_LABEL);

    // Get the number of 4 byte vectors to compare
    masm.shrl(resultReg, 1);

    // Check for odd-length arrays
    masm.andl(tmp2Reg, 1);
    masm.testl(tmp2Reg, tmp2Reg);
    masm.jcc(Assembler::zero, COMPARE_LOOP_HDR);

    // Compare 2-byte "tail" at end of arrays
    masm.load_unsigned_word(tmp1Reg, Address(ary1Reg, resultReg, Address::times_4, base_offset));
    masm.load_unsigned_word(tmp2Reg, Address(ary2Reg, resultReg, Address::times_4, base_offset));
    masm.cmpl(tmp1Reg, tmp2Reg);
    masm.jcc(Assembler::notEqual, FALSE_LABEL);
    masm.testl(resultReg, resultReg);
    masm.jcc(Assembler::zero, TRUE_LABEL);

    // Setup compare loop
    masm.bind(COMPARE_LOOP_HDR);
    // Shift tmp1Reg and tmp2Reg to the last 4-byte boundary of the arrays
    masm.leal(tmp1Reg, Address(ary1Reg, resultReg, Address::times_4, base_offset));
    masm.leal(tmp2Reg, Address(ary2Reg, resultReg, Address::times_4, base_offset));
    masm.negl(resultReg);

    // 4-byte-wide compare loop
    masm.bind(COMPARE_LOOP);
    masm.movl(ary1Reg, Address(tmp1Reg, resultReg, Address::times_4, 0));
    masm.movl(ary2Reg, Address(tmp2Reg, resultReg, Address::times_4, 0));
    masm.cmpl(ary1Reg, ary2Reg);
    masm.jcc(Assembler::notEqual, FALSE_LABEL);
    masm.increment(resultReg);
    masm.jcc(Assembler::notZero, COMPARE_LOOP);

    masm.bind(TRUE_LABEL);
    masm.movl(resultReg, 1);   // return true
    masm.jmp(DONE_LABEL);

    masm.bind(FALSE_LABEL);
    masm.xorl(resultReg, resultReg); // return false

    // That's it
    masm.bind(DONE_LABEL);
  }
}
void  compI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x3B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  compI_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  compI_eReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x3B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  testI_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x85 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  testI_reg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode(cbuf,(0xF7 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
  {
    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  }
}
void  testI_reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    emit_opcode(cbuf,(0x85 /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  compU_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x3B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  compU_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  }
}
void  compU_eReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x3B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  testU_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x85 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  compP_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x3B /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  compP_eReg_immNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
  }
  {
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      
if ( opnd_array(2)->constant_is_oop() ) {
  emit_d32_reloc(cbuf, opnd_array(2)->constant(), relocInfo::oop_type, 0);
} else {
  emit_d32(cbuf, opnd_array(2)->constant());
};
    }
  }
}
void  compP_eReg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x3B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  compP_mem_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x3B /*primary()*/));
  }
  {
    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1);
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  testP_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x85 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(1)->reg(ra_,this,idx1));
  }
}
void  testP_Reg_memNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xF7 /*primary()*/));
  }
  {
    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d32(cbuf,(0xFFFFFFFF));
  }
}
void  minI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Compare dst,src
    emit_opcode(cbuf,0x3B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
    // jmp dst < src around move
    emit_opcode(cbuf,0x7C);
    emit_d8(cbuf,2);
    // move dst,src
    emit_opcode(cbuf,0x8B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  maxI_eRegNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Compare dst,src
    emit_opcode(cbuf,0x3B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
    // jmp dst > src around move
    emit_opcode(cbuf,0x7F);
    emit_d8(cbuf,2);
    // move dst,src
    emit_opcode(cbuf,0x8B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2));
  }
}
void  jumpXtndNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    address table_base  = __ address_table_constant(_index2label);

    // Jump to Address(table_base + switch_reg)
    InternalAddress table(table_base);
    Address index(noreg, opnd_array(1)->as_Register(ra_,this,idx1), Address::times_1);
    __ jump(ArrayAddress(table, index));
  }
}
void  jmpDirNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xE9 /*primary()*/));
  }
  {
    Label *l = opnd_array(1)->label();
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.code_size()+4)) : 0);
  }
}
uint  jmpDirNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 5, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 5);
}
void  jmpConNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Label *l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.code_size()+4)) : 0);
  }
}
uint  jmpConNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void  jmpLoopEndNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Label *l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.code_size()+4)) : 0);
  }
}
uint  jmpLoopEndNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void  jmpLoopEndUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Label *l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.code_size()+4)) : 0);
  }
}
uint  jmpLoopEndUNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void  jmpLoopEndUCFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Label *l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.code_size()+4)) : 0);
  }
}
uint  jmpLoopEndUCFNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void  jmpConUNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Label *l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.code_size()+4)) : 0);
  }
}
uint  jmpConUNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void  jmpConUCFNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Label *l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.code_size()+4)) : 0);
  }
}
uint  jmpConUCFNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void  jmpConUCF2Node::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    Label* l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), Assembler::parity);
    int parity_disp = -1;
    bool ok = false;
    if (opnd_array(1)->ccode() == Assembler::notEqual) {
       // the two jumps 6 bytes apart so the jump distances are too
       parity_disp = l ? (l->loc_pos() - (cbuf.code_size() + 4)) : 0;
    } else if (opnd_array(1)->ccode() == Assembler::equal) {
       parity_disp = 6;
       ok = true;
    } else {
       ShouldNotReachHere();
    }
    emit_d32(cbuf, parity_disp);
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.code_size() + 4)) : 0;
    emit_d32(cbuf, disp);
  }
}
uint  jmpConUCF2Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 12);
}
void  partialSubtypeCheckNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    Register Redi = as_Register(EDI_enc); // result register
    Register Reax = as_Register(EAX_enc); // super class
    Register Recx = as_Register(ECX_enc); // killed
    Register Resi = as_Register(ESI_enc); // sub class
    Label hit, miss;

    MacroAssembler _masm(&cbuf);
    // Compare super with sub directly, since super is not in its own SSA.
    // The compiler used to emit this test, but we fold it in here,
    // to allow platform-specific tweaking on sparc.
    __ cmpptr(Reax, Resi);
    __ jcc(Assembler::equal, hit);
#ifndef PRODUCT
    __ incrementl(ExternalAddress((address)&SharedRuntime::_partial_subtype_ctr));
#endif //PRODUCT
    __ movptr(Redi,Address(Resi,sizeof(oopDesc) + Klass::secondary_supers_offset_in_bytes()));
    __ movl(Recx,Address(Redi,arrayOopDesc::length_offset_in_bytes()));
    __ addptr(Redi,arrayOopDesc::base_offset_in_bytes(T_OBJECT));
    __ repne_scan();
    __ jcc(Assembler::notEqual, miss);
    __ movptr(Address(Resi,sizeof(oopDesc) + Klass::secondary_super_cache_offset_in_bytes()),Reax);
    __ bind(hit);
    if( (0x1 /*primary()*/) )
      __ xorptr(Redi,Redi);
    __ bind(miss);
  }
}
void  partialSubtypeCheck_vs_ZeroNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Register Redi = as_Register(EDI_enc); // result register
    Register Reax = as_Register(EAX_enc); // super class
    Register Recx = as_Register(ECX_enc); // killed
    Register Resi = as_Register(ESI_enc); // sub class
    Label hit, miss;

    MacroAssembler _masm(&cbuf);
    // Compare super with sub directly, since super is not in its own SSA.
    // The compiler used to emit this test, but we fold it in here,
    // to allow platform-specific tweaking on sparc.
    __ cmpptr(Reax, Resi);
    __ jcc(Assembler::equal, hit);
#ifndef PRODUCT
    __ incrementl(ExternalAddress((address)&SharedRuntime::_partial_subtype_ctr));
#endif //PRODUCT
    __ movptr(Redi,Address(Resi,sizeof(oopDesc) + Klass::secondary_supers_offset_in_bytes()));
    __ movl(Recx,Address(Redi,arrayOopDesc::length_offset_in_bytes()));
    __ addptr(Redi,arrayOopDesc::base_offset_in_bytes(T_OBJECT));
    __ repne_scan();
    __ jcc(Assembler::notEqual, miss);
    __ movptr(Address(Resi,sizeof(oopDesc) + Klass::secondary_super_cache_offset_in_bytes()),Reax);
    __ bind(hit);
    if( (0x0 /*primary()*/) )
      __ xorptr(Redi,Redi);
    __ bind(miss);
  }
}
void  jmpDir_shortNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    emit_opcode(cbuf,(0xEB /*primary()*/));
  }
  {
    Label *l = opnd_array(1)->label();
    int disp = l ? (l->loc_pos() - (cbuf.code_size()+1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  }
}
uint  jmpDir_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  jmpCon_shortNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.code_size()+1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  }
}
uint  jmpCon_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  jmpLoopEnd_shortNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.code_size()+1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  }
}
uint  jmpLoopEnd_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  jmpLoopEndU_shortNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.code_size()+1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  }
}
uint  jmpLoopEndU_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  jmpLoopEndUCF_shortNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.code_size()+1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  }
}
uint  jmpLoopEndUCF_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  jmpConU_shortNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.code_size()+1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  }
}
uint  jmpConU_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  jmpConUCF_shortNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.code_size()+1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  }
}
uint  jmpConUCF_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void  jmpConUCF2_shortNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    Label* l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), Assembler::parity);
    int parity_disp = -1;
    if (opnd_array(1)->ccode() == Assembler::notEqual) {
      parity_disp = l ? (l->loc_pos() - (cbuf.code_size() + 1)) : 0;
    } else if (opnd_array(1)->ccode() == Assembler::equal) {
      parity_disp = 2;
    } else {
      ShouldNotReachHere();
    }
    emit_d8(cbuf, parity_disp);
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.code_size() + 1)) : 0;
    emit_d8(cbuf, disp);
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    assert(-128 <= parity_disp && parity_disp <= 127, "Displacement too large for short jmp");
  }
}
uint  jmpConUCF2_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void  cmpL3_reg_regNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // Define a MacroAssembler instance for use by the encoding.  The
    // name is chosen to match the __ idiom used for assembly in other
    // parts of hotspot and assumes the existence of the standard
    // #define __ _masm.
    MacroAssembler _masm(&cbuf);

    Label p_one, m_one, done;
    __ xorptr(opnd_array(0)->as_Register(ra_,this), opnd_array(0)->as_Register(ra_,this));
    __ cmpl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)), HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)));
    __ jccb(Assembler::less,    m_one);
    __ jccb(Assembler::greater, p_one);
    __ cmpl(opnd_array(1)->as_Register(ra_,this,idx1), opnd_array(2)->as_Register(ra_,this,idx2));
    __ jccb(Assembler::below,   m_one);
    __ jccb(Assembler::equal,   done);
    __ bind(p_one);
    __ incrementl(opnd_array(0)->as_Register(ra_,this));
    __ jmpb(done);
    __ bind(m_one);
    __ decrementl(opnd_array(0)->as_Register(ra_,this));
    __ bind(done);
  }
}
void  cmpL_zero_flags_LTGENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0x85 /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
  }
}
void  cmpL_reg_flags_LTGENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2) );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)) );
  }
}
void  cmovLL_reg_LTGENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)));
  }
}
void  cmovLL_mem_LTGENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3);
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3));  // Hi register of pair, computed from lo
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(4)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  cmovII_reg_LTGENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
}
void  cmovII_mem_LTGENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3);
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  cmovPP_reg_LTGENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
}
void  cmpL_zero_flags_EQNENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // MOV   $tmp,$src.lo
    emit_opcode(cbuf, 0x8B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(1)->reg(ra_,this,idx1));
    // OR    $tmp,$src.hi
    emit_opcode(cbuf, 0x0B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)));
  }
}
void  cmpL_reg_flags_EQNENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    // CMP    $src1.lo,$src2.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1), opnd_array(2)->reg(ra_,this,idx2) );
    // JNE,s  skip
    emit_cc(cbuf, 0x70, 0x5);
    emit_d8(cbuf,2);
    // CMP    $src1.hi,$src2.hi
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)) );
  }
}
void  cmovLL_reg_EQNENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)));
  }
}
void  cmovLL_mem_EQNENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3);
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3));  // Hi register of pair, computed from lo
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(4)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  cmovII_reg_EQNENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
}
void  cmovII_mem_EQNENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3);
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  cmovPP_reg_EQNENode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
}
void  cmpL_zero_flags_LEGTNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // XOR    $tmp,$tmp
    emit_opcode(cbuf,0x33);  // XOR
    emit_rm(cbuf,0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(3)->reg(ra_,this,idx3));
    // CMP    $tmp,$src.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(1)->reg(ra_,this,idx1) );
    // SBB    $tmp,$src.hi
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)) );
  }
}
void  cmpL_reg_flags_LEGTNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2), opnd_array(1)->reg(ra_,this,idx1) );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), HIGH_FROM_LOW(opnd_array(2)->reg(ra_,this,idx2)) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), HIGH_FROM_LOW(opnd_array(1)->reg(ra_,this,idx1)) );
  }
}
void  cmovLL_reg_LEGTNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3)), HIGH_FROM_LOW(opnd_array(4)->reg(ra_,this,idx4)));
  }
}
void  cmovLL_mem_LEGTNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3);
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    int reg_encoding = HIGH_FROM_LOW(opnd_array(3)->reg(ra_,this,idx3));  // Hi register of pair, computed from lo
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4) + 4;      // Offset is 4 further in memory
    assert( !opnd_array(4)->disp_is_oop(), "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, false/*disp_is_oop*/);
  }
}
void  cmovII_reg_LEGTNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
}
void  cmovII_mem_LEGTNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3);
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_is_oop);
  }
}
void  cmovPP_reg_LEGTNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  }
  {
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3), opnd_array(4)->reg(ra_,this,idx4));
  }
}
void  CallStaticJavaDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // If method sets FPU control word restore it here
    if( Compile::current()->in_24_bit_fp_mode() ) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
  }
  {
    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to determine
    // who we intended to call.
    cbuf.set_inst_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    if ( !_method ) {
      emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.code_end()) - 4),
                     runtime_call_Relocation::spec(), RELOC_IMM32 );
    } else if(_optimized_virtual) {
      emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.code_end()) - 4),
                     opt_virtual_call_Relocation::spec(), RELOC_IMM32 );
    } else {
      emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.code_end()) - 4),
                     static_call_Relocation::spec(), RELOC_IMM32 );
    }
    if( _method ) {  // Emit stub for static call
      emit_java_to_interp(cbuf);
    }
  }
  {
    if( VerifyStackAtCalls ) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize = ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP,-3*VMRegImpl::slots_per_word));
      if(framesize >= 128) {
        emit_opcode(cbuf, 0x81); // cmp [esp+0],0xbadb1ood
        emit_d8(cbuf,0xBC);
        emit_d8(cbuf,0x24);
        emit_d32(cbuf,framesize); // Find majik cookie from ESP
        emit_d32(cbuf, 0xbadb100d);
      }
      else {
        emit_opcode(cbuf, 0x81); // cmp [esp+0],0xbadb1ood
        emit_d8(cbuf,0x7C);
        emit_d8(cbuf,0x24);
        emit_d8(cbuf,framesize); // Find majik cookie from ESP
        emit_d32(cbuf, 0xbadb100d);
      }
      // jmp EQ around INT3
      // QQQ TODO
      const int jump_around = 5; // size of call to breakpoint, 1 for CC
      emit_opcode(cbuf,0x74);
      emit_d8(cbuf, jump_around);
      // QQQ temporary
      emit_break(cbuf);
      // Die if stack mismatch
      // emit_opcode(cbuf,0xCC);
    }
  }
  {
    // If method sets FPU control word do it here also
    if( Compile::current()->in_24_bit_fp_mode() ) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    }
  }
}
void  CallDynamicJavaDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // If method sets FPU control word restore it here
    if( Compile::current()->in_24_bit_fp_mode() ) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
  }
  {
    // !!!!!
    // Generate  "Mov EAX,0x00", placeholder instruction to load oop-info
    // emit_call_dynamic_prologue( cbuf );
    cbuf.set_inst_mark();
    emit_opcode(cbuf, 0xB8 + EAX_enc);        // mov    EAX,-1
    emit_d32_reloc(cbuf, (int)Universe::non_oop_word(), oop_Relocation::spec_for_immediate(), RELOC_IMM32);
    address  virtual_call_oop_addr = cbuf.inst_mark();
    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to determine
    // who we intended to call.
    cbuf.set_inst_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.code_end()) - 4),
                virtual_call_Relocation::spec(virtual_call_oop_addr), RELOC_IMM32 );
  }
  {
    if( VerifyStackAtCalls ) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize = ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP,-3*VMRegImpl::slots_per_word));
      if(framesize >= 128) {
        emit_opcode(cbuf, 0x81); // cmp [esp+0],0xbadb1ood
        emit_d8(cbuf,0xBC);
        emit_d8(cbuf,0x24);
        emit_d32(cbuf,framesize); // Find majik cookie from ESP
        emit_d32(cbuf, 0xbadb100d);
      }
      else {
        emit_opcode(cbuf, 0x81); // cmp [esp+0],0xbadb1ood
        emit_d8(cbuf,0x7C);
        emit_d8(cbuf,0x24);
        emit_d8(cbuf,framesize); // Find majik cookie from ESP
        emit_d32(cbuf, 0xbadb100d);
      }
      // jmp EQ around INT3
      // QQQ TODO
      const int jump_around = 5; // size of call to breakpoint, 1 for CC
      emit_opcode(cbuf,0x74);
      emit_d8(cbuf, jump_around);
      // QQQ temporary
      emit_break(cbuf);
      // Die if stack mismatch
      // emit_opcode(cbuf,0xCC);
    }
  }
  {
    // If method sets FPU control word do it here also
    if( Compile::current()->in_24_bit_fp_mode() ) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    }
  }
}
void  CallRuntimeDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // If method sets FPU control word restore it here
    if( Compile::current()->in_24_bit_fp_mode() ) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
  }
  {
    MacroAssembler masm(&cbuf);
    int start = masm.offset();
    if (UseSSE >= 2) {
      if (VerifyFPU) {
        masm.verify_FPU(0, "must be empty in SSE2+ mode");
      }
    } else {
      // External c_calling_convention expects the FPU stack to be 'clean'.
      // Compiled code leaves it dirty.  Do cleanup now.
      masm.empty_FPU_stack();
    }
    if (sizeof_FFree_Float_Stack_All == -1) {
      sizeof_FFree_Float_Stack_All = masm.offset() - start;
    } else {
      assert(masm.offset() - start == sizeof_FFree_Float_Stack_All, "wrong size");
    }
  }
  {
    // This is the instruction starting address for relocation info.
    cbuf.set_inst_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    // CALL directly to the runtime
    emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.code_end()) - 4),
                runtime_call_Relocation::spec(), RELOC_IMM32 );

    if (UseSSE >= 2) {
      MacroAssembler _masm(&cbuf);
      BasicType rt = tf()->return_type();

      if ((rt == T_FLOAT || rt == T_DOUBLE) && !return_value_is_used()) {
        // A C runtime call where the return value is unused.  In SSE2+
        // mode the result needs to be removed from the FPU stack.  It's
        // likely that this function call could be removed by the
        // optimizer if the C function is a pure function.
        __ ffree(0);
      } else if (rt == T_FLOAT) {
        __ lea(rsp, Address(rsp, -4));
        __ fstp_s(Address(rsp, 0));
        __ movflt(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  4));
      } else if (rt == T_DOUBLE) {
        __ lea(rsp, Address(rsp, -8));
        __ fstp_d(Address(rsp, 0));
        __ movdbl(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  8));
      }
    }
  }
  {
    // If method sets FPU control word do it here also
    if( Compile::current()->in_24_bit_fp_mode() ) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    }
  }
}
void  CallLeafDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // If method sets FPU control word restore it here
    if( Compile::current()->in_24_bit_fp_mode() ) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_std()));
    }
  }
  {
    MacroAssembler masm(&cbuf);
    int start = masm.offset();
    if (UseSSE >= 2) {
      if (VerifyFPU) {
        masm.verify_FPU(0, "must be empty in SSE2+ mode");
      }
    } else {
      // External c_calling_convention expects the FPU stack to be 'clean'.
      // Compiled code leaves it dirty.  Do cleanup now.
      masm.empty_FPU_stack();
    }
    if (sizeof_FFree_Float_Stack_All == -1) {
      sizeof_FFree_Float_Stack_All = masm.offset() - start;
    } else {
      assert(masm.offset() - start == sizeof_FFree_Float_Stack_All, "wrong size");
    }
  }
  {
    // This is the instruction starting address for relocation info.
    cbuf.set_inst_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    // CALL directly to the runtime
    emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.code_end()) - 4),
                runtime_call_Relocation::spec(), RELOC_IMM32 );

    if (UseSSE >= 2) {
      MacroAssembler _masm(&cbuf);
      BasicType rt = tf()->return_type();

      if ((rt == T_FLOAT || rt == T_DOUBLE) && !return_value_is_used()) {
        // A C runtime call where the return value is unused.  In SSE2+
        // mode the result needs to be removed from the FPU stack.  It's
        // likely that this function call could be removed by the
        // optimizer if the C function is a pure function.
        __ ffree(0);
      } else if (rt == T_FLOAT) {
        __ lea(rsp, Address(rsp, -4));
        __ fstp_s(Address(rsp, 0));
        __ movflt(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  4));
      } else if (rt == T_DOUBLE) {
        __ lea(rsp, Address(rsp, -8));
        __ fstp_d(Address(rsp, 0));
        __ movdbl(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  8));
      }
    }
  }
  {
    if( VerifyFPU ) {
      MacroAssembler masm(&cbuf);
      masm.verify_FPU( -3, "Returning from Runtime Leaf call");
    }
  }
  {
    // If method sets FPU control word do it here also
    if( Compile::current()->in_24_bit_fp_mode() ) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::addr_fpu_cntrl_wrd_24()));
    }
  }
}
void  CallLeafNoFPDirectNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    // This is the instruction starting address for relocation info.
    cbuf.set_inst_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    // CALL directly to the runtime
    emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.code_end()) - 4),
                runtime_call_Relocation::spec(), RELOC_IMM32 );

    if (UseSSE >= 2) {
      MacroAssembler _masm(&cbuf);
      BasicType rt = tf()->return_type();

      if ((rt == T_FLOAT || rt == T_DOUBLE) && !return_value_is_used()) {
        // A C runtime call where the return value is unused.  In SSE2+
        // mode the result needs to be removed from the FPU stack.  It's
        // likely that this function call could be removed by the
        // optimizer if the C function is a pure function.
        __ ffree(0);
      } else if (rt == T_FLOAT) {
        __ lea(rsp, Address(rsp, -4));
        __ fstp_s(Address(rsp, 0));
        __ movflt(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  4));
      } else if (rt == T_DOUBLE) {
        __ lea(rsp, Address(rsp, -8));
        __ fstp_d(Address(rsp, 0));
        __ movdbl(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  8));
      }
    }
  }
}
void  RetNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {
    emit_opcode(cbuf,(0xC3 /*primary()*/));
  }
}
void  TailCalljmpIndNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,(0xFF /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  tailjmpIndNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    emit_opcode(cbuf,0x5A);
  }
  {
    emit_opcode(cbuf,(0xFF /*primary()*/));
  }
  {
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) );
  }
}
void  CreateExceptionNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  CreateExceptionNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void  RethrowExceptionNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {
    cbuf.set_inst_mark();
    emit_opcode(cbuf, 0xE9);        // jmp    entry
    emit_d32_reloc(cbuf, (int)OptoRuntime::rethrow_stub() - ((int)cbuf.code_end())-4,
                   runtime_call_Relocation::spec(), RELOC_IMM32 );
  }
}
void  cmpFastLockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

    Register objReg = as_Register(opnd_array(1)->reg(ra_,this,idx1));
    Register boxReg = as_Register(opnd_array(2)->reg(ra_,this,idx2));
    Register tmpReg = as_Register(opnd_array(3)->reg(ra_,this,idx3));
    Register scrReg = as_Register(opnd_array(4)->reg(ra_,this,idx4));

    // Ensure the register assignents are disjoint
    guarantee (objReg != boxReg, "") ;
    guarantee (objReg != tmpReg, "") ;
    guarantee (objReg != scrReg, "") ;
    guarantee (boxReg != tmpReg, "") ;
    guarantee (boxReg != scrReg, "") ;
    guarantee (tmpReg == as_Register(EAX_enc), "") ;

    MacroAssembler masm(&cbuf);

    if (_counters != NULL) {
      masm.atomic_incl(ExternalAddress((address) _counters->total_entry_count_addr()));
    }
    if (EmitSync & 1) {
        // set box->dhw = unused_mark (3)
        // Force all sync thru slow-path: slow_enter() and slow_exit()
        masm.movptr (Address(boxReg, 0), int32_t(markOopDesc::unused_mark())) ;
        masm.cmpptr (rsp, (int32_t)0) ;
    } else
    if (EmitSync & 2) {
        Label DONE_LABEL ;
        if (UseBiasedLocking) {
           // Note: tmpReg maps to the swap_reg argument and scrReg to the tmp_reg argument.
           masm.biased_locking_enter(boxReg, objReg, tmpReg, scrReg, false, DONE_LABEL, NULL, _counters);
        }

        masm.movptr(tmpReg, Address(objReg, 0)) ;          // fetch markword
        masm.orptr (tmpReg, 0x1);
        masm.movptr(Address(boxReg, 0), tmpReg);           // Anticipate successful CAS
        if (os::is_MP()) { masm.lock();  }
        masm.cmpxchgptr(boxReg, Address(objReg, 0));          // Updates tmpReg
        masm.jcc(Assembler::equal, DONE_LABEL);
        // Recursive locking
        masm.subptr(tmpReg, rsp);
        masm.andptr(tmpReg, (int32_t) 0xFFFFF003 );
        masm.movptr(Address(boxReg, 0), tmpReg);
        masm.bind(DONE_LABEL) ;
    } else {
      // Possible cases that we'll encounter in fast_lock
      // ------------------------------------------------
      // * Inflated
      //    -- unlocked
      //    -- Locked
      //       = by self
      //       = by other
      // * biased
      //    -- by Self
      //    -- by other
      // * neutral
      // * stack-locked
      //    -- by self
      //       = sp-proximity test hits
      //       = sp-proximity test generates false-negative
      //    -- by other
      //

      Label IsInflated, DONE_LABEL, PopDone ;

      // TODO: optimize away redundant LDs of obj->mark and improve the markword triage
      // order to reduce the number of conditional branches in the most common cases.
      // Beware -- there's a subtle invariant that fetch of the markword
      // at [FETCH], below, will never observe a biased encoding (*101b).
      // If this invariant is not held we risk exclusion (safety) failure.
      if (UseBiasedLocking && !UseOptoBiasInlining) {
        masm.biased_locking_enter(boxReg, objReg, tmpReg, scrReg, false, DONE_LABEL, NULL, _counters);
      }

      masm.movptr(tmpReg, Address(objReg, 0)) ;         // [FETCH]
      masm.testptr(tmpReg, 0x02) ;                      // Inflated v (Stack-locked or neutral)
      masm.jccb  (Assembler::notZero, IsInflated) ;

      // Attempt stack-locking ...
      masm.orptr (tmpReg, 0x1);
      masm.movptr(Address(boxReg, 0), tmpReg);          // Anticipate successful CAS
      if (os::is_MP()) { masm.lock();  }
      masm.cmpxchgptr(boxReg, Address(objReg, 0));           // Updates tmpReg
      if (_counters != NULL) {
        masm.cond_inc32(Assembler::equal,
                        ExternalAddress((address)_counters->fast_path_entry_count_addr()));
      }
      masm.jccb (Assembler::equal, DONE_LABEL);

      // Recursive locking
      masm.subptr(tmpReg, rsp);
      masm.andptr(tmpReg, 0xFFFFF003 );
      masm.movptr(Address(boxReg, 0), tmpReg);
      if (_counters != NULL) {
        masm.cond_inc32(Assembler::equal,
                        ExternalAddress((address)_counters->fast_path_entry_count_addr()));
      }
      masm.jmp  (DONE_LABEL) ;

      masm.bind (IsInflated) ;

      // The object is inflated.
      //
      // TODO-FIXME: eliminate the ugly use of manifest constants:
      //   Use markOopDesc::monitor_value instead of "2".
      //   use markOop::unused_mark() instead of "3".
      // The tmpReg value is an objectMonitor reference ORed with
      // markOopDesc::monitor_value (2).   We can either convert tmpReg to an
      // objectmonitor pointer by masking off the "2" bit or we can just
      // use tmpReg as an objectmonitor pointer but bias the objectmonitor
      // field offsets with "-2" to compensate for and annul the low-order tag bit.
      //
      // I use the latter as it avoids AGI stalls.
      // As such, we write "mov r, [tmpReg+OFFSETOF(Owner)-2]"
      // instead of "mov r, [tmpReg+OFFSETOF(Owner)]".
      //
      #define OFFSET_SKEWED(f) ((ObjectMonitor::f ## _offset_in_bytes())-2)

      // boxReg refers to the on-stack BasicLock in the current frame.
      // We'd like to write:
      //   set box->_displaced_header = markOop::unused_mark().  Any non-0 value suffices.
      // This is convenient but results a ST-before-CAS penalty.  The following CAS suffers
      // additional latency as we have another ST in the store buffer that must drain.

      if (EmitSync & 8192) {
         masm.movptr(Address(boxReg, 0), 3) ;            // results in ST-before-CAS penalty
         masm.get_thread (scrReg) ;
         masm.movptr(boxReg, tmpReg);                    // consider: LEA box, [tmp-2]
         masm.movptr(tmpReg, 0);                         // consider: xor vs mov
         if (os::is_MP()) { masm.lock(); }
         masm.cmpxchgptr(scrReg, Address(boxReg, ObjectMonitor::owner_offset_in_bytes()-2)) ;
      } else
      if ((EmitSync & 128) == 0) {                      // avoid ST-before-CAS
         masm.movptr(scrReg, boxReg) ;
         masm.movptr(boxReg, tmpReg);                   // consider: LEA box, [tmp-2]

         // Using a prefetchw helps avoid later RTS->RTO upgrades and cache probes
         if ((EmitSync & 2048) && VM_Version::supports_3dnow() && os::is_MP()) {
            // prefetchw [eax + Offset(_owner)-2]
            masm.prefetchw(Address(rax, ObjectMonitor::owner_offset_in_bytes()-2));
         }

         if ((EmitSync & 64) == 0) {
           // Optimistic form: consider XORL tmpReg,tmpReg
           masm.movptr(tmpReg, 0 ) ;
         } else {
           // Can suffer RTS->RTO upgrades on shared or cold $ lines
           // Test-And-CAS instead of CAS
           masm.movptr(tmpReg, Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2)) ;   // rax, = m->_owner
           masm.testptr(tmpReg, tmpReg) ;                   // Locked ?
           masm.jccb  (Assembler::notZero, DONE_LABEL) ;
         }

         // Appears unlocked - try to swing _owner from null to non-null.
         // Ideally, I'd manifest "Self" with get_thread and then attempt
         // to CAS the register containing Self into m->Owner.
         // But we don't have enough registers, so instead we can either try to CAS
         // rsp or the address of the box (in scr) into &m->owner.  If the CAS succeeds
         // we later store "Self" into m->Owner.  Transiently storing a stack address
         // (rsp or the address of the box) into  m->owner is harmless.
         // Invariant: tmpReg == 0.  tmpReg is EAX which is the implicit cmpxchg comparand.
         if (os::is_MP()) { masm.lock();  }
         masm.cmpxchgptr(scrReg, Address(boxReg, ObjectMonitor::owner_offset_in_bytes()-2)) ;
         masm.movptr(Address(scrReg, 0), 3) ;          // box->_displaced_header = 3
         masm.jccb  (Assembler::notZero, DONE_LABEL) ;
         masm.get_thread (scrReg) ;                    // beware: clobbers ICCs
         masm.movptr(Address(boxReg, ObjectMonitor::owner_offset_in_bytes()-2), scrReg) ;
         masm.xorptr(boxReg, boxReg) ;                 // set icc.ZFlag = 1 to indicate success

         // If the CAS fails we can either retry or pass control to the slow-path.
         // We use the latter tactic.
         // Pass the CAS result in the icc.ZFlag into DONE_LABEL
         // If the CAS was successful ...
         //   Self has acquired the lock
         //   Invariant: m->_recursions should already be 0, so we don't need to explicitly set it.
         // Intentional fall-through into DONE_LABEL ...
      } else {
         masm.movptr(Address(boxReg, 0), 3) ;       // results in ST-before-CAS penalty
         masm.movptr(boxReg, tmpReg) ;

         // Using a prefetchw helps avoid later RTS->RTO upgrades and cache probes
         if ((EmitSync & 2048) && VM_Version::supports_3dnow() && os::is_MP()) {
            // prefetchw [eax + Offset(_owner)-2]
            masm.prefetchw(Address(rax, ObjectMonitor::owner_offset_in_bytes()-2));
         }

         if ((EmitSync & 64) == 0) {
           // Optimistic form
           masm.xorptr  (tmpReg, tmpReg) ;
         } else {
           // Can suffer RTS->RTO upgrades on shared or cold $ lines
           masm.movptr(tmpReg, Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2)) ;   // rax, = m->_owner
           masm.testptr(tmpReg, tmpReg) ;                   // Locked ?
           masm.jccb  (Assembler::notZero, DONE_LABEL) ;
         }

         // Appears unlocked - try to swing _owner from null to non-null.
         // Use either "Self" (in scr) or rsp as thread identity in _owner.
         // Invariant: tmpReg == 0.  tmpReg is EAX which is the implicit cmpxchg comparand.
         masm.get_thread (scrReg) ;
         if (os::is_MP()) { masm.lock(); }
         masm.cmpxchgptr(scrReg, Address(boxReg, ObjectMonitor::owner_offset_in_bytes()-2)) ;

         // If the CAS fails we can either retry or pass control to the slow-path.
         // We use the latter tactic.
         // Pass the CAS result in the icc.ZFlag into DONE_LABEL
         // If the CAS was successful ...
         //   Self has acquired the lock
         //   Invariant: m->_recursions should already be 0, so we don't need to explicitly set it.
         // Intentional fall-through into DONE_LABEL ...
      }

      // DONE_LABEL is a hot target - we'd really like to place it at the
      // start of cache line by padding with NOPs.
      // See the AMD and Intel software optimization manuals for the
      // most efficient "long" NOP encodings.
      // Unfortunately none of our alignment mechanisms suffice.
      masm.bind(DONE_LABEL);

      // Avoid branch-to-branch on AMD processors
      // This appears to be superstition.
      if (EmitSync & 32) masm.nop() ;


      // At DONE_LABEL the icc ZFlag is set as follows ...
      // Fast_Unlock uses the same protocol.
      // ZFlag == 1 -> Success
      // ZFlag == 0 -> Failure - force control through the slow-path
    }
  }
}
void  cmpFastUnlockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

    Register objReg = as_Register(opnd_array(1)->reg(ra_,this,idx1));
    Register boxReg = as_Register(opnd_array(2)->reg(ra_,this,idx2));
    Register tmpReg = as_Register(opnd_array(3)->reg(ra_,this,idx3));

    guarantee (objReg != boxReg, "") ;
    guarantee (objReg != tmpReg, "") ;
    guarantee (boxReg != tmpReg, "") ;
    guarantee (boxReg == as_Register(EAX_enc), "") ;
    MacroAssembler masm(&cbuf);

    if (EmitSync & 4) {
      // Disable - inhibit all inlining.  Force control through the slow-path
      masm.cmpptr (rsp, 0) ;
    } else
    if (EmitSync & 8) {
      Label DONE_LABEL ;
      if (UseBiasedLocking) {
         masm.biased_locking_exit(objReg, tmpReg, DONE_LABEL);
      }
      // classic stack-locking code ...
      masm.movptr(tmpReg, Address(boxReg, 0)) ;
      masm.testptr(tmpReg, tmpReg) ;
      masm.jcc   (Assembler::zero, DONE_LABEL) ;
      if (os::is_MP()) { masm.lock(); }
      masm.cmpxchgptr(tmpReg, Address(objReg, 0));          // Uses EAX which is box
      masm.bind(DONE_LABEL);
    } else {
      Label DONE_LABEL, Stacked, CheckSucc, Inflated ;

      // Critically, the biased locking test must have precedence over
      // and appear before the (box->dhw == 0) recursive stack-lock test.
      if (UseBiasedLocking && !UseOptoBiasInlining) {
         masm.biased_locking_exit(objReg, tmpReg, DONE_LABEL);
      }

      masm.cmpptr(Address(boxReg, 0), 0) ;            // Examine the displaced header
      masm.movptr(tmpReg, Address(objReg, 0)) ;       // Examine the object's markword
      masm.jccb  (Assembler::zero, DONE_LABEL) ;      // 0 indicates recursive stack-lock

      masm.testptr(tmpReg, 0x02) ;                     // Inflated?
      masm.jccb  (Assembler::zero, Stacked) ;

      masm.bind  (Inflated) ;
      // It's inflated.
      // Despite our balanced locking property we still check that m->_owner == Self
      // as java routines or native JNI code called by this thread might
      // have released the lock.
      // Refer to the comments in synchronizer.cpp for how we might encode extra
      // state in _succ so we can avoid fetching EntryList|cxq.
      //
      // I'd like to add more cases in fast_lock() and fast_unlock() --
      // such as recursive enter and exit -- but we have to be wary of
      // I$ bloat, T$ effects and BP$ effects.
      //
      // If there's no contention try a 1-0 exit.  That is, exit without
      // a costly MEMBAR or CAS.  See synchronizer.cpp for details on how
      // we detect and recover from the race that the 1-0 exit admits.
      //
      // Conceptually Fast_Unlock() must execute a STST|LDST "release" barrier
      // before it STs null into _owner, releasing the lock.  Updates
      // to data protected by the critical section must be visible before
      // we drop the lock (and thus before any other thread could acquire
      // the lock and observe the fields protected by the lock).
      // IA32's memory-model is SPO, so STs are ordered with respect to
      // each other and there's no need for an explicit barrier (fence).
      // See also http://gee.cs.oswego.edu/dl/jmm/cookbook.html.

      masm.get_thread (boxReg) ;
      if ((EmitSync & 4096) && VM_Version::supports_3dnow() && os::is_MP()) {
        // prefetchw [ebx + Offset(_owner)-2]
        masm.prefetchw(Address(rbx, ObjectMonitor::owner_offset_in_bytes()-2));
      }

      // Note that we could employ various encoding schemes to reduce
      // the number of loads below (currently 4) to just 2 or 3.
      // Refer to the comments in synchronizer.cpp.
      // In practice the chain of fetches doesn't seem to impact performance, however.
      if ((EmitSync & 65536) == 0 && (EmitSync & 256)) {
         // Attempt to reduce branch density - AMD's branch predictor.
         masm.xorptr(boxReg, Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2)) ;
         masm.orptr(boxReg, Address (tmpReg, ObjectMonitor::recursions_offset_in_bytes()-2)) ;
         masm.orptr(boxReg, Address (tmpReg, ObjectMonitor::EntryList_offset_in_bytes()-2)) ;
         masm.orptr(boxReg, Address (tmpReg, ObjectMonitor::cxq_offset_in_bytes()-2)) ;
         masm.jccb  (Assembler::notZero, DONE_LABEL) ;
         masm.movptr(Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2), 0) ;
         masm.jmpb  (DONE_LABEL) ;
      } else {
         masm.xorptr(boxReg, Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2)) ;
         masm.orptr(boxReg, Address (tmpReg, ObjectMonitor::recursions_offset_in_bytes()-2)) ;
         masm.jccb  (Assembler::notZero, DONE_LABEL) ;
         masm.movptr(boxReg, Address (tmpReg, ObjectMonitor::EntryList_offset_in_bytes()-2)) ;
         masm.orptr(boxReg, Address (tmpReg, ObjectMonitor::cxq_offset_in_bytes()-2)) ;
         masm.jccb  (Assembler::notZero, CheckSucc) ;
         masm.movptr(Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2), 0) ;
         masm.jmpb  (DONE_LABEL) ;
      }

      // The Following code fragment (EmitSync & 65536) improves the performance of
      // contended applications and contended synchronization microbenchmarks.
      // Unfortunately the emission of the code - even though not executed - causes regressions
      // in scimark and jetstream, evidently because of $ effects.  Replacing the code
      // with an equal number of never-executed NOPs results in the same regression.
      // We leave it off by default.

      if ((EmitSync & 65536) != 0) {
         Label LSuccess, LGoSlowPath ;

         masm.bind  (CheckSucc) ;

         // Optional pre-test ... it's safe to elide this
         if ((EmitSync & 16) == 0) {
            masm.cmpptr(Address (tmpReg, ObjectMonitor::succ_offset_in_bytes()-2), 0) ;
            masm.jccb  (Assembler::zero, LGoSlowPath) ;
         }

         // We have a classic Dekker-style idiom:
         //    ST m->_owner = 0 ; MEMBAR; LD m->_succ
         // There are a number of ways to implement the barrier:
         // (1) lock:andl &m->_owner, 0
         //     is fast, but mask doesn't currently support the "ANDL M,IMM32" form.
         //     LOCK: ANDL [ebx+Offset(_Owner)-2], 0
         //     Encodes as 81 31 OFF32 IMM32 or 83 63 OFF8 IMM8
         // (2) If supported, an explicit MFENCE is appealing.
         //     In older IA32 processors MFENCE is slower than lock:add or xchg
         //     particularly if the write-buffer is full as might be the case if
         //     if stores closely precede the fence or fence-equivalent instruction.
         //     In more modern implementations MFENCE appears faster, however.
         // (3) In lieu of an explicit fence, use lock:addl to the top-of-stack
         //     The $lines underlying the top-of-stack should be in M-state.
         //     The locked add instruction is serializing, of course.
         // (4) Use xchg, which is serializing
         //     mov boxReg, 0; xchgl boxReg, [tmpReg + Offset(_owner)-2] also works
         // (5) ST m->_owner = 0 and then execute lock:orl &m->_succ, 0.
         //     The integer condition codes will tell us if succ was 0.
         //     Since _succ and _owner should reside in the same $line and
         //     we just stored into _owner, it's likely that the $line
         //     remains in M-state for the lock:orl.
         //
         // We currently use (3), although it's likely that switching to (2)
         // is correct for the future.

         masm.movptr(Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2), 0) ;
         if (os::is_MP()) {
            if (VM_Version::supports_sse2() && 1 == FenceInstruction) {
              masm.mfence();
            } else {
              masm.lock () ; masm.addptr(Address(rsp, 0), 0) ;
            }
         }
         // Ratify _succ remains non-null
         masm.cmpptr(Address (tmpReg, ObjectMonitor::succ_offset_in_bytes()-2), 0) ;
         masm.jccb  (Assembler::notZero, LSuccess) ;

         masm.xorptr(boxReg, boxReg) ;                  // box is really EAX
         if (os::is_MP()) { masm.lock(); }
         masm.cmpxchgptr(rsp, Address(tmpReg, ObjectMonitor::owner_offset_in_bytes()-2));
         masm.jccb  (Assembler::notEqual, LSuccess) ;
         // Since we're low on registers we installed rsp as a placeholding in _owner.
         // Now install Self over rsp.  This is safe as we're transitioning from
         // non-null to non=null
         masm.get_thread (boxReg) ;
         masm.movptr(Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2), boxReg) ;
         // Intentional fall-through into LGoSlowPath ...

         masm.bind  (LGoSlowPath) ;
         masm.orptr(boxReg, 1) ;                      // set ICC.ZF=0 to indicate failure
         masm.jmpb  (DONE_LABEL) ;

         masm.bind  (LSuccess) ;
         masm.xorptr(boxReg, boxReg) ;                 // set ICC.ZF=1 to indicate success
         masm.jmpb  (DONE_LABEL) ;
      }

      masm.bind (Stacked) ;
      // It's not inflated and it's not recursively stack-locked and it's not biased.
      // It must be stack-locked.
      // Try to reset the header to displaced header.
      // The "box" value on the stack is stable, so we can reload
      // and be assured we observe the same value as above.
      masm.movptr(tmpReg, Address(boxReg, 0)) ;
      if (os::is_MP()) {   masm.lock();    }
      masm.cmpxchgptr(tmpReg, Address(objReg, 0)); // Uses EAX which is box
      // Intention fall-thru into DONE_LABEL


      // DONE_LABEL is a hot target - we'd really like to place it at the
      // start of cache line by padding with NOPs.
      // See the AMD and Intel software optimization manuals for the
      // most efficient "long" NOP encodings.
      // Unfortunately none of our alignment mechanisms suffice.
      if ((EmitSync & 65536) == 0) {
         masm.bind (CheckSucc) ;
      }
      masm.bind(DONE_LABEL);

      // Avoid branch to branch on AMD processors
      if (EmitSync & 32768) { masm.nop() ; }
    }
  }
}
void  safePoint_pollNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {
    cbuf.relocate(cbuf.inst_mark(), relocInfo::poll_type, 0);
    emit_opcode(cbuf,0x85);
    emit_rm (cbuf, 0x0, 0x7, 0x5);
    emit_d32(cbuf, (intptr_t)os::get_polling_page());
  }
}
uint  safePoint_pollNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void  tlsLoadPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    Register dstReg = as_Register(opnd_array(0)->reg(ra_,this));
    MacroAssembler* masm = new MacroAssembler(&cbuf);
      masm->get_thread(dstReg);
  }
}
void  ShouldNotReachHereNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
    cbuf.set_inst_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {
    MacroAssembler* masm = new MacroAssembler(&cbuf);
    masm->call(RuntimeAddress(CAST_FROM_FN_PTR(address, os::breakpoint)));
  }
}
const MachOper* loadBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadCNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadL_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLX_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLX_reg_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadRangeNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadXDNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadXD_partialNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadXNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA8BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA4SNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA4CNode::memory_operand() const { return _opnds[1]; }
const MachOper* load2IUNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA2FNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchr0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrNTANode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrT0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrT2Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchw0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwNTANode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwT0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwT2Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeCNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeL_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLX_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLX_reg_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmI16Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmPNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA8BNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA4CNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA2INode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmCMNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeD_roundedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeXDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeXNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA2FNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeF_roundedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeF_DroundedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeF_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeX_immNode::memory_operand() const { return _opnds[1]; }
const TypePtr *membar_acquireNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_acquireNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_acquire_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_acquire_lockNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_releaseNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_releaseNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_release_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_release_lockNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* cmovI_memNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovI_memUNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovI_memUCFNode::memory_operand() const { return _opnds[4]; }
const MachOper* addI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* incI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* decI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLX_LockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLX_reg_LockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeIConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* subI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulINode::memory_operand() const { return _opnds[2]; }
const MachOper* mulI_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* orI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* addL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* subL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orl_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orl_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorl_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorl_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* cmpXD_ccmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpXD_ccmemCFNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpXD_regmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* subD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addD_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addD_mem_regNode::memory_operand() const { return _opnds[1]; }
const MachOper* addD_mem_reg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addXD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addXD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* subXD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulXD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulXD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* divXD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulD_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulD_reg_mem_ciscNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulD_reg_mem_cisc_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* cmpX_ccmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpX_ccmemCFNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpX_regmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* addX_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addX_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* subX_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulX_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulX_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* divX_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* sqrtX_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* sqrtXD_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* addF24_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF24_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addF24_mem_regNode::memory_operand() const { return _opnds[1]; }
const TypePtr *addF24_mem_ciscNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* addF24_mem_ciscNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *addF24_mem_cisc_0Node::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* addF24_mem_cisc_0Node::memory_operand() const { return (MachOper*)-1; }
const TypePtr *addF24_mem_memNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* addF24_mem_memNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* mulF24_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF24_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const TypePtr *mulF24_mem_memNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* mulF24_mem_memNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* mulF_reg_load1Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulF_reg_load1_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* addF_mulF_reg_load1Node::memory_operand() const { return _opnds[1]; }
const MachOper* addF_mulF_reg_load1_1Node::memory_operand() const { return _opnds[2]; }
const MachOper* addF_mulF_reg_load1_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* addF_mulF_reg_load1_2Node::memory_operand() const { return _opnds[3]; }
const MachOper* convI2XD_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2D_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2F_SSF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2F_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* compI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testI_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compU_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_mem_eRegNode::memory_operand() const { return _opnds[2]; }
const MachOper* testP_Reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* cmovLL_mem_LTGENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_LTGENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovLL_mem_EQNENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_EQNENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovLL_mem_LEGTNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_LEGTNode::memory_operand() const { return _opnds[4]; }


const bool Matcher::has_match_rule(int opcode) {
  assert(_last_machine_leaf < opcode && opcode < _last_opcode, "opcode in range");
  return _hasMatchRule[opcode];
}

const bool Matcher::_hasMatchRule[_last_opcode] = {
    false,  // Node
    false,  // Set
    false,  // RegN
    false,  // RegI
    false,  // RegP
    false,  // RegF
    false,  // RegD
    false,  // RegL
    false,  // RegFlags
    false,  // _last_machine_leaf
    true ,  // AbsD
    true ,  // AbsF
    false,  // AbsI
    true ,  // AddD
    true ,  // AddF
    true ,  // AddI
    true ,  // AddL
    true ,  // AddP
    false,  // Allocate
    false,  // AllocateArray
    true ,  // AndI
    true ,  // AndL
    true ,  // AryEq
    true ,  // AtanD
    true ,  // Binary
    true ,  // Bool
    false,  // BoxLock
    true ,  // ReverseBytesI
    true ,  // ReverseBytesL
    false,  // CProj
    true ,  // CallDynamicJava
    false,  // CallJava
    true ,  // CallLeaf
    true ,  // CallLeafNoFP
    true ,  // CallRuntime
    true ,  // CallStaticJava
    true ,  // CastII
    true ,  // CastX2P
    true ,  // CastP2X
    true ,  // CastPP
    false,  // Catch
    false,  // CatchProj
    true ,  // CheckCastPP
    true ,  // ClearArray
    false,  // ConstraintCast
    true ,  // CMoveD
    true ,  // CMoveF
    true ,  // CMoveI
    true ,  // CMoveL
    true ,  // CMoveP
    false,  // CMoveN
    false,  // CmpN
    true ,  // CmpD
    true ,  // CmpD3
    true ,  // CmpF
    true ,  // CmpF3
    true ,  // CmpI
    true ,  // CmpL
    true ,  // CmpL3
    true ,  // CmpLTMask
    true ,  // CmpP
    true ,  // CmpU
    true ,  // CompareAndSwapI
    true ,  // CompareAndSwapL
    true ,  // CompareAndSwapP
    false,  // CompareAndSwapN
    false,  // Con
    false,  // ConN
    true ,  // ConD
    true ,  // ConF
    true ,  // ConI
    true ,  // ConL
    true ,  // ConP
    true ,  // Conv2B
    true ,  // ConvD2F
    true ,  // ConvD2I
    true ,  // ConvD2L
    true ,  // ConvF2D
    true ,  // ConvF2I
    true ,  // ConvF2L
    true ,  // ConvI2D
    true ,  // ConvI2F
    true ,  // ConvI2L
    true ,  // ConvL2D
    true ,  // ConvL2F
    true ,  // ConvL2I
    true ,  // CosD
    false,  // CountedLoop
    true ,  // CountedLoopEnd
    true ,  // CreateEx
    false,  // DecodeN
    true ,  // DivD
    true ,  // DivF
    true ,  // DivI
    true ,  // DivL
    false,  // DivMod
    true ,  // DivModI
    false,  // DivModL
    false,  // EncodeP
    true ,  // ExpD
    true ,  // FastLock
    true ,  // FastUnlock
    true ,  // Goto
    true ,  // Halt
    true ,  // If
    false,  // IfFalse
    false,  // IfTrue
    false,  // Initialize
    false,  // JProj
    true ,  // Jump
    false,  // JumpProj
    true ,  // LShiftI
    true ,  // LShiftL
    true ,  // LoadB
    true ,  // LoadC
    true ,  // LoadD
    false,  // LoadD_unaligned
    true ,  // LoadF
    true ,  // LoadI
    true ,  // LoadKlass
    false,  // LoadNKlass
    true ,  // LoadL
    false,  // LoadL_unaligned
    true ,  // LoadPLocked
    true ,  // LoadLLocked
    true ,  // LoadP
    false,  // LoadN
    true ,  // LoadRange
    true ,  // LoadS
    false,  // Lock
    true ,  // LogD
    true ,  // Log10D
    false,  // Loop
    false,  // Mach
    false,  // MachProj
    true ,  // MaxI
    true ,  // MemBarAcquire
    false,  // MemBarCPUOrder
    true ,  // MemBarRelease
    true ,  // MemBarVolatile
    false,  // MergeMem
    true ,  // MinI
    true ,  // ModD
    true ,  // ModF
    true ,  // ModI
    true ,  // ModL
    true ,  // MoveI2F
    true ,  // MoveF2I
    true ,  // MoveL2D
    true ,  // MoveD2L
    true ,  // MulD
    true ,  // MulF
    false,  // MulHiL
    true ,  // MulI
    true ,  // MulL
    false,  // Multi
    true ,  // NegD
    true ,  // NegF
    false,  // NeverBranch
    false,  // Opaque1
    false,  // Opaque2
    true ,  // OrI
    true ,  // OrL
    false,  // PCTable
    false,  // Parm
    true ,  // PartialSubtypeCheck
    false,  // Phi
    true ,  // PowD
    true ,  // PrefetchRead
    true ,  // PrefetchWrite
    false,  // Proj
    true ,  // RShiftI
    true ,  // RShiftL
    false,  // Region
    true ,  // Rethrow
    true ,  // Return
    false,  // Root
    true ,  // RoundDouble
    true ,  // RoundFloat
    true ,  // SafePoint
    false,  // SafePointScalarObject
    false,  // SCMemProj
    true ,  // SinD
    true ,  // SqrtD
    false,  // Start
    false,  // StartOSR
    true ,  // StoreB
    true ,  // StoreC
    true ,  // StoreCM
    true ,  // StorePConditional
    true ,  // StoreIConditional
    true ,  // StoreLConditional
    true ,  // StoreD
    true ,  // StoreF
    true ,  // StoreI
    true ,  // StoreL
    true ,  // StoreP
    false,  // StoreN
    true ,  // StrComp
    true ,  // SubD
    true ,  // SubF
    true ,  // SubI
    true ,  // SubL
    true ,  // TailCall
    true ,  // TailJump
    true ,  // TanD
    true ,  // ThreadLocal
    false,  // Unlock
    true ,  // URShiftI
    true ,  // URShiftL
    true ,  // XorI
    true ,  // XorL
    false,  // Vector
    false,  // AddVB
    false,  // AddVC
    false,  // AddVS
    false,  // AddVI
    false,  // AddVL
    false,  // AddVF
    false,  // AddVD
    false,  // SubVB
    false,  // SubVC
    false,  // SubVS
    false,  // SubVI
    false,  // SubVL
    false,  // SubVF
    false,  // SubVD
    false,  // MulVF
    false,  // MulVD
    false,  // DivVF
    false,  // DivVD
    false,  // LShiftVB
    false,  // LShiftVC
    false,  // LShiftVS
    false,  // LShiftVI
    false,  // URShiftVB
    false,  // URShiftVC
    false,  // URShiftVS
    false,  // URShiftVI
    false,  // AndV
    false,  // OrV
    false,  // XorV
    false,  // VectorLoad
    false,  // Load16B
    true ,  // Load8B
    false,  // Load4B
    false,  // Load8C
    true ,  // Load4C
    false,  // Load2C
    false,  // Load8S
    true ,  // Load4S
    false,  // Load2S
    false,  // Load4I
    true ,  // Load2I
    false,  // Load2L
    false,  // Load4F
    true ,  // Load2F
    false,  // Load2D
    false,  // VectorStore
    false,  // Store16B
    true ,  // Store8B
    false,  // Store4B
    false,  // Store8C
    true ,  // Store4C
    false,  // Store2C
    false,  // Store4I
    true ,  // Store2I
    false,  // Store2L
    false,  // Store4F
    true ,  // Store2F
    false,  // Store2D
    false,  // Pack
    false,  // PackB
    false,  // PackS
    false,  // PackC
    false,  // PackI
    false,  // PackL
    false,  // PackF
    false,  // PackD
    false,  // Pack2x1B
    false,  // Pack2x2B
    false,  // Replicate16B
    true ,  // Replicate8B
    false,  // Replicate4B
    false,  // Replicate8S
    true ,  // Replicate4S
    false,  // Replicate2S
    false,  // Replicate8C
    true ,  // Replicate4C
    false,  // Replicate2C
    false,  // Replicate4I
    true ,  // Replicate2I
    false,  // Replicate2L
    false,  // Replicate4F
    true ,  // Replicate2F
    false,  // Replicate2D
    false,  // Extract
    false,  // ExtractB
    false,  // ExtractS
    false,  // ExtractC
    false,  // ExtractI
    false,  // ExtractL
    false,  // ExtractF
    false   // ExtractD
};


bool Matcher::stack_direction() const { return false; }

int Compile::sync_stack_slots() const { return 1; }

uint Matcher::stack_alignment_in_bytes() { return StackAlignmentInBytes; }

OptoReg::Name Matcher::return_addr() const { return OptoReg::stack2reg(- 1 +
              round_to(1+VerifyStackAtCalls+
              Compile::current()->fixed_slots(),
              (StackAlignmentInBytes/wordSize))); }

uint Compile::in_preserve_stack_slots() { return 2+VerifyStackAtCalls; }

uint Compile::out_preserve_stack_slots() { return SharedRuntime::out_preserve_stack_slots(); }

uint Compile::varargs_C_out_slots_killed() const { return 0; }

void Matcher::calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length, bool is_outgoing) {

    // No difference between ingoing/outgoing just pass false
    SharedRuntime::java_calling_convention(sig_bt, regs, length, false);
  
}

void Matcher::c_calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length) {

    // This is obviously always outgoing
    (void) SharedRuntime::c_calling_convention(sig_bt, regs, length);
  
}

OptoRegPair Matcher::return_value(int ideal_reg, bool is_outgoing) {

    assert( ideal_reg >= Op_RegI && ideal_reg <= Op_RegL, "only return normal values" );
    static int lo[Op_RegL+1] = { 0, 0, OptoReg::Bad, EAX_num,      EAX_num,      FPR1L_num,    FPR1L_num, EAX_num };
    static int hi[Op_RegL+1] = { 0, 0, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, FPR1H_num, EDX_num };
    if( ideal_reg == Op_RegD && UseSSE>=2 )
      return OptoRegPair(XMM0b_num,XMM0a_num);
    if( ideal_reg == Op_RegF && UseSSE>=1 )
      return OptoRegPair(OptoReg::Bad,XMM0a_num);
    return OptoRegPair(hi[ideal_reg],lo[ideal_reg]);
  
}

OptoRegPair Matcher::c_return_value(int ideal_reg, bool is_outgoing) {

    assert( ideal_reg >= Op_RegI && ideal_reg <= Op_RegL, "only return normal values" );
    static int lo[Op_RegL+1] = { 0, 0, OptoReg::Bad, EAX_num,      EAX_num,      FPR1L_num,    FPR1L_num, EAX_num };
    static int hi[Op_RegL+1] = { 0, 0, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, FPR1H_num, EDX_num };

    // in SSE2+ mode we want to keep the FPU stack clean so pretend
    // that C functions return float and double results in XMM0.
    if( ideal_reg == Op_RegD && UseSSE>=2 )
      return OptoRegPair(XMM0b_num,XMM0a_num);
    if( ideal_reg == Op_RegF && UseSSE>=2 )
      return OptoRegPair(OptoReg::Bad,XMM0a_num);

    return OptoRegPair(hi[ideal_reg],lo[ideal_reg]);
  
}

OptoReg::Name Matcher::inline_cache_reg() { return OptoReg::Name(EAX_num); }

const RegMask &Matcher::inline_cache_reg_mask() { return INLINE_CACHE_REG_mask; }

int Matcher::inline_cache_reg_encode() { return _regEncode[inline_cache_reg()]; }

OptoReg::Name Matcher::interpreter_method_oop_reg() { return OptoReg::Name(EBX_num); }

const RegMask &Matcher::interpreter_method_oop_reg_mask() { return INTERPRETER_METHOD_OOP_REG_mask; }

int Matcher::interpreter_method_oop_reg_encode() { return _regEncode[interpreter_method_oop_reg()]; }

OptoReg::Name Matcher::interpreter_frame_pointer_reg() { return OptoReg::Name(EBP_num); }

const RegMask &Matcher::interpreter_frame_pointer_reg_mask() { return INTERPRETER_FRAME_POINTER_REG_mask; }

OptoReg::Name Matcher::c_frame_pointer() const { return OptoReg::Name(ESP_num); }

// Number of callee-save + always-save registers
int  Matcher::number_of_saved_registers() {
  return 0;
};

// Check consistency of C++ compilation with ADLC options:
// Check adlc -DLINUX=1
#ifndef LINUX
#  error "LINUX must be defined"
#endif // LINUX
// Check adlc -D_GNU_SOURCE=1
#ifndef _GNU_SOURCE
#  error "_GNU_SOURCE must be defined"
#endif // _GNU_SOURCE
// Check adlc -DIA32=1
#ifndef IA32
#  error "IA32 must be defined"
#endif // IA32
// Check adlc -U_LP64
#ifdef _LP64
#  error "_LP64 must not be defined"
#endif // _LP64
